package com.openexchange.datagenerator

import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import java.nio.file.{Files, Path}
import java.nio.file.StandardOpenOption._
import java.security.MessageDigest

import scala.collection.mutable.{Map => MutableMap}
import scala.util.Random

class FileWriter(blockSize: Int) {
  
  class Metadata(val size: Long, val md5: Array[Byte])
  
  type ArrayFiller = Array[Byte] => Unit
  
  def channel(path: Path): FileChannel = {
    Files.createDirectories(path.getParent)
    FileChannel.open(path, WRITE, CREATE, TRUNCATE_EXISTING)
  }
  
  def writeArrays(path: Path, size: Long, data: ArrayFiller): Metadata = {
    val digest = MessageDigest.getInstance("MD5")
    val fc = channel(path)
    try {
      val buf = ByteBuffer.allocate(blockSize)
      if (size >= blockSize) {
        val arr = new Array[Byte](blockSize)
        for (_ <- 0 until (size / blockSize).toInt) {
          buf.clear()
          data(arr)
          digest.update(arr)
          buf.put(arr).flip()
          fc.write(buf)
        }
      }
      val lastArr = new Array[Byte]((size % blockSize).toInt)
      buf.clear()
      data(lastArr)
      digest.update(lastArr)
      buf.put(lastArr).flip()
      fc.write(buf)
      new Metadata(size, digest.digest())
    } finally {
      fc.close()
    }
  }
  
  def random(path: Path, size: Long, rnd: Random): Metadata =
    writeArrays(path, size, a => rnd.nextBytes(a))
  
  private val cache: MutableMap[Path, Metadata] = MutableMap()
  
  def copy(source: Path, destination: Path): Metadata = {
    val src = FileChannel.open(source, READ)
    try {
      val dest = channel(destination)
      try {
        val meta = if (cache contains source) {
          cache(source)
        } else {
          val digest = MessageDigest.getInstance("MD5")
          val buf = ByteBuffer.allocate(blockSize)
          while (src.read(buf) >= 0) {
            buf.flip()
            digest.update(buf)
            buf.clear()
          }
          val meta = new Metadata(src.size(), digest.digest())
          cache(source) = meta
          meta
        }
        src.transferTo(0, meta.size, dest)
        meta
      } finally {
        dest.close()
      }
    } finally {
      src.close()
    }
  }
}
