package com.openexchange.datagenerator

import java.time.Instant

import Values._

import scala.annotation.tailrec
import scala.collection.Searching.search
import scala.util.Random

class Functions(val now: Instant, seed: Long) {
  val rnd = new Random(seed)
  type FuncType = (List[Val[_]]) => Val[_]
  def apply(name: String): FuncType = map(name)
  val map: Map[String, FuncType] = Map(
      
    "gauss" -> {
      case List(NumVal(average), NumVal(sigma)) =>
        NumVal(rnd.nextGaussian() * sigma() + average())
      case _ => throw new Exception("Expected gauss(average: num, sigma: num)")
    },

    "histogram" -> (list => {
      val first = list(1)
      val pairs = list.grouped(2).map({
        case List(NumVal(count), value) => (count(), value)
        case x => throw new Exception(s"Invalid histogram at $x. Expected histogram(num, val, [num, val, ...])")
      }).toIndexedSeq.sortBy(-_._1) // sort smallest first to preserve precision in cdf

      val values = pairs.map(_._2)
      val total = pairs.foldLeft(0.0) {
        case (sum, (count, _)) => sum + count
      }

      var sum = 0.0
      val cdf: IndexedSeq[Double] = pairs.map({
        case (count, _) => {
          sum = sum + count
          sum / total
        }
      })

      first.selector(values, () => cdf.search(rnd.nextDouble()).insertionPoint)
    }),

    "probability" -> {
      case List(NumVal(p)) => BoolVal(rnd.nextDouble() < p())
      case _ => throw new Exception("Expected probability(p: num)")
    },

    "truncate" -> {
      case List(NumVal(min), NumVal(max), NumVal(value)) =>
        NumVal(truncate(min(), max(), value))
      case _ => throw new Exception("Expected truncate(min: num, max: num, value: num)")
    },

    "uniform" -> {
      case List(NumVal(min), NumVal(range)) =>
        NumVal(rnd.nextDouble() * range() + min())
      case _ => throw new Exception("Expected uniform(min: num, range: num)")
    },

    "list" -> (list => {
      val first = list(0)
      val evaluatedList: List[_] = list map (v => first.cast(v).value())
      new Val[List[_]](() => evaluatedList)
    })
  )
  
  @tailrec
  private def truncate(min: Double, max: Double, value: () => Double): Double = {
    val v = value()
    if (v >= min && v < max) v else truncate(min, max, value)
  }
}
