package com.openexchange.datagenerator

import java.time._

import scala.annotation.tailrec
import scala.util.Random

/**
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 */
abstract class Generator(config: Configuration) {
  val rnd: Random = config.functions.rnd

  val timestamp: Instant = config.functions.now
  val now: LocalDateTime = LocalDateTime.ofInstant(timestamp, ZoneOffset.UTC)
  val today: LocalDate = LocalDate.from(now)
  
  def uuid(): String = new java.util.UUID(
      rnd.nextLong() & 0xffffffffffff0fffL | 0x0000000000004000L,
      rnd.nextLong() & 0x3fffffffffffffffL | 0x8000000000000000L).toString
  
  def gauss(avg: Double, sigma: Double): Double = rnd.nextGaussian() * sigma + avg
  def gauss(avg: Long, sigma: Double): Long = gauss(avg.toDouble, sigma).toLong
  
  def probability(p: Double): Boolean = rnd.nextDouble() < p
  
  @tailrec
  final def truncate[T](min: T, max: T, value: => T)(implicit t: T => Ordered[T]): T = {
    val v = value
    if (v >= min && v <= max) v else truncate(min, max, value)
  }
  
  def uniform(min: Double, range: Double): Double = rnd.nextDouble() * range + min
  def uniform(min: Int, range: Int): Int = rnd.nextInt(range) + min
  
  def str(maxLen: Int): String = {
    val n = truncate(0, 4, rnd.nextGaussian()) * maxLen / 4
    rnd.alphanumeric.take(n.toInt).mkString
  }
  
  def name(): String = Markov.names.chain(rnd)
  def text(): String = Markov.text.chain(rnd)
  
  def generate()
  
}
