package com.openexchange.datagenerator

import scala.collection.mutable
import scala.io.Source
import scala.util.Random

class Markov(n: Int, text: TraversableOnce[String]) {
  private val mask = (1L << (16 * n)) - 1
  private val dict = mutable.Map.empty[Long, String]
  for (word <- text) {
    val w = word + '\u0000'
    for (ngram <- w.take(n).inits; if !ngram.isEmpty) add(ngram)
    if (w.length > n) for (ngram <- w.sliding(n + 1)) add(ngram)
    
    def add(ngram: String) {
      val key = ngram.init.foldLeft(0L)((k, c) => k << 16 & mask | c.toLong)
      dict(key) = dict.getOrElse(key, "") + ngram.last
    }
  }
  val start = dict(0L)
  
  def chain(rnd: Random): String = {
    val sb = new StringBuilder
    var state = 0L
    var c: Char = start(rnd.nextInt(start.length))
    while (c != '\u0000') {
      sb.append(c)
      state = state << 16 & mask | c.toLong
      val allNext = dict(state)
      c = allNext(rnd.nextInt(allNext.length))
    }
    sb.result()
  }
}

object Markov {
  private def fromResource(name: String): Source = {
    Source.fromInputStream(getClass getResourceAsStream name, "UTF-8")
  }
  val names = new Markov(1, fromResource("female.txt").getLines() ++
                            fromResource("male.txt").getLines())
  val text = new Markov(2, fromResource("text.txt").getLines().map(_.trim))
}
