package com.openexchange.datagenerator

object Values {

  class Val[@specialized T](val value: () => T) {
    def cast(x: Val[_]): Val[T] = {
      x match {
        case x: Val[T] => x
        case _ => throw new Exception("Type error")
      }
    }

    def selector(list: TraversableOnce[Val[_]], index: () => Int): Val[T] = {
      val items = (list map cast).toIndexedSeq
      new Val[T](() => {
        val i = index()
        items(i).value()
      })
    }
  }

  class Helper[T] {
    def apply(v: => T): Val[T] = new Val[T](() => v)
    def unapply(v: Val[_]): Option[() => T] = v match {
      case v: Val[T] => Some(v.value)
      case _ => None
    }
  }

  type NumVal = Val[Double]
  type StrVal = Val[String]
  type BoolVal = Val[Boolean]
  type ListVal[T] = Val[List[T]]

  val NumVal = new Helper[Double]
  val StrVal = new Helper[String]
  val BoolVal = new Helper[Boolean]
  val NumListVal = new Helper[List[Double]]
  val StringListVal = new Helper[List[String]]
  val BoolListVal = new Helper[List[Boolean]]

  val trueVal = BoolVal(true)
  val falseVal = BoolVal(false)
}
