package com.openexchange.datagenerator.mail

import com.openexchange.datagenerator.Configuration
import com.openexchange.datagenerator.Generator

import scala.util.parsing.json._
import scala.reflect.ClassTag
import scala.io.Source
import Multipart.Writable

abstract class MailGenerator(config: Configuration, opt: Main.Options)
  extends Generator(config)
{
  implicit class Cast(x: Any) {
    def as[T: ClassTag]: Option[T] = Some(x) collect { case m: T => m }
  }
  
  def user(displayName: String, email: String): TraversableOnce[Writable]
  
  def parseJSON(json: String): Either[String, Map[String, Any]] = {
    JSON.parseFull(json).as[Map[String, Any]]
      .toRight("Error parsing JSON:\n${json}")
  }
  
  val html2json = """\((\{.*\})\)""".r
  
  override def generate () {
    val api = new API(opt)
    Source.fromFile(opt.input).getLines().foreach(line => {
      val userData = line.split(',')
      val oxLogin = userData(2)
      val email = userData(3)
      val reply = api.login(oxLogin, opt.password)
      val session = try {
        val loginJSON = JSON.parseFull(reply).get
        val loginObj = loginJSON.asInstanceOf[Map[String, Any]]
        loginObj("session").asInstanceOf[String]
      } catch {
        case e: Throwable => {
          print(s"Unable to log in. Received reply:\n$reply")
          throw e
        }
      }
      try {
        val userJSON = JSON.parseFull(api.getMyUser(session)).get
        val userObj = userJSON.asInstanceOf[Map[String, Any]]
        val data = userObj("data").asInstanceOf[Map[String, Any]]
        val displayName = data("display_name").asInstanceOf[String]
        val reply = api.importMails(user(displayName, email), "INBOX", session)
        val htmlMatch = html2json.findFirstMatchIn(reply)
        val replyPayload =
          if (htmlMatch.isDefined) htmlMatch.get.group(1) else reply
        val replyJSON = JSON.parseFull(replyPayload).get
        val replyObj = replyJSON.asInstanceOf[Map[String, Any]]
        if (replyObj.contains("error")) throw new Exception(replyPayload);
      } finally {
        api.logout(session)
      }
    })
  }
}
