package com.openexchange.datagenerator.mail

import java.nio.charset.StandardCharsets.UTF_8
import java.util.UUID

object Multipart {
  type Writable = Iterator[Array[Byte]]
  
  val CRLF = "\r\n".getBytes(UTF_8)
  
  def part(headers: Map[String, String], content: Writable): Writable = {
    headers.iterator.map {
      case (k, v) => s"${k}: ${v}\r\n".getBytes(UTF_8)
    } ++ Iterator(CRLF) ++ content
  }
  
  def multipart(parts: Iterator[Writable],
                boundary: String = UUID.randomUUID.toString): Writable = {
    val prefix = s"\r\n--${boundary}\r\n".getBytes(UTF_8)
    val suffix = s"\r\n--${boundary}--\r\n".getBytes(UTF_8)
    parts.flatMap(Iterator(prefix) ++ _) ++ Iterator(suffix)
  }

}
