package com.openexchange.datagenerator.sql

import java.time.temporal.ChronoField.DAY_OF_WEEK
import java.time.temporal.ChronoUnit.DAYS
import java.time.temporal.Temporal
import java.time.temporal.TemporalAdjuster
import java.time.temporal.TemporalAdjusters

object Week {
  
  val Count = Array.ofDim[Int](128)
  private val Backwards = Array.ofDim[Int](128)
  Count(0) = 0
  Backwards(0) = 0
  private var i = 1
  for (lastDay <- 0 to 6; previous <- 0 until 1 << lastDay) {
    Count(i) = Count(previous) + 1
    Backwards(i) = Backwards(previous) | 1 << (6 - lastDay)
    i += 1
  }
  
  val firstDayOfNextMonth = TemporalAdjusters.firstDayOfNextMonth()
  
  val InvalidOffset = Byte.MinValue
  
  // The following functions return offsets from the specified day to
  // the requested day, as a signed number of days.
  
  private val SameOrNextDay = Array.ofDim[Byte](128, 7)
  for (start <- 0 to 6) SameOrNextDay(0)(start) = InvalidOffset
  for (lastDay <- 0 to 6; lastBit = 1 << lastDay;
       previous <- 0 until lastBit; days = lastBit | previous;
       startDay <- 0 to 6)
  {
    SameOrNextDay(days)(startDay) = if (startDay == lastDay || previous == 0) { 
      ((lastDay - startDay + 7) % 7).toByte
    } else {
      SameOrNextDay(previous)(startDay)
    }
  }

  def sameOrNextDay(days: Int, start: Int): Int = {
    SameOrNextDay(days)(start % 7)
  }
  
  def nextDay(days: Int, start: Int): Int = {
    SameOrNextDay(days)((start + 1) % 7) + 1
  }
  
  def sameOrPreviousDay(days: Int, start: Int): Int = {
    -SameOrNextDay(Backwards(days))((13 - start) % 7)
  }
  
  def previousDay(days: Int, start: Int): Int = {
    val bd = Backwards(days)
    -SameOrNextDay(bd)((12 - start) % 7) - 1
  }
  
  def adjuster(f: (Int, Int) => Int, days: Int): TemporalAdjuster = {
    new TemporalAdjuster {
      def adjustInto(input: Temporal): Temporal = {
        input.plus(f(days, input.get(DAY_OF_WEEK)), DAYS)
      }
    }
  }
  
}
