/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.calendar

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.utils.AppointmentGenerator
import com.openexchange.testing.performance.groupware.{ Appointment, Participant }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder

import scala.util.Random

/**
 * [[CalendarChain]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object CalendarChain extends LazyLogging {
  private val random = ThreadLocalRandom.current()
  private val appointmentClassName = classOf[Appointment].getSimpleName

  def clear(): ChainBuilder = exec(session => {
    session.removeAll(appointmentClassName, "ALL_GAB_USER")
  })

  def createAppointment(folderId: Expression[String], rangeDays: Int = 0, allowRecurring: Boolean = true): ChainBuilder = {
    doIfOrElse(session => session.contains(CAPABILITIES) &&
      session(CAPABILITIES).validate[Seq[String]].get.contains("gab") &&
      session.contains("ALL_GAB_USER")) {
      exec(session => for {
        fid <- folderId(session)
        sizeOfGAB <- ThreadLocalRandom.current().nextInt(session("ALL_GAB_USER").validate[Seq[Int]].get.length)
      } yield {
        val participantsList = (0 to sizeOfGAB).map(i => Participant(Random.shuffle(session("ALL_GAB_USER").validate[Seq[Int]].get).apply(i), 1))
        val appointment = AppointmentGenerator.generateRandomAppointment(fid, participantsList, rangeDays, allowRecurring)
        logger.trace("{}", appointment)
        session.set(appointmentClassName, appointment)
      })
    } {
      exec(session => for {
        fid <- folderId(session)
        participantsList <- Participant(session(USER_ID).validate[String].get.toInt, 1)
      } yield {
        val appointment = AppointmentGenerator.generateRandomAppointment(fid, Seq(participantsList), rangeDays, allowRecurring)
        logger.trace("{}", appointment)
        session.set(appointmentClassName, appointment)
      })
    }
      .doIf(session => session.contains(appointmentClassName)) {
        exec(CalendarAPI.createAppointment("${" + appointmentClassName + "}"))
      }

  }

}
