/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.config

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.SessionConstants._
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[ConfigAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object ConfigAPI {

  /** Gets the identifier of the folder that gets all incoming mails. */
  val getDefaultFolderInbox: ChainBuilder = getDefaultFolder("modules/mail/defaultFolder/inbox", FID_INBOX)

  /** Gets the identifier of the folder with the mail drafts. */
  val getDefaultFolderDrafts: ChainBuilder = getDefaultFolder("modules/mail/defaultFolder/drafts", FID_DRAFTS)

  /** Gets the identifier of the folder with the sent mails */
  val getDefaultFolderSent: ChainBuilder = getDefaultFolder("modules/mail/defaultFolder/sent", FID_SENT)

  /** Gets the identifier of the folder with the spam mails. */
  val getDefaultFolderSpam: ChainBuilder = getDefaultFolder("modules/mail/defaultFolder/spam", FID_SPAM)

  /** Gets the identifier of the folder with the deleted mails. */
  val getDefaultFolderTrash: ChainBuilder = getDefaultFolder("modules/mail/defaultFolder/trash", FID_TRASH)

  /** Gets the identifier of the standard task folder. */
  val getDefaultFolderTasks: ChainBuilder = getDefaultFolder("folder/tasks", FID_TASKS)

  /** Gets the identifier of the standard calendar folder. */
  val getDefaultFolderCalendar: ChainBuilder = getDefaultFolder("folder/calendar", FID_CALENDAR)

  /** Gets the identifier of the standard contacts folder. */
  val getDefaultFolderContacts: ChainBuilder = getDefaultFolder("folder/contacts", FID_CONTACTS)

  /** Gets the identifier of the private infostore folder. */
  val getDefaultFolderInfostore: ChainBuilder = getDefaultFolder("folder/infostore", FID_INFOSTORE)

  /** Gets the identifier of the default infostore trash folder (read-only, since 7.6.0) */
  val getDefaultFolderInfostoreTrash: ChainBuilder = getDefaultFolder("modules/infostore/folder/trash", FID_INFOSTORE_TRASH)

  /** Gets the version string of the server. */
  val getServerVersion: ChainBuilder = getDefaultFolder("serverVersion", SERVER_VERSION)

  /** Gets the unique identifier of the user. */
  val getUserID: ChainBuilder = getDefaultFolder("identifier", USER_ID)

  /** Gets the UI settings */
  val getUISettings: ChainBuilder = group(ModuleNames.CONFIG) {
    exec(new GetConfig("ui"))
  }

  /** Gets the GUI settings */
  val getGUISettings: ChainBuilder = group(ModuleNames.CONFIG) {
    exec(new GetConfig("gui"))
  }

  val getAllDefaultMailFolderIds: ChainBuilder = group(ModuleNames.CONFIG) {
    exec(new GetConfig("").buildHttpRequest().silent
      .check(jsonPath("$.data.mail.folder.inbox").saveAs(FID_INBOX))
      .check(jsonPath("$.data.mail.folder.sent").saveAs(FID_SENT))
      .check(jsonPath("$.data.mail.folder.spam").saveAs(FID_SPAM))
      .check(jsonPath("$.data.mail.folder.drafts").saveAs(FID_DRAFTS))
      .check(jsonPath("$.data.mail.folder.trash").saveAs(FID_TRASH)))
  }

  val getAllDefaultFolderIds: ChainBuilder = group(ModuleNames.CONFIG) {
    exec(new GetConfig("").buildHttpRequest().silent
      .check(jsonPath("$.data.mail.folder.inbox").saveAs(FID_INBOX))
      .check(jsonPath("$.data.mail.folder.sent").saveAs(FID_SENT))
      .check(jsonPath("$.data.mail.folder.spam").saveAs(FID_SPAM))
      .check(jsonPath("$.data.mail.folder.drafts").saveAs(FID_DRAFTS))
      .check(jsonPath("$.data.mail.folder.trash").saveAs(FID_TRASH))
      .check(jsonPath("$.data.folder.infostore").saveAs(FID_INFOSTORE))
      .check(jsonPath("$.data.folder.tasks").saveAs(FID_TASKS))
      .check(jsonPath("$.data.folder.calendar").saveAs(FID_CALENDAR))
      .check(jsonPath("$.data.folder.contacts").saveAs(FID_CONTACTS))
      .check(jsonPath("$.data.modules.infostore.folder.trash").saveAs(FID_INFOSTORE_TRASH)))
  }

  /**
   * Gets data of a configuration node.
   *
   * @return
   */
  def getConfigNode = ???

  /**
   * Gets a property of the configuration.
   *
   * @return
   */
  def getConfigProperty = ???

  /**
   * Sets the value of a configuration node.
   *
   * @return
   */
  def putConfigNode = ???

  /**
   * Sets a property of the configuration.
   *
   * @return
   */
  def setConfigProperty = ???

  /**
   * Gets the default folder identifier for a specific config path.
   *
   * @param path the config path.
   */
  private def getDefaultFolder(path: String, storeAs: String): ChainBuilder = group(ModuleNames.CONFIG) {
    exec(new GetConfig(path).buildHttpRequest()
      .check(jsonPath("$.data").saveAs(storeAs)))
  }

}
