/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.contacts

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.contacts.ContactData
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[UpdateContact]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 * @constructor
 * @param id        Object ID of the updated contact.
 * @param folder    Folder identifier through that the contact is accessed.
 * @param timestamp Timestamp of the updated contact.
 * @param contact   Contact object.
 */
class UpdateContact(id: Expression[Int], folder: Expression[String], timestamp: Expression[Long], contact: Expression[ContactData]) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "contacts?action=update"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.FOLDER, folder)
      .queryParam(RequestParameters.ID, id)
      .queryParam(RequestParameters.TIMESTAMP, timestamp)
      .body(StringBody(session => JSONUtil().toJson(contact(session).get))).asJSON
  }
}
