/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.file

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[UploadFile]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param module    The module for which the file is uploaded to determine proper upload quota
 *                  constraints (e.g. "mail", "infostore", etc.)
 * @param mediaType The file type filter to define which file types are allowed during upload
 */
class UploadFile(module: String, file: String, mediaType: String, fileName: String) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = "file?action=new"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.MODULE, module)
      .queryParam("type", mediaType)
      .bodyPart(RawFileBodyPart("file", file).contentType(mediaType).fileName(fileName))

  }
}
