/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.find

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.core.body.Body
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[AutoComplete]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param module             The module name e.g. ''mail''.
 * @param optQueryParams Optional URL parameters.
 */
class AutoComplete(module: String, body: Body, optQueryParams: Option[Map[String, Any]] = None) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "find?action=autocomplete"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam("module", module)
      .queryParamMap(optQueryParams.getOrElse(Map.empty[String, Any])
        .filter({ case (key, value) => value != None }))
      .body(body).asJSON
  }
}
