/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.find

import java.net.URLEncoder

import io.gatling.core.Predef._
import io.gatling.core.body.StringBody
import io.gatling.core.structure.ChainBuilder

import scala.concurrent.duration._

/**
 * [[FindAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object FindAPI {
  val groupName = "Find"

  def autocomplete(module: String, body: StringBody, limit: Option[Int] = None): ChainBuilder = group(groupName) {
    exec(new AutoComplete(module, body, Option(Map("limit" -> limit.getOrElse(None)))))
  }

  def autocompleteSearch(query: String, module: String, limit: Option[Int] = None, paceTime: Duration = 1 second,
                         searchThreshold: Int = 0): ChainBuilder = {
      def feederData = Range(searchThreshold, query.length + 1)
        .map(i => query.substring(0, i))
        .map(word => Map("query" -> URLEncoder.encode(word, "UTF-8")))
        .toList

    foreach(feederData, "query") {
      exec(flattenMapIntoAttributes("${query}"))
        .exec(autocomplete(
          module,
          StringBody(
            """{"prefix":"${query}","options":{"timezone":"UTC","admin":false},"facets":[{"facet":"folder","value":"${OX_FID_INBOX}","filter":null}]}"""),
          limit))
    }
  }

}