/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.folders

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.folders.FolderBody
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.Predef._
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[CreateFolder]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param folder_id The parent folder of the newly created folder.
 * @param body      The request body
 * @param tree      The identifier of the folder tree. If missing ''0'' (primary folder tree) is assumed.
 */
class CreateFolder(folder_id: Expression[String], body: Expression[FolderBody], tree: Int = 0, optionalQueryParams: Option[Map[String, Any]] = None) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "folders?action=new"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.FOLDER_ID, folder_id)
      .queryParam("tree", tree)
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty))
      .body(StringBody(session => JSONUtil().toJson(body(session).get))).asJSON
      .check(jsonPath("$.data").ofType[String].saveAs(getClass.getSimpleName))
  }
}