/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.folders

import com.openexchange.testing.performance.groupware.folders.{ FolderBody, FolderData, Permission }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder

import scala.util.Random

/**
 * [[FolderChains]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object FolderChains extends LazyLogging {

  def createFolder(folder_id: Expression[String], module: String, permission: Option[Array[Permission]] = None): ChainBuilder = {
    exec(session => for {
      fid <- folder_id(session)
    } yield {
      val folderBody = FolderBody(FolderData(
        title = s"oxgatling-${Random.alphanumeric.take(8).mkString}",
        module = module,
        permissions = permission))
      session.set(classOf[FolderBody].getSimpleName, folderBody)
    })
      .doIf(session => session.contains(classOf[FolderBody].getSimpleName)) {
        exec(FoldersAPI.createFolder(folder_id, "${" + classOf[FolderBody].getSimpleName + "}"))
          .doIf(session => session.contains(classOf[CreateFolder].getSimpleName)) {
            exec(session => {
              logger.trace("Successfully created {} folder", module)
              session
            })
          }

      }
  }

}
