/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.folders

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[GetAllVisibleFolders]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param tree         The identifier of the folder tree.
 * @param content_type The desired content type (either numbers or strings; e.g. "tasks", "calendar", "contacts", "mail")
 * @param columns      A comma-separated list of columns to return.
 */
class GetAllVisibleFolders(content_type: String, columns: String, tree: Int = 0) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "folders?action=allVisible"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.COLUMNS, columns)
      .queryParam(RequestParameters.TREE, tree)
      .queryParam(RequestParameters.CONTENT_TYPE, content_type)
  }
}
