/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.halo

import com.openexchange.testing.performance.common.ModuleNames
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[HaloAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object HaloAPI {
  private val moduleName = ModuleNames.HALO

  /**
   * Gets a contact picture.
   */
  def getHaloContactPicture(
    internal_userid:      Option[String]  = None,
    userid:               Option[String]  = None,
    user_id:              Option[String]  = None,
    user:                 Option[String]  = None,
    context:              Option[String]  = None,
    id:                   Option[String]  = None,
    folder:               Option[String]  = None,
    email:                Option[String]  = None,
    email1:               Option[String]  = None,
    email2:               Option[String]  = None,
    email3:               Option[String]  = None,
    transformationNeeded: Option[Boolean] = None,
    cropWidth:            Option[Int]     = None,
    cropHeight:           Option[Int]     = None,
    rotate:               Option[Boolean] = None,
    compress:             Option[Boolean] = None,
    width:                Option[Int]     = None,
    height:               Option[Int]     = None,
    shrinkOnly:           Option[Boolean] = None,
    scaleType:            Option[String]  = None): ChainBuilder = group(moduleName) {
    exec(new GetHaloContactPicture(Option(Map(
      "internal_userid" -> internal_userid.getOrElse(None), "userid" -> userid.getOrElse(None),
      "user_id" -> user_id.getOrElse(None), "id" -> id.getOrElse(None), "email" -> email.getOrElse(None),
      "email1" -> email1.getOrElse(None), "email2" -> email2.getOrElse(None), "email3" -> email3.getOrElse(None),
      "transformationNeeded" -> transformationNeeded.getOrElse(None), "cropWidth" -> cropWidth.getOrElse(None),
      "cropHeight" -> cropHeight.getOrElse(None), "rotate" -> rotate.getOrElse(None),
      "compress" -> compress.getOrElse(None), "width" -> width.getOrElse(None), "height" -> height.getOrElse(None),
      "shrinkOnly" -> shrinkOnly.getOrElse(None), "scaleType" -> scaleType.getOrElse(None),
      "user" -> user.getOrElse(None), "context" -> context.getOrElse(None), "folder" -> folder.getOrElse(None)))))
  }

  /**
   * Gets all halo services.
   */
  def getHaloServices: ChainBuilder = group(moduleName) {
    exec(new ContactServices().buildHttpRequest().check(jsonPath("$.data[*]")
      .findAll.optional.saveAs(classOf[ContactServices].getSimpleName)))
  }

  /**
   * Investigates a contact.
   */
  def investigateContactHalo(
    provider: Expression[String], columns: String, timezone: Option[String] = None, start: Option[Long] = None,
    end: Option[Long] = None, sort: Option[String] = None, order: Option[String] = None,
    limit: Option[Int] = None): ChainBuilder = group(moduleName) {
    exec(new ContactInvestigate(provider, columns, Option(Map(
      "timezone" -> timezone.getOrElse(None),
      "start" -> start.getOrElse(None),
      "end" -> end.getOrElse(None),
      "sort" -> sort.getOrElse(None),
      "order" -> order.getOrElse(None),
      "limit" -> limit.getOrElse(None)))))
  }

}
