/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.infostore

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.infostore.InfoItemBody
import com.openexchange.testing.performance.groupware.{ DetailedInfoitemData, InfoItem }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[InfostoreAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object InfostoreAPI extends LazyLogging {

  /**
   * Checks if a given file name is valid.
   *
   * @return
   */
  def checkName = ???

  /**
   * Copies an infoitem.
   *
   * @return
   */
  def copyInfoItem = ???

  /**
   * Copies an infoitem.
   *
   * @return
   */
  def copyInfoItemAdvanced = ???

  /**
   * Creates an infoitem.
   */
  def createInfoItem = ???

  /**
   * Creates an infoitem.
   *
   * @return
   */
  def createInfoItemAdvanced(folderId: Expression[String], queryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new CreateInfoItem("${" + classOf[InfoItemBody].getSimpleName + "}").buildHttpRequest()
      .queryParamMap(queryParams.getOrElse(Map.empty))
      .check(jsonPath("$.data").optional.saveAs(classOf[CreateInfoItem].getSimpleName)))
  }

  /**
   * Deletes all versions of an infoitem leaving only the base object.
   *
   * @return
   */
  def deleteAllInfoItemVersions = ???

  /**
   * Deletes versions of an infoitem.
   *
   * @return
   */
  def deleteInfoItemVersions = ???

  /**
   * Deletes infoitems.
   *
   * @param timestamp
   * @param requestBody
   * @param optionalQueryParams
   * @return
   */
  def deleteInfoItems(timestamp: Long, requestBody: String, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new DeleteInfoItems(timestamp, requestBody).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty)))
  }

  /**
   * Gets all infoitems.
   *
   * @param folder
   * @param columns
   * @param optionalQueryParams
   * @return
   */
  def getAllInfoItems(folder: String, columns: String, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new GetAllInfoItems(folder, columns).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty))
      .check(jsonPath("$").transform(x => JSONUtil().fromJson[GetAllInfoItemsResponse](x)).saveAs("getAllInfoitemsResponse")))
  }

  /**
   * Gets all versions of an infoitem.
   *
   * @return
   */
  def getAllVersions = ???

  /**
   * Gets multiple documents as a ZIP archive.
   *
   * @return
   */
  def getDocumentsAsZip = ???

  /**
   * Gets a ZIP archive containing all ifoitems of a denoted folder.
   *
   * @return
   */
  def getFolderAsZip = ???

  /**
   * Gets an infoitem.
   *
   * @param id
   * @return
   */
  def getInfoItem(id: Expression[String], folder: Expression[String]): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new GetInfoItem(id, folder).buildHttpRequest()
      .check(jsonPath("$.data").transform(x => JSONUtil().fromJson[DetailedInfoitemData](x)).saveAs("DetailedInfoitemData")))
      .exec(session => {
        val infoData = session("DetailedInfoitemData").as[DetailedInfoitemData]
        session.set("getInfoitem", infoData).remove("DetailedInfoitemData")
      })
  }

  def getInfoItemVersions(id: Expression[String], columns: String) = group(ModuleNames.INFOSTORE) {
    exec(new GetInfoItemVersions(id, columns))
  }

  /**
   * Gets an infoitem document.
   *
   * @param id
   * @param folder
   * @param optionalQueryParams
   * @return
   */
  def getInfoItemDocument(id: String, folder: String, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new GetInfoItemDocument(id, folder).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty)))
  }

  def downloadInfoItemDocument(id: String, folder: String, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new DownloadInfoItemDocument(id, folder).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty)))
  }

  /**
   * Gets a list of infoitems.
   *
   * @return
   */
  def getInfoItemList = ???

  /**
   * Gets the new, modified and deleted infoitems.
   *
   * @return
   */
  def getInfoItemUpdates = ???

  /**
   * Gets shared infoitems.
   *
   * @return
   */
  def getSharedInfoItems = ???

  /**
   * Saves an attachment in the infostore.
   *
   * @return
   */
  def importAttachment = ???

  /**
   * Locks an infoitem.
   *
   * @return
   */
  def lockInfoItem = ???

  /**
   * Moves one or more infoitems to another folder.
   *
   * @return
   */
  def moveInfoItems = ???

  /**
   * Notifies users or groups about a shared infoitem.
   *
   * @return
   */
  def notifyAboutSharedInfoItem = ???

  /**
   * Search for infoitems.
   *
   * @return
   */
  def searchInfoItems = ???

  /**
   * Unlocks an infoitem.
   *
   * @return
   */
  def unlockInfoItem = ???

  /**
   * Updates an infoitem.
   *
   * @param id
   * @param timestamp
   * @param infoitem
   * @return
   */
  def updateInfoItem(id: Expression[String], timestamp: Expression[Long], infoitem: Expression[InfoItem]): ChainBuilder = group(ModuleNames.INFOSTORE) {
    exec(new UpdateInfoItem(id, timestamp, infoitem).buildHttpRequest())
  }

  /**
   * Updates an infoitem.
   *
   * @return
   */
  def updateInfoItemAdvanced = ???

}
