/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.infostore

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.common.utils.{ FileHelper, InfoItemGenerator }
import com.openexchange.testing.performance.groupware.infostore.InfoItemBody
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder

/**
 * [[InfostoreChain]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object InfostoreChain extends LazyLogging {
  private val random = ThreadLocalRandom.current()
  private val infoItemBodyName = classOf[InfoItemBody].getSimpleName
  private val createInfoItemName = classOf[CreateInfoItem].getSimpleName

  val clear: ChainBuilder = exec(session => {
    session.removeAll(infoItemBodyName, "filename", "fileExtension")
  })

  def createInfoItemAdvanced(folderId: Expression[String], optQueryParams: Option[Map[String, Any]] = None): ChainBuilder = {
    exec(session => for {
      fid <- folderId(session)
    } yield {
      val fileType = FileHelper.fileTypes(random.nextInt(FileHelper.fileTypes.length))
      val filename = s"${faker.name().name()}.$fileType"

      // generate infostore item
      val infoItem = InfoItemGenerator.generate(fid, filename)

      session.setAll(
        infoItemBodyName -> infoItem,
        "filename" -> filename,
        "fileExtension" -> fileType)
    })
      .doIf(session => session.contains(infoItemBodyName)) {
        exec(InfostoreAPI.createInfoItemAdvanced(folderId, optQueryParams))
          .doIfOrElse(session => session.contains(createInfoItemName)) {
            exec(session => for {
              id <- session(createInfoItemName).validate[String]
            } yield {
              logger.trace("Successfully created infoitem with id {}", id)
              session
            })
          } {
            exec(session => {
              logger.warn("{} failed.", createInfoItemName)
              session
            })
          }
      }
  }

}
