/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.jslob

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder

/**
 * [[JSlobAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object JSlobAPI {

  /** List core configuration */
  val getJSLobCore: ChainBuilder = getJSlobList("io.ox/core")

  /** List updates configuration */
  val getJSLobUpdates: ChainBuilder = getJSlobList("io.ox/core/updates")

  /** List mail configuration */
  val getJSLobMail: ChainBuilder = getJSlobList("io.ox/mail")

  /** List contacts configuration */
  val getJSLobContacts: ChainBuilder = getJSlobList("io.ox/contacts")

  /** List calendar configuration */
  val getJSLobCalendar: ChainBuilder = getJSlobList("io.ox/calendar")

  /** List upsell configuration */
  val getJSLobUpsell: ChainBuilder = getJSlobList("plugins/upsell/simple-wizard")

  /** List setting options configuration */
  val getJSLobSettingOptions: ChainBuilder = getJSlobList("io.ox/core/settingOptions")

  /** List caldav configuration */
  val getJSLobCalDAV: ChainBuilder = getJSlobList("io.ox/caldav")

  /** List emoji configuration */
  val getJSLobEmoji: ChainBuilder = getJSlobList("io.ox/mail/emoji")

  /** List office configuration */
  val getJSLobOffice: ChainBuilder = getJSlobList("io.ox/office")

  /** List tasks configuration */
  val getJSLobTasks: ChainBuilder = getJSlobList("io.ox/tasks")

  /** List files configuration */
  val getJSLobFiles: ChainBuilder = getJSlobList("io.ox/files")

  /** List tours configuration */
  val getJSLobTours: ChainBuilder = getJSlobList("io.ox/tours")

  /** List portal configuration */
  val getJSLobPortal: ChainBuilder = getJSlobList("io.ox/portal")

  /** List OXDriveClients configuration */
  val getJSLobDriveClients: ChainBuilder = getJSlobList("plugins/portal/oxdriveclients")

  val getJSLobOXGuard: ChainBuilder = getJSlobList("oxguard")

  /**
   * Gets a list of JSlobs.
   *
   * @param jsonBody A JSON array with the identifiers of the requested JSlobs.
   * @return JSlobsResponse
   */
  def getJSlobList(jsonBody: String): ChainBuilder = group(ModuleNames.JSLOB) {
    exec(new ListJSlobs(JSONUtil().toJson(Seq(jsonBody))))
  }

  /**
   * Stores or deletes a JSlob.
   *
   * @return
   */
  def setJSlob(jsonBody: String, id: String, serviceId: String = "com.openexchange.jslob.config"): ChainBuilder = group(ModuleNames.JSLOB) {
    exec(new SetJSlob(jsonBody, id, serviceId))
  }

  /**
   * Updates a JSlob.
   *
   * @return
   */
  def updateJSlob = ???
}
