/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.login

import com.openexchange.testing.performance.common.ModuleNames
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.check.HttpCheck

/**
 * [[LoginAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object LoginAPI {

  /** Performs a logout. */
  def logout(action: String = "logout"): ChainBuilder = group(ModuleNames.LOGIN) {
    exec(new Logout(action))
  }
  /** Refresh auto-login cookie. */
  val refreshAutoLogin: ChainBuilder = group(ModuleNames.LOGIN) {
    exec(new Store())
  }
  /** Refresh secret cookie. */
  val refreshSecret: ChainBuilder = group(ModuleNames.LOGIN) {
    exec(new RefreshSecret())
  }

  /** Performs a login. */
  def login(
    queryParams: Option[Map[String, Any]] = None,
    formParams:  Option[Map[String, Any]] = None,
    checks:      Seq[HttpCheck]           = Seq.empty[HttpCheck]): ChainBuilder = {
    group(ModuleNames.LOGIN) {
      exec(new Login().buildHttpRequest()
        .queryParamMap(queryParams.getOrElse(Map.empty))
        .formParamMap(formParams.getOrElse(Map.empty))
        .check(checks: _*))
    }
  }

  /** RampUp */
  def rampUp(optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(ModuleNames.LOGIN) {
    exec(new RampUp().buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty)))
  }

}
