/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.login

import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[RampUp]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @since 7.6.0
 */
class RampUp extends AbstractAuthenticatedAJAXRequest {

  import Login._

  override lazy val checks: Seq[HttpCheck] = Seq(
    // user specific identifier
    userLastname.optional.saveAs(USER_LAST_NAME),
    userFirstname.optional.saveAs(USER_FIRST_NAME),
    email1.optional.saveAs(USER_EMAIL1),
    // default system folder identifier
    fid_calender.optional.saveAs(FID_CALENDAR),
    fid_contacts.optional.saveAs(FID_CONTACTS),
    fid_tasks.optional.saveAs(FID_TASKS),
    fid_infostore.optional.saveAs(FID_INFOSTORE),
    fid_chronos.optional.saveAs(FID_CHRONOS),
    // default mail folder identifier
    fid_inbox.optional.saveAs(FID_INBOX),
    fid_sent.optional.saveAs(FID_SENT),
    fid_drafts.optional.saveAs(FID_DRAFTS),
    fid_spam.optional.saveAs(FID_SPAM),
    fid_trash.optional.saveAs(FID_TRASH),
    // default infostore folder identifier
    fid_infostore_pictures.optional.saveAs(FID_INFOSTORE_PICTURES),
    fid_infostore_trash.optional.saveAs(FID_INFOSTORE_TRASH),
    fid_infostore_music.optional.saveAs(FID_INFOSTORE_MUSIC),
    fid_infostore_videos.optional.saveAs(FID_INFOSTORE_VIDEOS),
    fid_infostore_documents.optional.saveAs(FID_INFOSTORE_DOCUMENTS),
    // capabilities
    capabilities.findAll.transform(_.toList).optional.saveAs(CAPABILITIES))
  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "login?action=rampup"

  override def buildHttpRequest(): HttpRequestBuilder = super.buildHttpRequest()
    .queryParam("rampup", true)
    .queryParam("rampupFor", "open-xchange-appsuite")
}