/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.mail

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[GetAllMails]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param columns A comma-separated list of columns to return.
 * @param folder  Object ID of the folder, whose contents are queried.
 */
class GetAllMails(folder: Expression[String], columns: String,
                  optQueryParams: Option[Map[String, Any]] = None) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "mail?action=all"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.FOLDER, folder)
      .queryParam(RequestParameters.COLUMNS, columns)
      .queryParamMap(optQueryParams.getOrElse(Map.empty).filter({ case (key, value) => value != None }))
  }
}
