/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.mail

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.mail.Mail
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[SendMail]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param mail The Mail object.
 */
class SendMail(mail: Expression[Mail]) extends AbstractAuthenticatedAJAXRequest {

  import SendMail._

  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = "mail?action=new"
  override lazy val checks: Seq[HttpCheck] = Seq(mailId)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .bodyPart(StringBodyPart("json_0", session => JSONUtil().toJson(mail(session).get))).asMultipartForm
  }
}

object SendMail {
  private[this] val sMAIL_ID = "data.*?(default[0-9]+.*[0-9])"

  def mailId = regex(sMAIL_ID).exists
}