/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.mail.account

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.groupware.mail.MailAccountData
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[MailAccountAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object MailAccountAPI {

  def getAllAccounts(columns: String): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new GetAllAccounts(columns).buildHttpRequest()
      .check(jsonPath("$.data[*][0]").ofType[String].findAll.transform(_.map(_.toInt).filter(_ > 0))
        .saveAs(classOf[GetAllAccounts].getSimpleName)))
  }

  def createAccount(body: Expression[MailAccountData]): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new NewAccount(body))
      .doIf(session => session.contains(classOf[MailAccountData].getSimpleName)) {
        exec(session => {
          session.removeAll(classOf[MailAccountData].getSimpleName, classOf[ValidateAccount].getSimpleName)
        })
      }
  }

  def deleteAccount(body: Expression[Seq[Int]]): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new DeleteAccount(body))
      .doIf(session => session.contains(classOf[GetAllAccounts].getSimpleName)) {
        exec(session => {
          session.remove(classOf[GetAllAccounts].getSimpleName)
        })
      }
  }

  def getAccount(id: Expression[Int]): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new GetAccount(id))
  }

  def updateAccount(body: Expression[MailAccountData]): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new UpdateAccount(body))
  }

  def validateAccount(
    body:                Expression[MailAccountData],
    optionalQueryParams: Option[Map[String, Any]]    = None): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new ValidateAccount(body, optionalQueryParams))
  }

  def getAccountStatus(id: Expression[Int]): ChainBuilder = group(ModuleNames.MAIL_ACCOUNT) {
    exec(new StatusAccount(id))
  }

}
