/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.mail.account

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.mail.MailAccountData
import io.gatling.core.Predef._
import io.gatling.core.check.CheckBuilder
import io.gatling.core.session.Expression
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[ValidateAccount]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class ValidateAccount(body: Expression[MailAccountData], optionalQueryParams: Option[Map[String, Any]] = None) extends AbstractAuthenticatedAJAXRequest {

  import ValidateAccount._

  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "account?action=validate"
  override lazy val checks: Seq[HttpCheck] = Seq(validate)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty))
      .body(StringBody(session => JSONUtil().toJson(body(session).get))).asJSON
      .check(jsonPath("$.data").ofType[Boolean].saveAs(getClass.getSimpleName))
  }
}

object ValidateAccount {
  private[this] val sVALIDATE = "$.data"

  def validate: CheckBuilder[HttpCheck, Response, Any, Boolean] = jsonPath(sVALIDATE)
    .ofType[Boolean].saveAs(getClass.getSimpleName)
}
