/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.oxodocumentconverter

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * Created by mhollmichel on 24.11.14.
 */
class ConvertDocument(uid: String, folder_id: String, file: String) extends AbstractAuthenticatedAJAXRequest {

  import ConvertDocument._

  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "oxodocumentconverter?action=convertdocument"
  override lazy val checks: Seq[HttpCheck] = Seq(pageCount)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(session => RequestParameters.FOLDER_ID.toString, folder_id)
      .queryParam("convert_action", "beginconvert")
      .queryParam("id", file)
      .queryParam("convert_format", "html")
      .queryParam("uid", uid)
  }
}

object ConvertDocument {
  private[this] val sPAGE_COUNT = "$.data.PageCount"

  def pageCount = jsonPath(sPAGE_COUNT).saveAs("OX_PAGE_COUNT")
}