/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.saml

import com.openexchange.testing.performance.common.request.{ AbstractSAMLRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.URLHelper
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[InitLoginFlow]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class InitLoginFlow extends AbstractSAMLRequest {
  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "saml/init?flow=login"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .check(InitLoginFlow.redirectUri.transform(url => {
        URLHelper.parseUrlParameters(url).filterKeys(s => s != "base_url")
      }).saveAs("OX_SAML_AUTH_PARAMS"))
  }
}

object InitLoginFlow {
  private val sREDIRECT_URI = "$.redirect_uri"

  def redirectUri = jsonPath(sREDIRECT_URI)
}