/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.share

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[DeleteShareLink]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param requestBody The share target.
 * @param timestamp   The timestamp.
 */
class DeleteShareLink(timestamp: Expression[Long], requestBody: String) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "share/management?action=deleteLink"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.TIMESTAMP, timestamp)
      .body(StringBody(requestBody)).asJSON
  }
}
