/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.share

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.{ ShareTarget, ShareTargetLink }
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[ShareManagementAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object ShareManagementAPI {

  /**
   * Creates or gets a share link.
   *
   * @param shareTarget
   * @return
   */
  def getShareLink(shareTarget: ShareTarget): ChainBuilder = group(ModuleNames.SHARE_MANAGEMENT) {
    exec(new GetShareLink(JSONUtil().toJson(shareTarget)).buildHttpRequest()
      .check(jsonPath("$").transform(x => JSONUtil().fromJson[GetLinkResponse](x)).saveAs("getLinkResponse")))
  }

  /**
   * Deletes a share link.
   *
   * @param timestamp
   * @param shareTarget
   * @return
   */
  def deleteShareLink(timestamp: Expression[Long], shareTarget: ShareTarget): ChainBuilder = group(ModuleNames.SHARE_MANAGEMENT) {
    exec(new DeleteShareLink(timestamp, JSONUtil().toJson(shareTarget)))
  }

  /**
   * Sends a share link.
   *
   * @return
   */
  def sendShareLink = ???

  /**
   * Updates a share link.
   *
   * @param timestamp
   * @param shareTargetLink
   * @return
   */
  def updateShareLink(timestamp: Expression[Long], shareTargetLink: ShareTargetLink): ChainBuilder = group(ModuleNames.SHARE_MANAGEMENT) {
    exec(new UpdateShareLink(timestamp, JSONUtil().toJson(shareTargetLink)).buildHttpRequest()
      .check(jsonPath("$.timestamp").saveAs("updateLinkResponse")))
  }

}
