/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.system

import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.check.body.HttpBodyJsonPathCheckBuilder

/**
 * [[WhoAmI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class WhoAmI extends AbstractAuthenticatedAJAXRequest {

  import WhoAmI._

  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = "system?action=whoami"
  override lazy val checks: Seq[HttpCheck] = Seq(
    userId.saveAs(USER_ID),
    contextId.saveAs(CONTEXT_ID),
    locale.saveAs(LOCALE))
}

object WhoAmI {
  private[this] val sSESSION_ID = "$.data.session"
  private[this] val sUSER_ID = "$.data.user_id"
  private[this] val sCONTEXT_ID = "$.data.context_id"
  private[this] val sLOCALE = "$.data.locale"

  def sessionId: HttpBodyJsonPathCheckBuilder[String] = jsonPath(sSESSION_ID).ofType[String]

  def userId: HttpBodyJsonPathCheckBuilder[Int] = jsonPath(sUSER_ID).ofType[Int]

  def contextId: HttpBodyJsonPathCheckBuilder[Int] = jsonPath(sCONTEXT_ID).ofType[Int]

  def locale: HttpBodyJsonPathCheckBuilder[String] = jsonPath(sLOCALE).ofType[String]
}
