/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.tasks

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.utils.JSONUtil
import com.openexchange.testing.performance.groupware.{ DetailedTaskData, Task, TaskAppointmentData }
import io.gatling.core.Predef._
import io.gatling.core.session._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[TasksAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object TasksAPI {
  private val moduleName = ModuleNames.TASKS

  /**
   * Confirms a task.
   */
  def confirmTask = ???

  /**
   * Creates a task.
   */
  def createTask(task: Expression[Task]): ChainBuilder = group(moduleName) {
    exec(new CreateTask(task).buildHttpRequest()
      .check(jsonPath("$.data.id").ofType[Int].saveAs(classOf[CreateTask].getSimpleName)))
  }

  /**
   * Deletes tasks.
   */
  def deleteTasks(timestamp: Expression[Long], requestBody: String): ChainBuilder = group(moduleName) {
    exec(new DeleteTasks(timestamp, requestBody))
  }

  /**
   * Gets all tasks.
   */
  def getAllTasks(folder: String, columns: String, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(moduleName) {
    exec(new GetAllTasks(folder, columns).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty))
      .check(bodyString.transform(x => JSONUtil().fromJson[GetAllTasksResponse](x)).optional.saveAs("getAllTasksResponse")))
  }

  /**
   * Gets a task.
   */
  def getTask(id: String, folder: String): ChainBuilder = group(moduleName) {
    exec(new GetTask(id, folder).buildHttpRequest()
      .check(jsonPath("$.data").transform(x => JSONUtil().fromJson[TaskAppointmentData](x)).saveAs("TaskAppointmentData"))
      .check(jsonPath("$.data").transform(x => JSONUtil().fromJson[DetailedTaskData](x)).saveAs("DetailedTaskData")))
      .exec(session => {
        val taskAppointmentData = session("TaskAppointmentData").as[TaskAppointmentData]
        val detailedTaskData = session("DetailedTaskData").as[DetailedTaskData]

        val task = Task(taskAppointmentData, detailedTaskData)

        session.set("getTask", task)
      })
      .exec(session => {
        session.removeAll("TaskAppointmentData", "DetailedTaskData") // TODO cleanup
      })
  }

  /**
   * Gets a list of tasks.
   */
  def getTaskList(columns: String, objectIds: Expression[Array[Map[String, Any]]]): ChainBuilder = group(moduleName) {
    exec(new GetTaskList(columns, objectIds))
  }

  /**
   * Search for tasks.
   */
  def searchTasks(columns: String, searchTask: SearchTasks, optionalQueryParams: Option[Map[String, Any]] = None): ChainBuilder = group(moduleName) {
    exec(new SearchTasks(columns, JSONUtil().toJson(searchTask)).buildHttpRequest()
      .queryParamMap(optionalQueryParams.getOrElse(Map.empty)))
  }

}
