/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.ajax.user

import com.openexchange.testing.performance.common.ModuleNames
import com.openexchange.testing.performance.common.SessionConstants._
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[UserAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object UserAPI {

  /**
   * Gets a user.
   */
  val getUser: ChainBuilder = group(ModuleNames.USER) {
    exec(new GetUser().buildHttpRequest()
      .check(jsonPath("$.data.first_name").saveAs(USER_FIRST_NAME))
      .check(jsonPath("$.data.last_name").saveAs(USER_LAST_NAME))
      .check(jsonPath("$.data.email1").saveAs(USER_EMAIL1)))
  }

  /**
   * Gets a list of users.
   */
  def getUserList(columns: String, body: String): ChainBuilder = group(ModuleNames.USER) {
    exec(new ListUser(columns, body))
  }

  /**
   * Gets all users.
   */
  def getAllUsers(columns: String): ChainBuilder = group(ModuleNames.USER) {
    exec(new GetAllUsers(columns))
  }

  /**
   * Gets a user attribute.
   */
  def getUserAttribute(id: String, name: String): ChainBuilder = group(ModuleNames.USER) {
    exec(new GetUserAttribute(id, name))
  }

  /**
   * Search for users.
   *
   * @return
   */
  def searchUsers = ???

  /**
   * Sets a user attribute.
   *
   * @return
   */
  def setUserAttribute = ???

  /**
   * Updates a user.
   *
   * @return
   */
  def updateUser = ???
}
