/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.guard.keys

import com.openexchange.testing.performance.common.APINames
import com.openexchange.testing.performance.guard.ModuleNames
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.core.structure.ChainBuilder

/**
 * [[GuardKeysAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object GuardKeysAPI {

  def createKeys(name: Expression[String], password: Expression[String]): ChainBuilder = groupChain(
    exec(new Create(name, password)))

  def deleteKey(keyId: Expression[String], password: Expression[String]): ChainBuilder = groupChain(
    exec(new Delete(keyId, password)))

  def getKeys(): ChainBuilder = groupChain(exec(new GetKeys))

  def downloadKey(
    keyType:  Option[String]             = Some("public_private"),
    keyid:    Option[Int]                = None,
    password: Option[Expression[String]] = None): ChainBuilder = groupChain(
    exec(new DownloadKey(keyType, keyid, password)))

  def getExternalPublicKeys(): ChainBuilder = groupChain(exec(new GetExternalPublicKeys))

  def getRecipKey(
    create: Option[Boolean] = None,
    email:  Option[String]  = None,
    keyid:  Option[Int]     = None): ChainBuilder = groupChain(
    exec(new GetRecipKey(create, email, keyid)))

  def hasKey(): ChainBuilder = groupChain(exec(new HasKey))

  def revoke(
    password: Expression[String],
    reason:   Option[String]     = Some("no_reason"),
    keyid:    Option[String]     = None): ChainBuilder = groupChain(
    exec(new Revoke(password, reason, keyid)))

  def setCurrentKey(keyid: String): ChainBuilder = groupChain(exec(new SetCurrentKey(keyid)))

  def shareExternalPublicKey(ids: String, share: Boolean): ChainBuilder = groupChain(
    exec(new ShareExternalPublicKey(ids, share)))

  def upload(
    key:         Expression[String],
    keyPassword: Option[String]     = None,
    newPassword: Option[String]     = None): ChainBuilder = groupChain(
    exec(new Upload(key, keyPassword, newPassword)))

  private def groupChain(chain: ChainBuilder*): ChainBuilder = group(APINames.GUARDAPI) {
    group(ModuleNames.KEYS) {
      exec(chain)
    }
  }
}
