/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.guard.login

import com.openexchange.testing.performance.common.APINames
import com.openexchange.testing.performance.guard.ModuleNames
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder

/**
 * [[GuardLoginAPI]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
object GuardLoginAPI {

  def login(
    language:      Option[String] = None,
    encr_password: Option[String] = None): ChainBuilder = groupChain(
    exec(new Login(LoginData(language, encr_password))))

  def logoutGuard(): ChainBuilder = groupChain(exec(new Logout)) // TODO cleanup guard related things from the gatling session

  def changePassword(
    newpass: Option[String] = None,
    oldpass: Option[String] = None): ChainBuilder = groupChain(
    exec(new ChangePass(ChangePasswordData(newpass, oldpass)))) // TODO update auth pass in gatling session

  def resetPassword(): ChainBuilder = groupChain(exec(new Reset))

  def changeSecondary(
    email:    Option[String] = None,
    password: Option[String] = None): ChainBuilder = groupChain(
    exec(new ChangeSecondary(ChangeSecondaryData(email, password))))

  def getSecondary(): ChainBuilder = groupChain(exec(new Secondary))

  private def groupChain(chain: ChainBuilder*): ChainBuilder = group(APINames.GUARDAPI) {
    group(ModuleNames.LOGIN) {
      exec(chain)
    }
  }
}
