/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.guard.login

import com.openexchange.testing.performance.ajax.jslob.JSlobAPI
import com.openexchange.testing.performance.common.Capabilities.guard
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.utils.CapabilityCheck
import com.openexchange.testing.performance.guard.keys.GuardKeysAPI
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder

object GuardLoginChain extends LazyLogging {

  def login(): ChainBuilder = {
    CapabilityCheck(guard, GuardLoginAPI.login())
      .doIf(session => session.contains(GUARD_AUTH)) {
        doIfEquals(session => session(GUARD_AUTH).as[String], "No Key") {
          exec(session => for {
            uid <- session(USER_ID).validate[String]
            cid <- session(CONTEXT_ID).validate[String]
          } yield {
            logger.debug("Key not found!\nCreating keys for user with id {} in context {}", uid, cid)
            logger.debug("Creating keys can take very long. Please consider creating keys before running the simulation.", uid, cid)
            session
          })
            .exec(GuardKeysAPI.createKeys(USERNAME, PASSWORD))
        }
          .exec(JSlobAPI.getJSLobOXGuard)
      }
  }

  val logout: ChainBuilder = exec(GuardLoginAPI.logoutGuard())
}
