/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.time.temporal.ChronoUnit
import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.ajax.chronos.{ ChronosAPI, ChronosChain }
import com.openexchange.testing.performance.ajax.contacts.ContactChain
import com.openexchange.testing.performance.ajax.folders.FoldersAPI
import com.openexchange.testing.performance.ajax.login.LoginChain
import com.openexchange.testing.performance.ajax.mail.MailAPI
import com.openexchange.testing.performance.ajax.mail.account.MailAccountAPI
import com.openexchange.testing.performance.ajax.reminder.ReminderAPI
import com.openexchange.testing.performance.ajax.tasks.{ TaskChain, TasksAPI }
import com.openexchange.testing.performance.ajax.user.UserAPI
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.utils.DateTimeHelper
import com.openexchange.testing.performance.common.{ BaseSimulation, Capabilities }
import com.openexchange.testing.performance.guard.login.GuardLoginChain
import io.gatling.core.Predef._
import io.gatling.core.structure.{ ChainBuilder, ScenarioBuilder }

import scala.concurrent.duration._

/**
 * [[AppSuiteSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class AppSuiteSimulation extends BaseSimulation {

  object Idle {
    val chain: ChainBuilder =
      doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.webmail) && settings.appsuite.modules.getOrElse(Capabilities.webmail, 25.0) > 0) {
        exec(MailAccountAPI.getAllAccounts("1001,1002,1003,1004,1005,1006,1007,1008," +
          "1009,1010,1011,1012,1013,1014,1015,1016,1017,1018,1019,1020,1021,1022,1023,1024,1025,1026,1027,1028,1029," +
          "1030,1031,1032,1033,1034,1035,1036,1037,1038,1039,1040,1041,1042,1043,1044,1045,1046,1047,1048"))
      }
        // TODO multiple
        // TODO tasks all
        .doIf(
          session => (session(CAPABILITIES).as[Seq[String]].contains(Capabilities.calendarChronos) && settings.appsuite.modules.getOrElse(Capabilities.calendarChronos, 25.0) > 0) || (session(CAPABILITIES).as[Seq[String]].contains(Capabilities.tasks) && settings.appsuite.modules.getOrElse(Capabilities.tasks, 25.0) > 0)) {
            exec(ReminderAPI.getReminderRange(
              end = Some(DateTimeHelper.forOrBackward(-1, ChronoUnit.HOURS).getTimestamp),
              modules = Some("tasks")))
          }
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.webmail) && settings.appsuite.modules.getOrElse(Capabilities.webmail, 25.0) > 0) {
          exec(MailAPI.getAllMails($(FID_INBOX), "610,600,601,611", unseen = Some(true), deleted = Some(false),
            sort = Some("610"), order = Some("desc"), limit = Some("100")))
        }
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.calendarChronos) && settings.appsuite.modules.getOrElse(Capabilities.calendarChronos, 25.0) > 0) {
          exec(ChronosAPI.needsAction(
            DateTimeHelper.forOrBackward(-2, ChronoUnit.HOURS),
            DateTimeHelper.now()))
            .exec(ChronosAPI.pending(DateTimeHelper.forOrBackward(10, ChronoUnit.HOURS), Some("DISPLAY,AUDIO")))
        }
        // TODO multiple mail quota
        // TODO mailfilterv2
        // parent=1 is always the "virtual root" folder, call unconditionally.
        .exec(FoldersAPI.getSubFolders("1", "1,2,3,4,5,6,20,23,300,301,302,304,305,306,307,308,309,310," +
          "311,312,313,314,315,316,317,318,319,3010,3020,3030,3050,3201,3203,3204,3205,3220", Some(Map(
          "all" -> 0, "altNames" -> true, "timezone" -> "UTC", "tree" -> 0))))
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.tasks) && settings.appsuite.modules.getOrElse(Capabilities.tasks, 25.0) > 0) {
          exec(TasksAPI.getAllTasks($(FID_TASKS), "1,5,20,101,200,203,220,300,301,317,401", Some(Map(
            "sort" -> "317", "timezone" -> "UTC", "order" -> "asc", "cache" -> true))))
        }
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.webmail) && settings.appsuite.modules.getOrElse(Capabilities.webmail, 25.0) > 0) {
          exec(MailAPI.getAllMails($(FID_INBOX), "102,600,601,602,603,604,605,606,607,608,610,611,614,652,656," +
            "X-Open-Xchange-Share-URL,662,664", sort = Some("610"), order = Some("desc"), deleted = Some(false), limit = Some("0,50")))
        }
        .exec(UserAPI.getUserList("1,20,500,501,502,505,524,555,606,614", "[${OX_USER_ID}]"))
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.webmail) && settings.appsuite.modules.getOrElse(Capabilities.webmail, 25.0) > 0) {
          exec(FoldersAPI.getSubFolders($(FID_INBOX), "1,2,3,4,5,6,20,23,300,301,302,304,305,306,307,308,309," +
            "310,311,312,313,314,315,316,317,318,319,3010,3020,3030,3050,3201,3203,3204,3205,3220",
            Some(Map("all" -> 0, "altNames" -> true, "timezone" -> "UTC", "tree" -> 0))))
        }
        // TODO what is folder 9?! -> seems to be infostore folder
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.infostore) && settings.appsuite.modules.getOrElse(Capabilities.infostore, 0.0) > 0) {
          exec(FoldersAPI.getSubFolders("9", "1,2,3,4,5,6,20,23,300,301,302,304,305,306,307,308,309," +
            "310,311,312,313,314,315,316,317,318,319,3010,3020,3030,3050,3201,3203,3204,3205,3220", Some(Map(
            "all" -> 0, "altNames" -> true, "timezone" -> "UTC", "tree" -> 1))))
        }
  }

  override def scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .feed(csv(settings.simulation.userFeeder).circular)
    .exec(LoginChain.doLogin())

    .doIf(session => session.contains(SESSION_ID)) {
      doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.guard)) {
        exec(GuardLoginChain.login())
      }
        .during(finiteDuration, exitASAP = false) {
          pace(new FiniteDuration(settings.simulation.injection.loopTime, TimeUnit.SECONDS))
            .exec(Idle.chain)
            .randomSwitch(
              // if for production we want to have an idle duration of 300 seconds and assume a user creates 10 objects per workday
              // (mails / appointments / ...), this gives approx 10% chance to enter the "create something" chain
              // like, an 8 hour day has 28800 seconds, 10/28800*300 is approxmately 0.1
              settings.simulation.data.probabilities.nonIdle.getOrElse(100).toDouble -> randomSwitch(
                settings.appsuite.modules.getOrElse(Capabilities.webmail, 25.0) ->
                  doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.webmail) && settings
                    .appsuite.modules.contains(Capabilities.webmail)) {
                    exec(OXAppSuiteChains.mailChain)
                  },
                settings.appsuite.modules.getOrElse(Capabilities.calendarChronos, 25.0) -> exec(ChronosChain.chronosChain),
                settings.appsuite.modules.getOrElse(Capabilities.tasks, 25.0) -> exec(TaskChain.taskChain),
                settings.appsuite.modules.getOrElse(Capabilities.contacts, 25.0) -> exec(ContactChain.contactChain),
                settings.appsuite.modules.getOrElse(Capabilities.infostore, 0.0) ->
                  doIf(session => session(CAPABILITIES).as[Seq[String]].contains(Capabilities.infostore) && settings
                    .appsuite.modules.contains(Capabilities.infostore)) {
                    exec(OXAppSuiteChains.driveChain)
                  }))
        }
        .exec(LoginChain.doLogout())
    }

}
