/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.ajax.login.LoginAPI
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder

import scala.concurrent.duration._

/**
 * [[CreatePGPKeys]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
class CreatePGPKeys extends Simulation with Configuration with LazyLogging {

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .feed(csv(settings.simulation.userFeeder).queue)
    .tryMax(5) {
      exec(LoginAPI.login(formParams = Some(
        Map(
          "client" -> "open-xchange-appsuite",
          "timeout" -> "10000",
          "rampup" -> true,
          "rampupFor" -> "open-xchange-appsuite"))))
    }

    .doIf(session => session.contains(FID_INBOX) && session.contains(USER_EMAIL1) && session.contains(SESSION_ID)) {
      // FIXME exec(login($(USER_ID), $(CONTEXT_ID), $(USER_EMAIL1), PASSWORD)).
      doIfEqualsOrElse(session => session(GUARD_AUTH).as[String], "No Key") {
        exec(session => for {
          uid <- session(USER_ID).validate[String]
          cid <- session(CONTEXT_ID).validate[String]
        } yield {
          logger.debug("Creating keys for user with id {} in context {} ...", uid, cid)
          session
        })
        /* FIXME
          .randomSwitch(
            80d -> exec(createKeys(Some(
              List(
                "email" -> "" /*"secondary@example.com"*/ ,
                "language" -> "en_US",
                "name" -> USERNAME,
                "password" -> PASSWORD,
                "sessionID" -> $(SESSION_ID),
                "user_id" -> $(USER_ID))))))
        */
      } {
        exec(session => for {
          uid <- session(USER_ID).validate[String]
          cid <- session(CONTEXT_ID).validate[String]
        } yield {
          logger.debug("User with id {} in context {} has already keys!", uid, cid)
          session
        })
      }
    }

  setUp(scn.inject(constantUsersPerSec(settings.simulation.injection.usersPerSec) during new FiniteDuration(settings.simulation
    .injection.duration, TimeUnit.SECONDS)))
    .protocols(HTTPConfig.appsuiteConf
      .acceptCharsetHeader("text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"))
}
