/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.{ ThreadLocalRandom, TimeUnit }

import com.openexchange.testing.performance.ajax.calendar.CalendarChain
import com.openexchange.testing.performance.ajax.config.ConfigAPI
import com.openexchange.testing.performance.ajax.contacts.ContactChain
import com.openexchange.testing.performance.ajax.login.LoginChain
import com.openexchange.testing.performance.ajax.mail.MailAPI
import com.openexchange.testing.performance.ajax.tasks.TaskChain
import com.openexchange.testing.performance.common.Capabilities._
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.utils.MailGenerator
import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._

import scala.concurrent.duration._

/**
 * [[PSOXAppSuiteFillerSimulation]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 *
 *         based on PSOXAppSuiteRealisticSimulation
 */
class PSOXAppSuiteFillerSimulation extends Simulation with Configuration with LazyLogging {

  // l to be an array with some elements
  // e.g. Array(100, 195, 5, 10)
  // which represents (generate 100 mails, 195 appointments w.o. recurring, 5 appointments with recurring, 10 contacts)
  // take one random element of this list and return it
  // and modify the list to look like e.g. (99, 195, 5, 10) if returned value is 1
  def takeOne(l: Array[Int]): Int = {
    val i = ThreadLocalRandom.current().nextInt(l.sum)
    var sum = 0
    var j = 0
    while (j < l.length && i >= sum + l(j)) {
      sum = sum + l(j)
      j = j + 1
    }
    l(j) = l(j) - 1
    j
  }

  val scn = scenario(getClass.getSimpleName)
    .repeat(settings.simulation.injection.repeatCounter) {
      feed(csv(settings.simulation.userFeeder))
        // TODO
        //        .doIf(settings.simulation.data.password.isDefined) {
        //          exec(session => {
        //            session.set("password", settings.simulation.data.password.get)
        //          })
        //        }
        .exec(LoginChain.doLogin())
        .doIf(session => session.contains(SESSION_ID)) {
          // pause(10 seconds, 25 seconds)
          pause(100 milliseconds, 200 milliseconds)
            .foreach("${modules}", "module") {
              doSwitch("${module}")(
                webmail -> exec(
                  ConfigAPI.getDefaultFolderInbox,
                  ConfigAPI.getDefaultFolderDrafts,
                  ConfigAPI.getDefaultFolderSent,
                  ConfigAPI.getDefaultFolderSpam,
                  ConfigAPI.getDefaultFolderTrash),
                calendar -> ConfigAPI.getDefaultFolderCalendar,
                contacts -> ConfigAPI.getDefaultFolderContacts,
                tasks -> ConfigAPI.getDefaultFolderTasks)
            }
            .exec(session => {
              val itemsToCreate = Array(100, 100, 195, 5, 10)
              session.set("itemsToCreate", itemsToCreate)
            })
            .asLongAs(session => session("itemsToCreate").as[Array[Int]].sum > 0) {
              pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
                .exec(session => {
                  var itemsToCreate = session("itemsToCreate").as[Array[Int]]
                  val itemToCreate = takeOne(itemsToCreate)
                  session.setAll("itemsToCreate" -> itemsToCreate, "itemToCreate" -> itemToCreate)
                })
                .doSwitch("${itemToCreate}")(
                  0 -> // create a mail
                    exec(session => {
                      val firstName = session(USER_FIRST_NAME).as[String]
                      val lastName = session(USER_LAST_NAME).as[String]
                      val mailAddress = session(USER_EMAIL1).as[String]
                      val mail = MailGenerator.generateRandomAppSuiteMail(firstName, lastName, mailAddress)
                      session.set("newMail", mail)
                    })
                    .exec(MailAPI.sendMail("${newMail}")),
                  1 -> // create appt w.o. recurring
                    exec(ContactChain.createChain($(FID_CONTACTS))),
                  2 -> // create appt w.o. recurring
                    exec(CalendarChain.createAppointment($(FID_CALENDAR), rangeDays = 20, allowRecurring = false)),
                  3 -> // create appt w. recurring
                    exec(CalendarChain.createAppointment($(FID_CALENDAR), rangeDays = 20, allowRecurring = true)),
                  4 -> //create task
                    exec(TaskChain.createChain($(FID_TASKS))))
            }
            .exec(LoginChain.doLogout())
        }
    }

  setUp(scn.inject(
    rampUsers(settings.simulation.injection.users) over new FiniteDuration(settings.simulation.injection.users / 2, TimeUnit
      .SECONDS)))
    .protocols(HTTPConfig.appsuiteConf
      .acceptCharsetHeader("text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"))
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
