/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.ajax.config.ConfigAPI
import com.openexchange.testing.performance.ajax.folders.FoldersAPI
import com.openexchange.testing.performance.ajax.login.LoginChain
import com.openexchange.testing.performance.common.Capabilities._
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.simulations.GuardChains._
import com.openexchange.testing.performance.simulations.PSOXAppSuiteChains._
import io.gatling.core.Predef._

import scala.concurrent.duration._

/**
 * [[PSOXAppSuiteRealisticSimulation]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 *
 *         chains and structure copied from QAOXAppSuiteSoakSimulation
 *         Flow logic and rampup and timings taken from that old OXAppSuiteJMreal
 *         simulation from commit 4335a5ba08465980e5b1119844c4474868fdec7d
 */
class PSOXAppSuiteRealisticSimulation extends Simulation with Configuration {

  val scn = scenario(getClass.getSimpleName)

    .feed(csv(settings.simulation.userFeeder))
    // TODO
    //    .doIf(settings.simulation.data.password.isDefined) {
    //      exec(session => {
    //        session.set("password", settings.data.password.get)
    //      })
    //    }
    .exec(LoginChain.doLogin())

    .doIf(session => session.contains(SESSION_ID)) {
      // pause(10 seconds, 25 seconds)
      pause(100 milliseconds, 200 milliseconds)
        .foreach("${modules}", "module") {
          doSwitch("${module}")(
            webmail -> exec(
              ConfigAPI.getDefaultFolderInbox,
              ConfigAPI.getDefaultFolderDrafts,
              ConfigAPI.getDefaultFolderSent,
              ConfigAPI.getDefaultFolderSpam,
              ConfigAPI.getDefaultFolderTrash),
            calendar -> ConfigAPI.getDefaultFolderCalendar,
            contacts -> ConfigAPI.getDefaultFolderContacts,
            infostore -> exec(ConfigAPI.getDefaultFolderInfostore, ConfigAPI.getDefaultFolderInfostoreTrash),
            tasks -> ConfigAPI.getDefaultFolderTasks)
        }
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(guard)) {
          exec(loginChainGuard)
        }

        .during(new FiniteDuration(settings.simulation.injection.duration, TimeUnit.SECONDS), exitASAP = false) {
          //.repeat(1) {
          pace(new FiniteDuration(settings.simulation.injection.loopTime, TimeUnit.SECONDS))
            .exec(idleChain)
            .pause(100 milliseconds, 200 milliseconds)
            .randomSwitch(
              // if for production we want to have an idle duration of 300 seconds and assume a user creates 10 objects per workday
              // (mails / appointments / ...), this gives approx 10% chance to enter the "create something" chain
              // like, an 8 hour day has 28800 seconds, 10/28800*300 is approxmately 0.1
              10.0 -> randomSwitch(
                60d -> doIf(session => session("modules").as[Seq[String]].contains(webmail) && session(CAPABILITIES).as[Seq[String]].contains(webmail)) {
                  realisticMailChain
                },
                20d -> doIf(session => session("modules").as[Seq[String]].contains(calendar) && session(CAPABILITIES).as[Seq[String]].contains(calendar)) {
                  realisticCalendarChain
                },
                10d -> doIf(session => session("modules").as[Seq[String]].contains(tasks) && session(CAPABILITIES).as[Seq[String]].contains(tasks)) {
                  realisticTasksChain
                },
                10d -> doIf(session => session("modules").as[Seq[String]].contains(contacts) && session(CAPABILITIES).as[Seq[String]].contains(contacts)) {
                  realisticContactsChain
                } /* ,
                0d -> doIf(session => session("modules").as[Seq[String]].contains(infostore) && session(CAPABILITIES).as[Seq[String]].contains(infostore)) {
                  driveChain
                } */
              ))
        }

        // cleanup
        .doIf(settings.simulation.data.keepData) {
          foreach(settings.appsuite.modules.keys.toList, "module") {
            doSwitch("${module}")(
              "webmail" -> FoldersAPI.clearFolders(Array($(FID_SENT), $(FID_DRAFTS), $(FID_TRASH))),
              "calendar" -> FoldersAPI.clearFolders(Array($(FID_CALENDAR))),
              "contacts" -> FoldersAPI.clearFolders(Array($(FID_CONTACTS))),
              "infostore" -> FoldersAPI.clearFolders(Array($(FID_INFOSTORE), $(FID_INFOSTORE_TRASH))),
              "tasks" -> FoldersAPI.clearFolders(Array($(FID_TASKS))))
          }
        }

        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(guard)) {
          exec(logoutChainGuard)
        }
        .exec(LoginChain.doLogout())
    }

  setUp(scn.inject(
    rampUsers(settings.simulation.injection.users) over new FiniteDuration(settings.simulation.injection.duration / settings
      .simulation.injection.rampup, TimeUnit.SECONDS)))
    .protocols(HTTPConfig.appsuiteConf
      .acceptCharsetHeader("text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8").disableWarmUp)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}

