/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.ajax.login.LoginChain
import com.openexchange.testing.performance.common.Capabilities._
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.simulations.OXAppSuiteChains._
import io.gatling.core.Predef._

import scala.concurrent.duration._

/**
 * [[QAOXAppSuitePerfSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class QAOXAppSuitePerfSimulation extends Simulation with Configuration {

  val scn = scenario(getClass.getSimpleName)

    .during(new FiniteDuration(settings.simulation.injection.duration - (settings.simulation.injection.duration /
      settings.simulation.injection.rampup), TimeUnit.SECONDS), exitASAP = false) {
      feed(csv(settings.simulation.userFeeder).circular)
        .exec(LoginChain.doLogin())
        .doIf(session => session(CAPABILITIES).as[Seq[String]].contains(guard)) {
          exec(GuardChains.loginChainGuard)
        }

        .doIf(session => session.contains(SESSION_ID)) {
          foreach(settings.appsuite.modules.keys.toList, "module") {
            doSwitch("${module}")(
              // TODO add portal chain
              webmail -> doIf(session => session(CAPABILITIES).as[Seq[String]].contains(webmail)) {
                mailChain
              },
              calendar -> doIf(session => session(CAPABILITIES).as[Seq[String]].contains(calendar)) {
                calendarChain
              },
              contacts -> doIf(session => session(CAPABILITIES).as[Seq[String]].contains(contacts)) {
                contactChain
              },
              infostore -> doIf(session => session(CAPABILITIES).as[Seq[String]].contains(infostore)) {
                driveChain
              },
              tasks -> doIf(session => session(CAPABILITIES).as[Seq[String]].contains(tasks)) {
                taskChain
              })
          }
            .exec(LoginChain.doLogout())
        }
    }

  setUp(scn.inject(rampUsers(settings.simulation.injection.users) over new FiniteDuration(settings.simulation.injection
    .duration / settings.simulation.injection.rampup, TimeUnit.SECONDS)))
    .protocols(HTTPConfig.appsuiteConf
      .acceptCharsetHeader("application/json, text/javascript, */*; q=0.01")
      .maxConnectionsPerHostLikeChrome)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
