import Dependencies._
import Resolvers._
import com.typesafe.sbt.SbtScalariform.ScalariformKeys

import scalariform.formatter.preferences._

//================================================================================
// Basic Settings
//================================================================================
lazy val commonSettings = Defaults.coreDefaultSettings ++ Seq(
  startYear := Some(2018),
  organizationName := "OX Software GmbH",
  version := "1.0.0",
  scalaVersion := "2.12.8",
  resolvers := repoResolver,
  autoAPIMappings := true,
  autoScalaLibrary := false,
  libraryDependencies ++= gatlingDependencies,
  shellPrompt := { s => Project.extract(s).currentProject.id + "> " },
  fork := true,
  javacOptions := Seq(
    "-Xlint:-options",
    "-source", "1.8",
    "-target", "1.8"),
  scalacOptions := Seq(
    "-encoding",
    "UTF-8",
    "-target:jvm-1.8",
    "-deprecation",
    "-feature",
    "-unchecked",
    "-language:postfixOps"),
  javaOptions in Gatling := overrideDefaultJavaOptions("-Xms2048m", "-Xmx4096m"),
  headerLicense := Some(HeaderLicense.Custom(
    s"""|Copyright (C) ${startYear.value.get} ${organizationName.value}
        |
        |This file is part of OX API-Performance Tests.
        |
        |OX API-Performance Tests is free software: you can redistribute it and/or modify
        |it under the terms of the GNU General Public License as published by
        |the Free Software Foundation, either version 3 of the License, or
        |(at your option) any later version.
        |
        |OX API-Performance Tests is distributed in the hope that it will be useful,
        |but WITHOUT ANY WARRANTY; without even the implied warranty of
        |MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        |GNU General Public License for more details.
        |
        |You should have received a copy of the GNU General Public License
        |along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
        |""".stripMargin))
)

//================================================================================
// Formatting
//================================================================================
lazy val formattingSettings =  Seq(
  ScalariformKeys.preferences := ScalariformKeys.preferences.value
    .setPreference(DoubleIndentConstructorArguments, true)
    .setPreference(AlignParameters, true)
    .setPreference(AlignSingleLineCaseStatements, true)
    .setPreference(IndentLocalDefs, true)
    .setPreference(SpacesAroundMultiImports, true)
)

//================================================================================
// Projects
//================================================================================
lazy val root = (project in file("."))
  .settings(commonSettings: _*)
  .settings(name := "OX API-Performance Tests")
  .enablePlugins(PreprocessPlugin)
  .enablePlugins(SiteScaladocPlugin)
  .aggregate(ajax, core, drive, facade, soap, mim, mdb, dav, imageconverter)

lazy val core = (project in file("core"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .enablePlugins(AutomateHeaderPlugin)
  .settings(libraryDependencies ++= coreDependencies)

lazy val ajax = (project in file("ajax"))
  .settings(commonSettings: _*)
  .settings(name := "OX HTTP-API-Performance Tests")
  .settings(formattingSettings: _*)
  .enablePlugins(GatlingPlugin, PreprocessPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")

lazy val drive = (project in file("drive"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX Drive-API-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(ajax % "compile->compile;test->test")
  .dependsOn(core % "compile->compile;test->test")

lazy val facade = (project in file("facade"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX Mobile-Facade-API-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")
  .dependsOn(ajax % "compile->compile;test->test")

lazy val soap = (project in file("soap"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX SOAP-API-Performance Tests")
  .enablePlugins(GatlingPlugin)
  .dependsOn(core % "compile->compile;test->test")

lazy val mim = (project in file("mim"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX MIM-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")

lazy val mdb = (project in file("mdb"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX MDB-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")

lazy val dav = (project in file("dav"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX CalDAV/CardDAV-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")

lazy val imageconverter = (project in file("imageconverter"))
  .settings(commonSettings: _*)
  .settings(formattingSettings: _*)
  .settings(name := "OX ImageConverter-Performance Tests")
  .enablePlugins(GatlingPlugin, AutomateHeaderPlugin)
  .dependsOn(core % "compile->compile;test->test")
