/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.feeder

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.groupware.mail.MailAccountData
import io.gatling.core.feeder.Feeder

/**
 * [[MailAccountFeeder]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object MailAccountFeeder {
  val random: ThreadLocalRandom = ThreadLocalRandom.current

  def apply(): Feeder[Any] = {
    Iterator.continually({
      val username: String = faker.name().username()

      Map(classOf[MailAccountData].getSimpleName -> MailAccountData(
        unified_inbox_enabled = Some(false),
        transport_auth = Some("mail"),
        primary_address = Some(s"$username@localhost"),
        spam_handler = Some("NoSpamHandler"),
        personal = Some(s"$username@localhost"),
        mail_server = Some("localhost"),
        mail_port = Some(143),
        mail_protocol = Some("imap"),
        login = Some(s"$username"),
        mail_url = Some("imap://localhost:143"),
        transport_url = Some("smtp://localhost:587"),
        name = Some(s"$username@localhost"),
        transport_server = Some("localhost"),
        transport_port = Some(587),
        transport_protocol = Some("smtp"),
        password = Some("secret")))
    })
  }
}
