/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.request

import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.action.sync.HttpRequestActionBuilder
import io.gatling.http.request.builder.HttpRequestBuilder
import io.gatling.http.response.{ ResponseWrapper, StringResponseBody }

import scala.language.implicitConversions
import scala.util.{ Failure, Success, Try }

/**
 * [[AbstractAJAXRequest]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
abstract class AbstractAJAXRequest extends Request {

  override val path: String = settings.middleware.path

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      // extract JSON part
      // from responses where content type is text/html
      .transformResponse({
        case response if response.isReceived =>
          new ResponseWrapper(response) {
            override val body = new StringResponseBody(
              if (response.header("Content-Type").getOrElse("").contains("text/html")) {
                val pattern = """\{.*\}""".r
                val resp = pattern.findFirstIn(response.body.string)
                Try(resp.get) match {
                  case Success(x) => x
                  case Failure(exception) =>
                    logger.warn("Could not extract JSON from HTML")
                    logger.debug("Replacing body: {} with empty JSON.", exception.getMessage)
                    "{}"
                }
              } else if (response.body.string.isEmpty) {
                logger.debug("Response body is empty.")
                "{}"
              } else response.body.string, response.charset)
          }
      })
      // check for OX Exceptions
      .check(jsonPath("$[?(@.categories=='ERROR')].code").notExists)
  }

  override def toString = s"$requestName ($requestMethod, $path/$relativeURL, $headers, $checks)"
}

object AbstractAJAXRequest {
  implicit def toActionBuilder(request: Request): HttpRequestActionBuilder =
    new HttpRequestActionBuilder(request.buildHttpRequest())
}