/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.request

import com.openexchange.testing.performance.config.Configuration
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder
import org.slf4j.{ Logger, LoggerFactory }

/**
 * [[Request]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
trait Request extends Configuration {
  lazy val logger: Logger = LoggerFactory.getLogger(getClass)

  /**
   * The name of the request
   *
   * @example Login
   */
  lazy val requestName: String = getClass.getSimpleName

  /**
   * The request method of the request
   *
   * @example GET, PUT, POST
   */
  val requestMethod: String

  /**
   * The path of the request.
   */
  val path: String

  /**
   * The relative path
   *
   * @example /ajax/login?action=login
   */
  val relativeURL: String

  /**
   * The request headers.
   */
  lazy val headers: Map[String, String] = Map.empty[String, String]

  /**
   * Optional HTTP checks.
   */
  lazy val checks: Seq[HttpCheck] = Seq.empty[HttpCheck]

  def buildHttpRequest(): HttpRequestBuilder = {
    http(requestName)
      .httpRequest(requestMethod, s"$path$relativeURL")
      .headers(headers)
      .check(checks: _*)
  }

}