/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.utils

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.DateTimeHelper._
import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.groupware.contacts.ContactData

import scala.util.Random

/**
 * [[ContactGenerator]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object ContactGenerator {
  def apply(folderId: String): ContactData = new ContactGenerator(folderId).contactDataIterator.next()
}

class ContactGenerator(folderId: String) {
  val tlrandom: ThreadLocalRandom = ThreadLocalRandom.current()
  val random: Random = Random

  val contactDataIterator: Iterator[ContactData] = Iterator.continually(ContactData(
    folder_id = folderId,
    first_name = Some(faker.name().firstName()),
    last_name = Some(faker.name().lastName()),
    suffix = Some(faker.name().suffix()),
    title = Some(faker.name().prefix),
    birthday = Some(DateTimeHelper.birthday()),
    anniversary = Some(DateTimeHelper.birthday()),
    number_of_children = Some(tlrandom.nextInt(10)),
    spouse_name = Some(faker.name().firstName()),
    url = Some("http://example.com/" + random.alphanumeric.take(8).mkString),
    company = Some(faker.company().name()),
    department = Some(faker.commerce().department()),
    position = Some(faker.name().title),
    email1 = Some(faker.internet().emailAddress(faker.lorem().word())),
    email2 = Some(faker.internet().emailAddress(faker.lorem().word())),
    email3 = Some(faker.internet().emailAddress(faker.lorem().word())),
    instant_messenger1 = Some(random.alphanumeric.take(16).mkString),
    instant_messenger2 = Some(random.alphanumeric.take(8).mkString),
    telephone_home1 = Some(faker.phoneNumber().phoneNumber),
    telephone_home2 = Some(faker.phoneNumber().phoneNumber),
    telephone_business1 = Some(faker.phoneNumber().phoneNumber),
    telephone_business2 = Some(faker.phoneNumber().phoneNumber),
    fax_business = Some(faker.phoneNumber().phoneNumber),
    telephone_callback = Some(faker.phoneNumber().phoneNumber),
    telephone_car = Some(faker.phoneNumber().phoneNumber),
    fax_home = Some(faker.phoneNumber().phoneNumber),
    cellular_telephone1 = Some(faker.phoneNumber().phoneNumber),
    cellular_telephone2 = Some(faker.phoneNumber().phoneNumber),
    telephone_other = Some(faker.phoneNumber().phoneNumber),
    fax_other = Some(faker.phoneNumber().phoneNumber),
    telephone_assistant = Some(faker.phoneNumber().phoneNumber),
    telephone_company = Some(faker.phoneNumber().phoneNumber),
    street_home = Some(faker.address().streetName()),
    postal_code_home = Some(tlrandom.nextInt(10000, 99999)),
    city_home = Some(faker.address().city),
    state_home = Some(faker.address().state),
    country_home = Some(faker.address().country),
    street_business = Some(faker.address().streetName()),
    postal_code_business = Some(tlrandom.nextInt(10000, 99999)),
    city_business = Some(faker.address().city),
    state_business = Some(faker.address().state),
    country_business = Some(faker.address().country),
    street_other = Some(faker.address().streetName()),
    city_other = Some(faker.address().city),
    postal_code_other = Some(tlrandom.nextInt(10000, 99999)),
    country_other = Some(faker.address().state),
    state_other = Some(faker.address().country)))

}
