/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.utils

import java.math.BigInteger
import java.security.MessageDigest
import javax.crypto.Mac
import javax.crypto.spec.SecretKeySpec

import org.apache.commons.codec.binary.Base64
import org.apache.commons.codec.digest.DigestUtils

/**
 * [[CryptoHelper]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
object CryptoHelper {

  def hmacSha1(value: String, key: String): String = {
    val keyBytes: Array[Byte] = key.getBytes("utf-8")
    val signingKey: SecretKeySpec = new SecretKeySpec(keyBytes, "HmacSHA1")
    val mac: Mac = Mac.getInstance("HmacSHA1")
    mac.init(signingKey)
    val rawHmac: Array[Byte] = mac.doFinal(value.getBytes("utf-8"))
    new String(Base64.encodeBase64(rawHmac), "utf-8")
  }

  def sha256(value: String): String = {
    return String.format("%064x", new BigInteger(1, MessageDigest.getInstance("SHA-256").digest(value.getBytes("UTF-8"))))
  }

}
