/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.utils

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.config.Configuration
import com.openexchange.testing.performance.groupware.mail.{ Mail, MailAttachment, MailData, MailSecurity }
import io.gatling.commons.util.Io
import io.gatling.core.feeder.Feeder

import scala.io.Source

/**
 * [[MailGenerator]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object MailGenerator extends Configuration {
  private val random: ThreadLocalRandom = ThreadLocalRandom.current()

  val lorem: String = settings.simulation.data.locale match {
    case "ja" => "lorem_jp.txt"
    case _    => "lorem.txt"
  }

  lazy val mailContent: String = Io.withSource(Source.fromFile(getClass.getResource(s"/bodies/$lorem").getPath, "UTF-8")) {
    _.mkString
  }

  def generateRandomOX6Mail(
    firstName:   String,
    lastName:    String,
    mailAddress: String) = Mail(MailData(
    from = Array(Array[String](s"$firstName $lastName", mailAddress)),
    to = Array(Array[String](s"$firstName $lastName", mailAddress)),
    subject = java.util.UUID.randomUUID().toString,
    priority = ThreadLocalRandom.current().nextInt(0, 6),
    attachments = Some(Array(MailAttachment(content_type = "ALTERNATIVE", content = mailContent))),
    dataSources = Some(Array.empty[String])))

  def generateRandomAppSuiteMail(
    firstName:   String,
    lastName:    String,
    mailAddress: String) = Mail(MailData(
    from = Array(Array[String](s"$firstName $lastName", mailAddress)),
    to = Array(Array[String](s"$firstName $lastName", mailAddress)),
    subject = java.util.UUID.randomUUID().toString,
    priority = ThreadLocalRandom.current().nextInt(0, 6),
    attachments = Some(Array(MailAttachment(content_type = "text/html", content = mailContent))),
    vcard = Some(ThreadLocalRandom.current().nextInt(0, 2)),
    contacts_ids = Some(Array.empty[String]),
    infostore_ids = Some(Array.empty[String]),
    dataSources = Some(Array.empty[String])))

  def generateRandomSecureAppSuiteMail(
    firstName:   String,
    lastName:    String,
    mailAddress: String,
    enrcypt:     Boolean,
    pgpInline:   Boolean,
    sign:        Boolean) = Mail(MailData(
    from = Array(Array[String](s"$firstName $lastName", mailAddress)),
    to = Array(Array[String](s"$firstName $lastName", mailAddress)),
    subject = java.util.UUID.randomUUID().toString,
    priority = ThreadLocalRandom.current().nextInt(0, 6),
    security = Some(MailSecurity(enrcypt, pgpInline, sign)),
    attachments = Some(Array(MailAttachment(content_type = "text/html", content = mailContent))),
    vcard = Some(ThreadLocalRandom.current().nextInt(0, 2)),
    contacts_ids = Some(Array.empty[String]),
    infostore_ids = Some(Array.empty[String]),
    dataSources = Some(Array.empty[String])))

  def generateAppSuiteMail(
    firstName:   String,
    lastName:    String,
    mailAddress: String,
    security:    Option[MailSecurity] = None) = Mail(MailData(
    from = Array(Array[String](s"$firstName $lastName", mailAddress)),
    to = Array(Array[String](s"$firstName $lastName", mailAddress)),
    subject = java.util.UUID.randomUUID().toString,
    priority = ThreadLocalRandom.current().nextInt(0, 6),
    security = security,
    attachments = Some(Array(MailAttachment(content_type = "text/html", content = mailContent))),
    vcard = Some(ThreadLocalRandom.current().nextInt(0, 2)),
    contacts_ids = Some(Array.empty[String]),
    infostore_ids = Some(Array.empty[String]),
    dataSources = Some(Array.empty[String])))

  val promoMailContentFeeder: Feeder[Any] = Iterator.continually(
    Map(
      "RND_SENTENCE" -> faker.lorem().sentence(random.nextInt(10)),
      "RND_STRING" -> faker.lorem().word(),
      "RND_EMAIL" -> faker.internet().emailAddress(),
      "RND_GOT_HOUSE" -> faker.gameOfThrones().house(),
      "RND_LINK" -> faker.internet().image()))
}
