/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.utils

import java.awt.image.BufferedImage
import java.io.{ ByteArrayOutputStream, IOException }
import java.util.concurrent.ThreadLocalRandom
import javax.imageio.ImageIO

import scala.language.implicitConversions

/**
 * [[RandomImgFaker]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class RandomImgFaker(width: Int, height: Int, format: String) {
  val random: ThreadLocalRandom = ThreadLocalRandom.current()
  val baos: ByteArrayOutputStream = new ByteArrayOutputStream()
  var img: Option[BufferedImage] = None

  format match {
    case "jpg" | "jpeg" => img = Some(new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR))
    case "png"          => img = Some(new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB))
    case "bmp"          => img = Some(new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB))
    case _              => throw new IllegalArgumentException("File format not supported.")
  }

  for (y <- 0 until height; x <- 0 until width) yield {
    val a: Int = random.nextDouble() * 256
    val r: Int = random.nextDouble() * 256
    val g: Int = random.nextDouble() * 256
    val b: Int = random.nextDouble() * 256

    val p: Int = (a << 24) | (r << 16) | (g << 8) | b

    img.get.setRGB(x, y, p)
  }

  try {
    ImageIO.write(img.get, format.toUpperCase, baos)
  } catch {
    case e: IOException => e.printStackTrace()
  }

  def getByteArray(): Array[Byte] = {
    baos.toByteArray
  }

  implicit def double2Int(x: Double): Int = x.toInt
}

object RandomImgFaker {
  def apply(width: Int = 640, height: Int = 640, format: String = "png"): Array[Byte] = new RandomImgFaker(width, height, format).getByteArray()
}
