/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.common.utils

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.DateTimeHelper._
import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.groupware.{ DetailedTaskData, Task, TaskAppointmentData }
import org.dmfs.rfc5545.Duration

import scala.util.Random

/**
 * [[TaskGenerator]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object TaskGenerator {
  private val tlRandom: ThreadLocalRandom = ThreadLocalRandom.current()

  def generateRandomTask(folderId: String): Task = {
    val startDate = DateTimeHelper.forward()
    val endDate = startDate.addDuration(Duration.parse(
      s"+PT${tlRandom.nextInt(0, 24)}H${tlRandom.nextInt(0, 60)}M"))

    Task(
      TaskAppointmentData(
        Some(folderId),
        title = Some(s"${faker.name().firstName()}-${faker.name().lastName()}-${faker.address().city()}"),
        start_date = startDate.getTimestamp,
        end_date = endDate.getTimestamp,
        note = Some(faker.lorem().paragraph())), DetailedTaskData(
        status = Some(tlRandom.nextInt(1, 5 + 1)),
        actual_costs = Some(BigDecimal(tlRandom.nextDouble(-99999999, 99999999))
          .setScale(2, BigDecimal.RoundingMode.HALF_UP).toDouble),
        target_costs = Some(BigDecimal(tlRandom.nextDouble(-99999999, 99999999))
          .setScale(2, BigDecimal.RoundingMode.HALF_UP).toDouble),
        priority = Some(tlRandom.nextInt(1, 4)),
        billing_information = Some(faker.address().streetAddress()),
        trip_meter = Some(Random.alphanumeric.take(24).mkString),
        companies = Some(faker.company().name())))
  }
}
