/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.config

/**
 * [[ConfigKeys]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object ConfigKeys {

  object middleware {
    val Protocol = "ox.middleware.protocol"
    val Host = "ox.middleware.host"
    val Port = "ox.middleware.port"
    val DispatcherPath = "ox.middleware.dispatcherPath"
    val Authentication = "ox.middleware.authentication"
    val BasicAuthUser = "ox.middleware.basicAuthUser"
    val BasicAuthPass = "ox.middleware.basicAuthPass"

    object ws {
      val Protocol = "ox.middleware.ws.protocol"
      val Host = "ox.middleware.ws.host"
      val Port = "ox.middleware.ws.port"
      val Path = "ox.middleware.ws.path"
    }
  }

  object appsuite {
    val UiWebPath = "ox.appsuite.uiWebPath"
    val Modules = "ox.appsuite.modules"
  }

  object facade {
    val Protocol = "ox.facade.protocol"
    val Host = "ox.facade.host"
    val Port = "ox.facade.port"
    val Path = "ox.facade.path"
    val Version = "ox.facade.version"
  }

  object mim {
    val Protocol = "ox.mim.protocol"
    val Host = "ox.mim.host"
    val Port = "ox.mim.port"
    val Path = "ox.mim.path"
    val Tenant = "ox.mim.tenant"
    val OngageSecret = "ox.mim.ongageSecret"
    val CampaignPlaceholderType = "ox.mim.campaignPlaceholderType"
    val CampaignPlaceholderAbsolute = "ox.mim.campaignPlaceholderAbsolute"
    val CampaignPlaceholderPercentage = "ox.mim.campaignPlaceholderPercentage"
    val CampaignTimeType = "ox.mim.campaignTimeType"
    val CampaignDurationTime = "ox.mim.campaignDurationTime"

    object cava {
      val N = "ox.mim.cava.n"
      val Offset = "ox.mim.cava.offset"
      val Onset = "ox.mim.cava.onset"
    }

    val CampaignCount = "ox.mim.campaignCount"
    val CampaignScenarioUsers = "ox.mim.campaignScenarioUsers"
    val TmpFilePath = "ox.mim.tmpFilePath"

    object aws {
      val AccessKey = "ox.mim.aws.accessKey"
      val SecretKey = "ox.mim.aws.secretKey"
      val Endpoint = "ox.mim.aws.endpoint"
      val Bucket = "ox.mim.aws.bucket"
      val Region = "ox.mim.aws.region"
      val PathStyleAccess = "ox.mdb.aws.pathStyleAccess"
      val Signer = "ox.mdb.aws.signer"
    }

    val CampaignUserPattern = "ox.mim.campaignUserPattern"
    val CassandraEndpoints = "ox.mim.cassandraEndpoints"
  }

  object mdb {
    val Protocol = "ox.mdb.protocol"
    val Host = "ox.mdb.host"
    val Port = "ox.mdb.port"
    val TenantList = "ox.mdb.tenantList"
    val TenantSecretKey = "ox.mdb.tenantSecretKey"
    val UsersPerTenant = "ox.mdb.usersPerTenant"
    val EspImportId = "ox.mdb.espImportId"
    val S3Import = "ox.mdb.s3Import"
    val DurationTime = "ox.mdb.durationTime"

    object aws {
      val AccessKey = "ox.mdb.aws.accessKey"
      val SecretKey = "ox.mdb.aws.secretKey"
      val Endpoint = "ox.mdb.aws.endpoint"
      val Bucket = "ox.mdb.aws.bucket"
      val Region = "ox.mdb.aws.region"
      val PathStyleAccess = "ox.mdb.aws.pathStyleAccess"
      val Signer = "ox.mdb.aws.signer"
    }

    val tmpFilePath = "ox.mdb.tmpFilePath"
  }

  object soap {
    val StartId = "ox.soap.startId"
    val StartName = "ox.soap.startName"
    val EndId = "ox.soap.endId"
    val MaxQuota = "ox.soap.maxQuota"
    val Adminmaster = "ox.soap.adminmaster"
    val Admin = "ox.soap.admin"
    val Masterpassword = "ox.soap.masterpassword"
    val Adminpassword = "ox.soap.adminpassword"
    val Userpassword = "ox.soap.userpassword"
    val ImapServer = "ox.soap.imapServer"
    val SmtpServer = "ox.soap.smtpServer"
    val Reseller = "ox.soap.reseller"
    val AccessCombinationName = "ox.soap.accessCombinationName"
    val FilestoreId = "ox.soap.filestoreId"
    val WriteDBPoolId = "ox.soap.writeDBPoolId"
    val SchemaFeeder = "ox.soap.schemaFeeder"
    val MissingFeeder = "ox.soap.missingFeeder"
    val CreateContext = "ox.soap.createContext"
    val UsersPerContext = "ox.soap.usersPerContext"
    val ContextFeeder = "ox.soap.contextFeeder"
    val UserFeeder = "ox.soap.userFeeder"
    val CapabilitiesToAdd = "ox.soap.capabilitiesToAdd"
    val CapabilitiesToRemove = "ox.soap.capabilitiesToRemove"
    val CapabilitiesToDrop = "ox.soap.capabilitiesToDrop"
  }

  object simulation {
    val UserFeeder = "ox.simulation.userFeeder"
    val Type = "ox.simulation.type"
    val UserAgent = "ox.simulation.userAgent"

    object assertions {
      val SuccessfulRequests = "ox.simulation.assertions.successfulRequests"
    }

    object data {
      val KeepData = "ox.simulation.data.keepData"
      val Locale = "ox.simulation.data.locale"

      object calendar {
        val NumberOfAppointments = "ox.simulation.data.calendar.numberOfAppointments"
        val NumberOfRecurringAppointments = "ox.simulation.data.calendar.numberOfRecurringAppointments"
      }

      object contacts {
        val NumberOfContacts = "ox.simulation.data.contacts.numberOfContacts"
      }

      object drive {
        val NumberOfFiles = "ox.simulation.data.drive.numberOfFiles"
        val NumberOfFolder = "ox.simulation.data.drive.numberOfFolder"
        val MaxFileSize = "ox.simulation.data.drive.maxFileSize"
      }

      object mail {
        val NumberOfMails = "ox.simulation.data.mail.numberOfMails"
        val ConversationView = "ox.simulation.data.mail.conversationView"
      }

      object probabilities {
        val NonIdle = "ox.simulation.data.probabilities.nonIdle"
        val Mail = "ox.simulation.data.probabilities.mail"
        val Calendar = "ox.simulation.data.probabilities.calendar"
        val Contacts = "ox.simulation.data.probabilities.contacts"
        val Tasks = "ox.simulation.data.probabilities.tasks"
        val Drive = "ox.simulation.data.probabilities.drive"
      }

      object user {
        val Domain = "ox.simulation.data.user.domain"
      }

      object tasks {
        val NumberOfTasks = "ox.simulation.data.tasks.numberOfTasks"
      }

    }

    object injection {
      val DurationTime = "ox.simulation.injection.durationTime"
      val Rampup = "ox.simulation.injection.rampup"
      val Users = "ox.simulation.injection.users"
      val UsersPerSec = "ox.simulation.injection.usersPerSec"
      val LoopTime = "ox.simulation.injection.loopTime"
      val RepeatCounter = "ox.simulation.injection.repeatCounter"
    }
  }

  object metrics {
    val Protocol = "ox.metrics.protocol"
    val Host = "ox.metrics.host"
    val Port = "ox.metrics.port"
    val DB = "ox.metrics.db"
  }
}
