/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.config

import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.protocol.HttpProtocolBuilder

/**
 * [[HTTPConfig]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object HTTPConfig extends Configuration {

  val appsuiteConf: HttpProtocolBuilder = http
    .baseURL(settings.middleware.baseUrl)
    .wsBaseURL(settings.middleware.ws.baseUrl)
    .acceptCharsetHeader("charset=UTF-8")
    .acceptLanguageHeader("en-US,en;q=0.5")
    .acceptEncodingHeader("gzip, deflate")
    .connectionHeader("keep-alive")
    .userAgentHeader(_ => s"${settings.simulation.userAgent}/${getClass.getSimpleName}")
    .disableCaching
    .disableWarmUp
    .disableFollowRedirect

  val mimConf: HttpProtocolBuilder = http
    .baseURL(settings.mim.baseUrl)
    .acceptEncodingHeader("gzip, deflate")
    .connectionHeader("keep-alive")
    .userAgentHeader(_ => s"${settings.simulation.userAgent}/${getClass.getSimpleName}")
    .disableCaching
    .disableWarmUp
    .shareConnections

  val mdbConf: HttpProtocolBuilder = http
    .baseURL(settings.mdb.baseUrl)
    .acceptEncodingHeader("gzip, deflate")
    .connectionHeader("keep-alive")
    .userAgentHeader(_ => s"${settings.simulation.userAgent}/${getClass.getSimpleName}")
    .disableCaching
    .disableWarmUp
    .shareConnections

  val facadeConf: HttpProtocolBuilder = http
    .baseURL(settings.facade.baseUrl)
    .acceptHeader("application/json")
    .acceptCharsetHeader("charset=UTF-8")
    .acceptLanguageHeader("en-US")
    .acceptEncodingHeader("gzip, deflate")
    .connectionHeader("keep-alive")
    .userAgentHeader(_ => s"OpenXchange.Gatling.Mail/2.0.1 (OS: unknown; device: unknown)")
    .disableCaching
    .disableWarmUp
    .disableFollowRedirect

}
