/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware

import com.fasterxml.jackson.annotation.{ JsonProperty, JsonUnwrapped }

/**
 * [[Appointment]] object
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @param detailed_appointment_data see [[DetailedAppointmentData]]
 * @param task_appointment_data see TaskAppointmentData
 */
case class Appointment(
    @JsonUnwrapped task_appointment_data:     TaskAppointmentData,
    @JsonUnwrapped detailed_appointment_data: DetailedAppointmentData)

/**
 * [[DetailedAppointmentData]]
 *
 * @param recurrence_id Object ID of the entire appointment sequence.
 * @param recurrence_position 1-based position of an individual appointment in a sequence.
 *                            Present if and only if recurrence_type > 0.
 * @param location Location.
 * @param full_time True if the appointment is a whole day appointment, not present otherwise.
 * @param shown_as Describes, how this appointment appears in availability queries.
 * @param timezone Timezone.
 * @param ignore_conflicts Ignore soft conflicts for the new or modified appointment.
 */
case class DetailedAppointmentData(
    recurrence_id:       Option[Int] = None,
    recurrence_position: Option[Int] = None,
    //recurrence_date_position TODO Date
    //change_exceptions TODO Array
    //delete_exceptions TODO Array
    location:  Option[String]  = None,
    full_time: Option[Boolean] = None,
    shown_as:  Option[Int]     = None,
    timezone:  Option[String]  = None,
    //recurrence_start TODO Date
    ignore_conflicts: Option[Boolean] = None)

/**
 * [[UserParticipantObject]]
 *
 * @param id User ID. Confirming for other users only works for appointments and not for tasks.
 * @param display_name Displayable name of the participant.
 * @param confirmation Confirmation status: {{{0 -> none, 1 -> accepted, 2 -> declined, 3 -> tentative}}}
 * @param confirmationmessage Confirm Message of the participant.
 */
case class UserParticipantObject(
    id:                  Option[Int]    = None,
    display_name:        Option[String] = None,
    confirmation:        Option[Int]    = None,
    confirmationmessage: Option[String] = None)

/**
 * [[ConfirmingParticipant]]
 *
 * @param typ Type of participant: {{{1 -> user, 5 -> external user}}}
 * @param mail Email address of external participant.
 * @param display_name Display name of external participant.
 * @param status Status: {{{0 -> none, 1 -> accepted, 2 -> declined, 3 -> tentative}}}
 */
case class ConfirmingParticipant(
    @JsonProperty("type") typ: Option[Int]    = None,
    mail:                      Option[String] = None,
    display_name:              Option[String] = None,
    status:                    Option[Int]    = None)