/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware

/**
 * [[BaseObject]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
abstract class BaseObject {
  /** Object ID */
  val id: Option[String] = None

  /** User ID of the user who created this object. */
  val created_by: Option[String] = None

  /** User ID of the user who last modified this object. */
  val modified_by: Option[String] = None

  /** Date and time of creation. */
  val creation_date: Option[Long] = None

  /** Date and time of the last modification. */
  val last_modified: Option[Long] = None

  /** Object ID of the parent folder. */
  val folder_id: Option[String] = None

  /**
   * String containing comma separated the categories. Order is preserved. Changing the
   * order counts as modification of the object. Not present in folder objects.
   */
  val categories: Option[String] = None

  /**
   * Overrides folder permissions in shared private folders: When true, this object is
   * not visible to anyone except the owner. Not present in folder objects.
   */
  val private_flag: Option[Boolean] = None

  /**
   * Color number used by Outlook to label the object. The assignment of colors to
   * numbers is arbitrary and specified by the client. The numbers are integer numbers
   * between 0 and 10 (inclusive). Not present in folder objects.
   */
  val color_label: Option[Int] = None

  /** Number of attachments */
  val number_of_attachments: Option[Int] = None

}