/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware

import com.fasterxml.jackson.annotation.JsonUnwrapped

/**
 * [[ShareManagement]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
abstract class ShareManagement

/**
 * [[ShareTarget]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param module The folder's module name, i.e. one of "tasks", "calendar", "contacts", "infostore"
 * @param folder The folder identifier.
 * @param item The object identifier, in case the share targets a single item.
 */
case class ShareTarget(module: String, folder: String, item: Option[String] = None) extends ShareManagement

/**
 * [[ShareLink]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param url The link to the share (read-only).
 * @param entity The identifier of the anonymous user entity for the share (read-only).
 * @param is_new Whether the share link is new, i.e. it has been created by the getLink-request,
 *               or if it already existed (read-only).
 * @param expiry_date The end date / expiration time after which the share link is no longer accessible.
 * @param password An additional secret / pin number an anonymous user needs to enter when accessing the share.
 * @param meta Arbitrary JSON data saved along with the share as specified by client.
 */
case class ShareLink(
    url:         Option[String]  = None,
    entity:      Option[Int]     = None,
    is_new:      Option[Boolean] = None,
    expiry_date: Option[Long]    = None,
    password:    Option[String]  = None,
    meta:        Option[String]  = None) extends ShareManagement

case class ShareTargetLink(
    @JsonUnwrapped shareTarget: ShareTarget,
    @JsonUnwrapped shareLink:   ShareLink) extends ShareManagement