/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware.chronos

import com.fasterxml.jackson.annotation.JsonProperty
import org.dmfs.rfc5545.DateTime

/**
 * [[EventData]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
case class EventData(
    startDate:                    Option[DateTimeData]             = None,
    endDate:                      Option[DateTimeData]             = None,
    created:                      Option[Long]                     = None,
    timestamp:                    Option[Long]                     = None,
    categories:                   Option[Array[String]]            = None,
    filename:                     Option[String]                   = None,
    uid:                          Option[String]                   = None,
    description:                  Option[String]                   = None,
    attendees:                    Option[Array[Attendee]]          = None,
    alarms:                       Option[Array[Alarm]]             = None,
    lastModified:                 Option[Long]                     = None,
    modifiedBy:                   Option[CalendarUser]             = None,
    summary:                      Option[String]                   = None,
    sequence:                     Option[Int]                      = None,
    id:                           Option[String]                   = None,
    @JsonProperty("class") clazz: Option[String]                   = None,
    organizer:                    Option[CalendarUser]             = None,
    transp:                       Option[String]                   = None,
    color:                        Option[String]                   = None,
    folder:                       Option[String]                   = None,
    createdBy:                    Option[CalendarUser]             = None,
    changeExceptionDates:         Option[Array[String]]            = None,
    deleteExceptionDates:         Option[Array[String]]            = None,
    recurrenceId:                 Option[String]                   = None,
    calendarUser:                 Option[CalendarUser]             = None,
    rrule:                        Option[String]                   = None,
    attachments:                  Option[Array[ChronosAttachment]] = None,
    extendedProperties:           Option[String]                   = None,
    geo:                          Option[Geo]                      = None,
    url:                          Option[String]                   = None,
    seriesId:                     Option[String]                   = None,
    status:                       Option[String]                   = None,
    location:                     Option[String]                   = None,
    flags:                        Option[Array[String]]            = None,
    relatedTo:                    Option[String]                   = None)

case class Attendee(
    cuType:   Option[String]        = None,
    folder:   Option[String]        = None,
    partStat: Option[String]        = None,
    role:     Option[String]        = None,
    comment:  Option[String]        = None,
    rsvp:     Option[Boolean]       = None,
    member:   Option[Array[String]] = None,
    contact:  Option[Contact]       = None,
    resource: Option[Resource]      = None,
    group:    Option[Group]         = None,
    uri:      Option[String]        = None,
    cn:       Option[String]        = None,
    email:    Option[String]        = None,
    entity:   Option[Int]           = None)

case class Alarm(
    uid:                Option[String]            = None,
    action:             Option[String]            = None,
    acknowledged:       Option[Int]               = None,
    trigger:            Option[Trigger]           = None,
    attachments:        Option[ChronosAttachment] = None,
    attendees:          Option[Attendee]          = None,
    summary:            Option[String]            = None,
    description:        Option[String]            = None,
    extendedProperties: Option[String]            = None)

case class CalendarUser(
    uri:    Option[String] = None,
    cn:     Option[String] = None,
    email:  Option[String] = None,
    entity: Option[Int]    = None)

case class ChronosAttachment(
    filename:  Option[String] = None,
    fmtType:   Option[String] = None,
    size:      Option[Int]    = None,
    created:   Option[Int]    = None,
    managedId: Option[Int]    = None,
    uri:       Option[String])

case class Geo(
    lat:  Option[Numeric[Number]] = None,
    long: Option[Numeric[Number]] = None)

case class Contact(
    first_name:   Option[String] = None,
    last_name:    Option[String] = None,
    title:        Option[String] = None,
    display_name: Option[String] = None,
    image1_url:   Option[String] = None)

case class Resource(
    id:                Option[Int]     = None,
    display_name:      Option[String]  = None,
    name:              Option[String]  = None,
    mailaddress:       Option[String]  = None,
    availability:      Option[Boolean] = None,
    description:       Option[String]  = None,
    last_modified:     Option[String]  = None,
    last_modified_utc: Option[String])

case class Group(
    id:                Option[Int]        = None,
    display_name:      Option[String]     = None,
    name:              Option[String]     = None,
    members:           Option[Array[Int]] = None,
    last_modified_utc: Option[String]     = None)

case class Trigger(
    duration: Option[String]   = None,
    related:  Option[String]   = None,
    dateTime: Option[DateTime] = None)