/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware.contacts

/**
 * [[SearchContact]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
abstract class SearchContact

/**
 * [[SearchContacts]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param pattern Search pattern to find contacts.
 * @param startletter Search contacts with the given startletter.
 * @param folder If a list of folder identifiers or at least a single folder identifier is given, only in that
 *               folders will be searched for contacts.
 */
case class SearchContacts(
    pattern:     Option[String]     = None,
    startletter: Option[String]     = None,
    folder:      Option[Array[Int]]) extends SearchContact

/**
 * [[SearchContactsAlternative]]
 *
 * @param last_name Searches contacts where the last name match with the given last name.
 * @param first_name Searches contacts where the first name match with the given first name.
 * @param display_name Searches contacts where the display name match with the given display name.
 * @param email1 Searches contacts where the email1 address match with the given search pattern.
 * @param email2 Searches contacts where the email2 address match with the given search pattern.
 * @param email3 Searches contacts where the email3 address match with the given search pattern.
 * @param company Searches contacts where the company match with the given search pattern.
 * @param categories Searches contacts where the categories match with the given search pattern.
 * @param orSearch  If set to true, a contact is returned if any specified pattern matches at the start
 *                  of the corresponding field.
 * @param emailAutoComplete If set to true, results are guaranteed to contain at least one email adress
 *                          and the search is performed as if orSearch were set to true.
 * @param exactMatch If set to true, contacts are returned where the specified patterns match the corresponding
 *                   fields exactly.
 */
case class SearchContactsAlternative(
    last_name:         Option[String]  = None,
    first_name:        Option[String]  = None,
    display_name:      Option[String]  = None,
    email1:            Option[String]  = None,
    email2:            Option[String]  = None,
    email3:            Option[String]  = None,
    company:           Option[String]  = None,
    categories:        Option[String]  = None,
    orSearch:          Option[Boolean] = None,
    emailAutoComplete: Option[Boolean] = None,
    exactMatch:        Option[Boolean] = None) extends SearchContact