/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.groupware.infostore

import com.fasterxml.jackson.annotation.{ JsonProperty, JsonUnwrapped }
import com.openexchange.testing.performance.groupware.folders.{ ExtendedPermission, Permission }

/**
 * [[InfoItemBody]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
case class InfoItemBody(@JsonUnwrapped file: InfoItemData, @JsonUnwrapped notification: Option[InfoItemBodyNotification] = None)

case class InfoItemData(
    object_permissions:                                                                          Option[Array[Permission]]         = None,
    shareable:                                                                                   Option[Boolean]                   = None,
    title:                                                                                       Option[String]                    = None,
    url:                                                                                         Option[String]                    = None,
    filename:                                                                                    Option[String]                    = None,
    file_mimetype:                                                                               Option[String]                    = None,
    file_size:                                                                                   Option[Int]                       = None,
    version:                                                                                     Option[String]                    = None,
    description:                                                                                 Option[String]                    = None,
    locked_until:                                                                                Option[Int]                       = None,
    file_md5sum:                                                                                 Option[String]                    = None,
    version_comment:                                                                             Option[String]                    = None,
    current_version:                                                                             Option[Boolean]                   = None,
    number_of_versions:                                                                          Option[Int]                       = None,
    @JsonProperty("com.openexchange.share.extendedObjectPermissions") extendedObjectPermissions: Option[Array[ExtendedPermission]] = None,
    @JsonProperty("com.openexchange.realtime.resourceID") resourceID:                            Option[String]                    = None,
    @JsonProperty("com.openexchange.file.storage.mail.mailMetadata") mailMetadata:               Option[Any]                       = None,
    id:                                                                                          Option[String]                    = None,
    created_by:                                                                                  Option[String]                    = None,
    modified_by:                                                                                 Option[String]                    = None,
    creation_date:                                                                               Option[String]                    = None,
    last_modified:                                                                               Option[Int]                       = None,
    folder_id:                                                                                   String,
    categories:                                                                                  Option[String]                    = None,
    private_flag:                                                                                Option[Boolean]                   = None,
    color_label:                                                                                 Option[Int]                       = None,
    number_of_attachments:                                                                       Option[Int]                       = None,
    lastModifiedOfNewestAttachmentUTC:                                                           Option[Int]                       = None)

case class InfoItemBodyNotification(transport: Option[String] = None, message: Option[String] = None)
