/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.drive

import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.Predef._

/**
 * [[DriveAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object DriveAPI {
  private val MODULE_NAME = "Drive"

  /**
   *
   * @param rootFolderID
   * @param original_versions
   * @param client_versions
   * @return
   */
  def synchronizeFolders(
    rootFolderID:      String,
    original_versions: Option[Array[DirectoryVersion]] = None,
    client_versions:   Array[DirectoryVersion]) = group(MODULE_NAME) {
    exec(new SynchronizeFolders(
      rootFolderId = rootFolderID,
      jsonBody = SynchronizeFoldersJSONData(originalVersions = original_versions, clientVersions = client_versions.toArray)))
  }

  /**
   *
   * @param rootFolderID
   * @param folderPath
   * @param original_versions
   * @param client_versions
   * @return
   */
  def synchronizeFiles(
    rootFolderID:      String,
    folderPath:        String,
    original_versions: Option[Array[FileVersion]] = Some(Array.empty[FileVersion]),
    client_versions:   Array[FileVersion]) = group(MODULE_NAME) {
    exec(new SynchronizeFiles(
      rootFolderId = rootFolderID,
      path = folderPath,
      jsonBody = SynchronizeFoldersJSONData(originalVersions = original_versions, clientVersions = client_versions.toArray))
      .buildHttpRequest()
      .check(jsonPath("$.data[*].action").ofType[String].findAll.saveAs("driveActions"))
      .check(jsonPath("$.data[*].path").ofType[String].findAll.saveAs("drivePaths"))
      .check(jsonPath("$.data[*].newVersion").ofType[String].findAll.saveAs("newVersionsName")))
      .exec(session => {
        val driveActions = session("driveActions").as[Seq[String]]
        val drivePaths = session("drivePaths").as[Seq[String]]
        val newVersions = session("newVersionsName").as[Seq[String]].map(x => JSONUtil().fromJson[FileVersion](x))

        /**
         * Zip them all together:
         * SyncFilesResponse(upload,/gatling_2486930397162556694,FileVersion(Some(PREFIX2700084125684738960.txt),Some(09d63b1ae1e17d6f84719b9ab9e0daa5))
         */
        val response = (driveActions, drivePaths, newVersions).zipped.map(SynchronizeFilesResponse)

        // Save SyncFilesResponse into the session
        session.set("syncFilesResponse", response)
      })

  }

  def upload(rootFolderId: String, syncFilesResponse: Expression[SynchronizeFilesResponse]) = group(MODULE_NAME) {
    exec(new UploadRequest(rootFolderId, syncFilesResponse))
  }

  def getDriveQuota(rootFolderID: String) = group(MODULE_NAME) {
    exec(new GetQuota(rootFolderID))
  }
}
