/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.drive

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[SynchronizeFolders]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param rootFolderId The ID of the referenced root folder on the server.
 * @param apiVersion   The API version that the client is using. If not set, the initial
 *                     version ''0'' is assumed.
 * @param jsonBody     A JSON object containing two JSON arrays named ''clientVersions'' and ''originalVersions''.
 */
class SynchronizeFolders(
    rootFolderId: Expression[String],
    apiVersion:   String                                       = "1",
    jsonBody:     SynchronizeFoldersJSONData[DirectoryVersion]) extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = s"${settings.middleware.path}drive?action=syncfolders"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.ROOT, rootFolderId)
      .queryParam("apiVersion", apiVersion)
      .body(StringBody(JSONUtil().toJson(jsonBody))).asJSON
  }
}
