/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.drive

import com.fasterxml.jackson.annotation.JsonProperty

/**
 * [[SynchronizeFoldersJSONData]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @tparam T
 */
case class SynchronizeFoldersJSONData[T](
    clientVersions:   Array[T],
    originalVersions: Option[Array[T]] = None)

/**
 * [[DirectoryVersion]] A directory is uniquely identified by it's full path,
 * relative to the root folder and the checksum of it's content.
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @param path The path of the directory, expressed as a lowercase hexadecimal number string,
 *             32 characters long.
 * @example ''f8cacac95379527cd4fa15f0cb782a09''
 */
case class DirectoryVersion(path: String, checksum: String)

/**
 * [[FileVersion]] A file in a directory is uniquely identified by it's filename and
 * the checksum of it's content.
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @param name The name of the file, including it's extension
 * @example ''test.doc''
 */
case class FileVersion(name: Option[String] = None, checksum: Option[String] = None)

/**
 * File pattern.
 *
 * @param path
 * @param name
 * @param patternType
 */
case class FilePattern(
    path:                              Option[String] = None,
    name:                              Option[String] = None,
    @JsonProperty("type") patternType: Option[String] = None)

/**
 * Directory pattern.
 *
 * @param path
 * @param patternType
 */
case class DirectoryPattern(
    path:                              Option[String] = None,
    @JsonProperty("type") patternType: Option[String] = None)
