/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.drive

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedAJAXRequest, RequestMethod, RequestParameters }
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[UploadRequest]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param rootFolderId      The ID of the referenced root folder on the server.
 * @param syncFilesResponse The [[SynchronizeFilesResponse]] JSON-Data object.
 * @param binary            Expected to be set to ''true'' to indicate the binary content.
 * @param apiVersion        The API version that the client is using.
 */
class UploadRequest(
    rootFolderId:      Expression[String],
    syncFilesResponse: Expression[SynchronizeFilesResponse],
    binary:            Boolean                              = true,
    apiVersion:        String                               = "1") extends AbstractAuthenticatedAJAXRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = s"${settings.middleware.path}drive?action=upload"

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam(RequestParameters.ROOT, rootFolderId)
      .queryParam("path", session => syncFilesResponse(session).get.path)
      .queryParam("newName", session => syncFilesResponse(session).get.fileVersion.name.get)
      .queryParam("newChecksum", session => syncFilesResponse(session).get.fileVersion.checksum.get)
      .queryParam("binary", binary)
      .queryParam("apiVersion", apiVersion)
      .body(RawFileBody(session => "src/test/resources/bodies/" + syncFilesResponse(session).get.path + "/" + syncFilesResponse(session).get.fileVersion.name.get))
  }
}
