/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.facade.folders

import com.openexchange.testing.performance.common.SessionConstants._
import io.gatling.core.Predef._
import io.gatling.core.session._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

/**
 * [[FoldersAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object FoldersAPI {
  private val moduleName = "Folders"

  def getAllMailFolders: ChainBuilder = group(moduleName) {
    exec(new MailFoldersRequest().buildHttpRequest()
      .check(jsonPath("$..folders[*].id").findAll.saveAs("folders"))
      // default mail folder identifier
      .check(jsonPath("$..standardFolders.inboxId").ofType[String].saveAs(FID_INBOX))
      .check(jsonPath("$..standardFolders.draftsId").ofType[String].saveAs(FID_DRAFTS))
      .check(jsonPath("$..standardFolders.sentId").ofType[String].saveAs(FID_SENT))
      .check(jsonPath("$..standardFolders.spamId").ofType[String].saveAs(FID_SPAM))
      .check(jsonPath("$..standardFolders.trashId").ofType[String].saveAs(FID_TRASH)))
  }

  def createFolder(parentFolderId: Expression[String], name: Expression[String]): ChainBuilder = group(moduleName) {
    exec(new CreateFolderRequest(parentFolderId, name).buildHttpRequest()
      .check(jsonPath("$.folderId").ofType[String].saveAs("createdFolder"))
      .check(jsonPath("$.name").exists))
      .exec(session => for {
        folders <- session.get("folders").validate[Vector[String]]
        createdFolder <- session.get("createdFolder").validate[String]
      } yield {
        session.set("folders", folders :+ createdFolder)
      })
  }

  def deleteFolder(folderId: Expression[String]): ChainBuilder = group(moduleName) {
    exec(new DeleteFolderRequest(folderId).buildHttpRequest())
      .exec(session => for {
        folders <- session.get("folders").validate[Vector[String]]
      } yield {
        session.set("folders", folders.filterNot(_ == folderId(session).get))
      })
  }

  def clearFolder(folderId: Expression[String]): ChainBuilder = group(moduleName) {
    exec(new ClearFolderRequest(folderId))
  }

}
