/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.facade.mails

import com.openexchange.testing.performance.common.request.{ AbstractAuthenticatedFacadeRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.request.builder.HttpRequestBuilder

abstract class SetFlag(folderId: Expression[String], mailId: Expression[String]) extends AbstractAuthenticatedFacadeRequest {
  override val requestMethod: String = RequestMethod.POST

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .body(StringBody(session => JSONUtil().toJson(Map(
        "folderId" -> folderId(session).get,
        "mailIds" -> Set(mailId(session).get))))).asJSON
  }
}

class SetAnsweredFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/answered"
}

class SetDeletedFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/deleted"
}

class SetDraftFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/draft"
}

class SetFlaggedFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/flagged"
}

class SetForwardedFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = s"/api-facade/v1/mails/flags/forwarded"
}

class SetUnreadFlag(folderId: Expression[String], mailId: Expression[String]) extends SetFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/unread"
}

abstract class ClearFlag(folderId: Expression[String], mailId: Expression[String]) extends AbstractAuthenticatedFacadeRequest {
  override val requestMethod: String = RequestMethod.DELETE

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .body(StringBody(session => JSONUtil().toJson(Map(
        "folderId" -> folderId(session).get,
        "mailIds" -> Set(mailId(session).get))))).asJSON
  }
}

class ClearAnsweredFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/answered"
}

class ClearDeletedFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/deleted"
}

class ClearDraftFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/draft"
}

class ClearFlaggedFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/flagged"
}

class ClearForwardedFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/forwarded"
}

class ClearUnreadFlag(folderId: Expression[String], mailId: Expression[String]) extends ClearFlag(folderId, mailId) {
  override val relativeURL: String = "/api-facade/v1/mails/flags/unread"
}
