/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.facade.mails

import io.gatling.core.Predef._
import io.gatling.core.session.Expression
import io.gatling.http.Predef._

/**
 * [[MailsAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object MailsAPI {
  private val moduleName = "Mails"

  def getStubs(folderId: Expression[String]) = group(moduleName) {
    exec(new StubsRequest(folderId).buildHttpRequest()
      .check(jsonPath("$.threads[*].mails[*].id")
        .findAll
        .transform(list => scala.util.Random.shuffle(list).take(20))
        .optional
        .saveAs("mailIds")))
  }

  def getEnvelopes(folderId: Expression[String], mailIds: Expression[Seq[String]]) = group(moduleName) {
    exec(new EnvelopesRequest(folderId, mailIds))
  }

  def complete(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new CompleteMailRequest(folderId, mailId))
  }

  def setAnsweredFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    println(mailId)
    exec(new SetAnsweredFlag(folderId, mailId))
  }

  def setDeletedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new SetDeletedFlag(folderId, mailId))
  }

  def setDraftFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new SetDraftFlag(folderId, mailId))
  }

  def setFlaggedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new SetFlaggedFlag(folderId, mailId))
  }

  def setForwardedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new SetForwardedFlag(folderId, mailId))
  }

  def setUnreadFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new SetUnreadFlag(folderId, mailId))
  }

  def clearAnsweredFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearAnsweredFlag(folderId, mailId))
  }

  def clearDeletedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearDeletedFlag(folderId, mailId))
  }

  def clearDraftFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearDraftFlag(folderId, mailId))
  }

  def clearFlaggedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearFlaggedFlag(folderId, mailId))
  }

  def clearForwardedFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearForwardedFlag(folderId, mailId))
  }

  def clearUnreadFlag(folderId: Expression[String], mailId: Expression[String]) = group(moduleName) {
    exec(new ClearUnreadFlag(folderId, mailId))
  }

  def setColor(folderId: Expression[String], mailIds: Expression[Set[String]], color: Expression[Int]) = group(moduleName) {
    exec(new SetColorRequest(folderId, mailIds, color))
      .doIf("${mailIdsToColor.exists()}") {
        exec(session => session.remove("mailIdsToColor"))
      }
  }

  def move(folderId: Expression[String], mailIds: Expression[Set[String]], targetFolderId: Expression[String]) = group(moduleName) {
    exec(new MoveMailsRequest(folderId, mailIds, targetFolderId))
      .doIf("${mailIdsToMove.exists()}") {
        exec(session => session.remove("mailIdsToMove"))
      }
      .doIf("${targetFolderId.exists()}") {
        exec(session => session.remove("targetFolderId"))
      }
  }

  def delete(folderId: Expression[String], mailIds: Expression[Set[String]]) = group(moduleName) {
    exec(new DeleteMailsRequest(folderId, mailIds))
      .doIf("${mailIdsToDelete.exists()}") {
        exec(session => session.remove("mailIdsToDelete"))
      }
  }

}
