/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.common.BaseSimulation
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.config.HTTPConfig
import com.openexchange.testing.performance.facade.authentication.AuthenticationAPI
import com.openexchange.testing.performance.facade.config.ConfigAPI
import com.openexchange.testing.performance.facade.folders.FoldersAPI
import com.openexchange.testing.performance.facade.mails.MailsAPI
import com.openexchange.testing.performance.facade.push.PushAPI
import com.openexchange.testing.performance.facade.users.UsersAPI
import com.openexchange.testing.performance.facade.version.VersionAPI
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder
import io.gatling.http.protocol.HttpProtocolBuilder

import scala.concurrent.duration.FiniteDuration

/**
 * [[PerformanceSim]]
 *
 * @author <a href="mailto:michael.koch@open-xchange.com">Michael Koch</a>
 */
class PerformanceSim extends BaseSimulation {

  override lazy val httpConf: HttpProtocolBuilder = HTTPConfig.facadeConf

  override def scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .during(new FiniteDuration(settings.simulation.injection.duration - (settings.simulation.injection.duration /
      settings.simulation.injection.rampup), TimeUnit.SECONDS), exitASAP = false) {
      feed(csv(settings.simulation.userFeeder).circular)
        .exec(AuthenticationAPI.login)
        .doIf(session => session.contains(SESSION_ID) && session(SESSION_ID).as[String].nonEmpty) {
          exec(PushAPI.subscribe("gcm", "1234567890abcdefghijklmnopqrstuvwxyz", Set("ox:mail:new")))
            .exec(VersionAPI.getVersion)
            .exec(UsersAPI.getCurrentUser)
            .exec(ConfigAPI.getConfig)
            .exec(FoldersAPI.getAllMailFolders)
            .exec(MailsAPI.getStubs("${OX_FID_INBOX}"))
            .doIf(session => session.contains("mailIds") && session("mailIds").as[Seq[String]].nonEmpty) {
              exec(MailsAPI.getEnvelopes("${OX_FID_INBOX}", "${mailIds}"))
                .exec(MailsAPI.complete("${OX_FID_INBOX}", "${mailIds.random()}"))
            }
            .exec(MailsAPI.getStubs(session => scala.util.Random.shuffle(session("folders").as[Vector[String]]).get(0)))
            .doIf(session => session.contains("mailIds") && session("mailIds").as[Seq[String]].nonEmpty) {
              exec(MailsAPI.getEnvelopes("${OX_FID_INBOX}", "${mailIds}"))
                .exec(MailsAPI.complete("${OX_FID_INBOX}", "${mailIds.random()}"))
            }
        }
    }
}
