/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import com.openexchange.testing.performance.ajax.folders.{ FoldersAPI => AjaxFoldersAPI }
import com.openexchange.testing.performance.common.BaseSimulation
import com.openexchange.testing.performance.common.SessionConstants._
import com.openexchange.testing.performance.common.utils.RandomHelper
import com.openexchange.testing.performance.config.HTTPConfig
import com.openexchange.testing.performance.facade.authentication.AuthenticationAPI
import com.openexchange.testing.performance.facade.config.ConfigAPI
import com.openexchange.testing.performance.facade.contacts.ContactsAPI
import com.openexchange.testing.performance.facade.folders.FoldersAPI
import com.openexchange.testing.performance.facade.mails.MailsAPI
import com.openexchange.testing.performance.facade.push.PushAPI
import com.openexchange.testing.performance.facade.users.UsersAPI
import com.openexchange.testing.performance.facade.version.VersionAPI
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder
import io.gatling.http.protocol.HttpProtocolBuilder

/**
 * [[SmokeSim]]
 *
 * @author <a href="mailto:michael.koch@open-xchange.com">Michael Koch</a>
 */
class SmokeSim extends BaseSimulation {

  override lazy val httpConf: HttpProtocolBuilder = HTTPConfig.facadeConf

  override def scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .feed(csv(settings.simulation.userFeeder).circular)

    .exec(AuthenticationAPI.login)

    .exec(VersionAPI.getVersion)

    .exec(ConfigAPI.getConfig)

    .exec(UsersAPI.getCurrentUser)

    .exec(FoldersAPI.getAllMailFolders)

    .exec(session => session.set("folderToCreate", scala.util.Random.alphanumeric.take(10).mkString))
    .exec(FoldersAPI.createFolder("${OX_FID_INBOX}", "${folderToCreate}"))
    .exec(FoldersAPI.clearFolder("${createdFolder}"))
    .exec(FoldersAPI.deleteFolder("${createdFolder}"))

    .exec(MailsAPI.getStubs("${OX_FID_INBOX}"))
    .exec(MailsAPI.getEnvelopes("${OX_FID_INBOX}", "${mailIds}"))
    .exec(MailsAPI.complete("${OX_FID_INBOX}", "${mailIds.random()}"))

    .exec(session => for {
      folderIds <- session("folders").validate[Seq[String]]
      mailIds <- session("mailIds").validate[Seq[String]]
    } yield {
      session.setAll(
        "targetFolderId" -> RandomHelper.getRandomElement(folderIds),
        "mailIdsToMove" -> Set(RandomHelper.getRandomElement(mailIds)))
    })
    .exec(MailsAPI.move("${OX_FID_INBOX}", "${mailIdsToMove}", "${targetFolderId}"))

    .exec(session => for {
      mailIds <- session("mailIds").validate[Seq[String]]
    } yield {
      session.set("mailIdsToDelete", Set(RandomHelper.getRandomElement(mailIds)))
    })
    .exec(MailsAPI.delete("${OX_FID_INBOX}", "${mailIdsToDelete}"))

    .exec(session => for {
      mailIds <- session("mailIds").validate[Seq[String]]
    } yield {
      session.set("mailIdToFlag", RandomHelper.getRandomElement(mailIds))
    })
    .exec(MailsAPI.setAnsweredFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.setDeletedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.setDraftFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.setFlaggedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.setForwardedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.setUnreadFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearAnsweredFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearDeletedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearDraftFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearFlaggedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearForwardedFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))
    .exec(MailsAPI.clearUnreadFlag("${OX_FID_INBOX}", "${mailIdToFlag}"))

    .exec(session => for {
      mailIds <- session("mailIds").validate[Seq[String]]
    } yield {
      session.set("mailIdsToColor", Set(RandomHelper.getRandomElement(mailIds)))
    })
    .exec(MailsAPI.setColor("${OX_FID_INBOX}", "${mailIdsToColor}", 3))

    .exec(AjaxFoldersAPI.clearFolders(Array($(FID_TRASH))))

    .exec(PushAPI.subscribe("gcm", "1234567890", Set("ox:mail:new")))
    .exec(PushAPI.unsubscribe("gcm", "1234567890"))

    .exec(ContactsAPI.getContacts)

    .exec(AuthenticationAPI.logout)
}
