/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.requests

import java.util.UUID

import com.openexchange.testing.performance.common.utils.FileHelper
import io.gatling.core.Predef._
import io.gatling.core.structure.ChainBuilder
import io.gatling.http.Predef._

import scala.util.Random

/**
 * [[ImageConverterAPI]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object ImageConverterAPI {
  private val moduleName = "ImageConverter"

  def getStatus(metrics: Boolean = false): ChainBuilder = group(moduleName) {
    exec(new GetStatus(metrics))
  }

  def getKeys(context: Option[String] = None): ChainBuilder = group(moduleName) {
    exec(new GetKeys(context))
  }

  def getKeyCount(context: Option[String] = None): ChainBuilder = group(moduleName) {
    exec(new GetKeyCount(context))
  }

  def clearImages(context: Option[String] = None): ChainBuilder = group(moduleName) {
    exec(new ClearImages(context))
  }

  def clearImagesByKey(imageKey: String = "${GetKeys.random()}"): ChainBuilder = group(moduleName) {
    exec(new ClearImagesByKey(imageKey))
  }

  def getMetadata(imageKey: String = "${GetKeys.random()}"): ChainBuilder = group(moduleName) {
    exec(new GetMetadata(imageKey))
  }

  def getImageAndMetadata(
    imageKey:      String         = "${GetKeys.random()}",
    requestFormat: String         = "auto",
    context:       Option[String] = None): ChainBuilder = group(moduleName) {
    exec(new GetImageAndMetadata(imageKey, requestFormat, context))
  }

  def cacheImage(
    imageMimeType: String         = Random.shuffle(Seq("png", "jpeg")).head,
    imageKey:      String         = UUID.randomUUID().toString,
    context:       Option[String] = None,
    fileSize:      Int            = 1024): ChainBuilder = group(moduleName) {
    exec(new CacheImage(imageKey, ByteArrayBodyPart(
      "sourceImage", FileHelper.getByteArray(imageMimeType, fileSize))
      .contentType(s"image/$imageMimeType")
      .fileName(s"$imageKey.$imageMimeType"),
      context))
  }

  def cacheAndGetImage(
    imageMimeType: String         = Random.shuffle(Seq("png", "jpeg")).head,
    imageKey:      String         = UUID.randomUUID().toString,
    context:       Option[String] = None,
    requestFormat: String         = "auto",
    fileSize:      Int            = 1024): ChainBuilder = group(moduleName) {
    exec(new CacheAndGetImage(imageKey, requestFormat,
      ByteArrayBodyPart(
        "sourceImage", FileHelper.getByteArray(imageMimeType, fileSize))
        .contentType(s"image/$imageMimeType")
        .fileName(s"$imageKey.$imageMimeType")))
  }

  def cacheAndGetImageAndMetadata(
    imageMimeType: String         = Random.shuffle(Seq("png", "jpeg")).head,
    imageKey:      String         = UUID.randomUUID().toString,
    context:       Option[String] = None,
    requestFormat: String         = "auto",
    fileSize:      Int            = 1024): ChainBuilder = group(moduleName) {
    exec(new CacheAndGetImageAndMetadata(imageKey, requestFormat,
      ByteArrayBodyPart(
        "sourceImage", FileHelper.getByteArray(imageMimeType, fileSize))
        .contentType(s"image/$imageMimeType")
        .fileName(s"$imageKey.$imageMimeType")))
  }
}
