/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mdb

import com.openexchange.testing.performance.common.request.{ AbstractMDBRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.http.Predef.jsonPath
import io.gatling.http.check._
import io.gatling.http.check.status.HttpStatusCheckBuilder

/**
 * [[GETUserImports]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
class GETUserImports(tenantId: String, importId: Option[String] = None) extends AbstractMDBRequest {

  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "/user-imports" + "/" + tenantId + "/" + importId.getOrElse("");
  override lazy val checks: Seq[HttpCheck] = Seq(GETUserImports.status, GETUserImports.result)

  object GETUserImports {
    private[this] val sALL = "$"
    private[this] val sSTATUS = "$.status"
    private[this] val sRESULT = "$.result"

    def status = if (importId != None) jsonPath(sSTATUS).find.saveAs("status") else jsonPath(sALL).find.exists
    def result = if (importId != None) jsonPath(sRESULT).find.saveAs("result") else jsonPath(sALL).find.exists
  }

}
