/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mdb

import com.openexchange.testing.performance.common.request.{ AbstractMDBRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[POSTUserImportsFormData]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
class POSTUserImportsFormData(tenantId: String, date: String, hash: String, csv: String) extends AbstractMDBRequest {

  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = "/user-imports/" + tenantId + "/file"
  override lazy val checks: Seq[HttpCheck] = Seq(POSTUserImportsFormData.importId)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam("date", date)
      .queryParam("hash", hash)
      .bodyPart(StringBodyPart("file", csv).contentType("text/csv").fileName("import.csv")).asMultipartForm
  }

  object POSTUserImportsFormData {
    private[this] val sIMPORTID = "$.importId"

    def importId = jsonPath(sIMPORTID).find.saveAs("importId")
  }

}
