/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mdb

import com.openexchange.testing.performance.common.request.{ AbstractMDBRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.http.Predef.jsonPath
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[POSTUserImportsS3]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
class POSTUserImportsS3(tenantId: String, dataSourceId: String, espImportId: String, date: String, hash: String) extends AbstractMDBRequest {

  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = "/user-imports/" + tenantId + "/" + espImportId
  override lazy val checks: Seq[HttpCheck] = Seq(POSTUserImportsS3.importId)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam("dataSourceId", dataSourceId)
      .body(StringBody(JSONUtil().toJson(
        Map("date" -> date, "hash" -> hash)))).asJSON
  }

  object POSTUserImportsS3 {
    private[this] val sIMPORTID = "$.importId"

    def importId = jsonPath(sIMPORTID).find.saveAs("importId")
  }

}
