/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.util

import java.io.FileWriter

import com.openexchange.testing.performance.common.utils.CryptoHelper
import com.typesafe.scalalogging.LazyLogging
import com.openexchange.testing.performance.common.utils.FakerHelper.faker

object CSVGenerator extends com.openexchange.testing.performance.config.Configuration {

  def createCSV(numberOfUsers: Integer, tenantName: String): String = {
    val stringBuilder = StringBuilder.newBuilder
    stringBuilder.append("email,shared_id,display_name\n")
    for (j <- 1 to numberOfUsers) {
      stringBuilder.append(CryptoHelper.sha256(s"user$j@$tenantName.com") + "," + s"user$j@$tenantName.com" + "," + faker.name().fullName() + "\n")
    }
    stringBuilder.mkString
  }

  def createCSVonFilesystem(numberOfUsers: Integer, tenantName: String): Unit = {
    val filewriter = new FileWriter(s"${settings.mdb.tmpFilePath}/import-$tenantName.csv")
    filewriter.write("email,shared_id,display_name\n")
    for (j <- 1 to numberOfUsers) {
      filewriter.write(CryptoHelper.sha256(s"user$j@$tenantName.com") + "," + s"user$j@$tenantName.com" + "," + faker.name().fullName() + "\n")
    }
    filewriter.close()
  }

}