/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mim

import com.openexchange.testing.performance.common.request.{ AbstractMiMRequest, RequestMethod }
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[GetUserCampaigns]] lists active campaigns for a user.
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class GetUserCampaigns(userHash: String) extends AbstractMiMRequest {
  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = "/ads/list"
  override lazy val checks: Seq[HttpCheck] = Seq(GetUserCampaigns.guids)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .queryParam("user_hash", userHash)
  }
}

object GetUserCampaigns {
  private[this] val sGUIDS = "$..guid"

  def guids = jsonPath(sGUIDS).findAll.optional.saveAs("allUserCampaignGuids")
}