/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mim

import com.openexchange.testing.performance.common.request.{ AbstractMiMRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil

import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[ImportAsyncCampaign]] Triggers an asynchronous import for a given tenant and id.
 *
 * @author <a href="mailto:alexander.schulze-ardey@open-xchange.com">Alexander Schulze-Ardey</a>
 * @constructor
 * @param tenantId The tenant to import the campaign for.
 * @param campaignId The campaign to import.
 * @param date the date/time of the call.
 * @param hash hmac sha1 of guid + date.
 */
class ImportAsyncCampaign(tenantId: String, campaignId: String, date: String, hash: String) extends AbstractMiMRequest {
  override val requestMethod: String = RequestMethod.POST
  override val relativeURL: String = s"/api/campaign-imports/$tenantId/$campaignId/"
  override lazy val checks: Seq[HttpCheck] = Seq(ImportAsyncCampaign.importId)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .body(StringBody(JSONUtil().toJson(
        Map("date" -> date, "hash" -> hash)))).asJSON
  }
}

object ImportAsyncCampaign {
  private[this] val sIMPORT_ID = "$.importId"

  def importId = jsonPath(sIMPORT_ID).find.saveAs("importId")
}