/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mim

import com.openexchange.testing.performance.common.request.{ AbstractMiMRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[ImportCampaign]] Read campaign data for a given guid, or any available from Amazon S3.
 * If guid is empty, all campaigns are imported, otherwise only the campaign with the given
 * guid is imported.
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @constructor
 * @param date the date/time of the call.
 * @param hash hmac sha1 of guid + date.
 * @param guid of the campaign.
 */
class ImportCampaign(guid: String, date: String, hash: String) extends AbstractMiMRequest {
  override val requestMethod: String = RequestMethod.PUT
  override val relativeURL: String = "/ads/modified"
  override lazy val checks: Seq[HttpCheck] = Seq(ImportCampaign.status)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
      .body(StringBody(JSONUtil().toJson(
        Map("date" -> date, "guid" -> guid, "hash" -> hash)))).asJSON
  }
}

object ImportCampaign {
  private[this] val sSTATUS = "$.status"

  def status = jsonPath(sSTATUS).is("ok")
}