/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.mim

import com.openexchange.testing.performance.common.request.{ AbstractMiMRequest, RequestMethod }
import com.openexchange.testing.performance.common.utils.JSONUtil

import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.check.HttpCheck
import io.gatling.http.request.builder.HttpRequestBuilder

/**
 * [[ImportCampaignStatus]] Read the import status for previously started import.
 *
 * @author <a href="mailto:alexander.schulze-ardey@open-xchange.com">Alexander Schulze-Ardey</a>
 */
class ImportCampaignStatus(tenantId: String, campaignId: String, importId: String) extends AbstractMiMRequest {
  override val requestMethod: String = RequestMethod.GET
  override val relativeURL: String = s"/api/campaign-imports/$tenantId/$campaignId/$importId"
  override lazy val checks: Seq[HttpCheck] = Seq(
    ImportCampaignStatus.status,
    ImportCampaignStatus.result,
    ImportCampaignStatus.startDate,
    ImportCampaignStatus.endDate)

  override def buildHttpRequest(): HttpRequestBuilder = {
    super.buildHttpRequest()
  }
}

object ImportCampaignStatus {
  private[this] val sIMPORT_ID = "$.id"
  private[this] val sSTATUS = "$.status"
  private[this] val sRESULT = "$.result"
  private[this] val sSTART_DATE = "$.startDate"
  private[this] val sEND_DATE = "$.endDate"

  def importId = jsonPath(sIMPORT_ID).find.saveAs("importId")
  def status = jsonPath(sSTATUS).find.saveAs("status")
  def result = jsonPath(sRESULT).find.saveAs("result")
  def startDate = jsonPath(sSTART_DATE).find.saveAs("startDate")
  def endDate = jsonPath(sEND_DATE).find.optional.saveAs("endDate")
}