/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.ThreadLocalRandom

import com.openexchange.testing.performance.common.utils.CryptoHelper
import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.mim._
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder

import scala.concurrent.duration._

/**
 * [[CampaignSimulation]]
 *
 * @author <a href="mailto:philipp.schumacher@open-xchange.com">Philipp Schumacher</a>
 */
class CampaignSimulation extends Simulation with Configuration with LazyLogging {

  private val campaignScenario: ScenarioBuilder = scenario("campaignScenario")
    .exec(session => {
      session.set("startChooseRandomUser", System.currentTimeMillis())
    })
    .exec(session => {
      session.set("randomUser", s"${settings.mim.campaignUserPattern}${
        ThreadLocalRandom.current().nextInt(1, settings
          .mim.campaignPlaceholderAbsolute)
      }")
    })
    .exec(session => {
      logger.debug("choosing random '{}' user took: {} ms", session("randomUser").as[String], System.currentTimeMillis() - session("startChooseRandomUser").as[Long])
      session
    })
    .exec(session => {
      session.set("createAndSetUserHash", System.currentTimeMillis())
    })
    .exec(session => {
      session.set("randomUserHash", CryptoHelper.sha256(session("randomUser").as[String]))
    })
    .exec(session => {
      logger.debug("creating user hash {} took: {} ms", session("randomUserHash").as[String], System.currentTimeMillis() - session("createAndSetUserHash").as[Long])
      session
    })
    .exec(new GetUserCampaigns("${randomUserHash}").buildHttpRequest())
    .randomSwitch(0.02d -> doIf(session => session.contains("allUserCampaignGuids")) {
      exec(session => {
        session.set("createRandomGuid", System.currentTimeMillis())
      })
        .exec(session => {
          val campaignVector = session("allUserCampaignGuids").as[Vector[Any]]
          session.set("randomCampaignGuid", campaignVector(ThreadLocalRandom.current().nextInt(campaignVector.length)))
        })
        .exec(session => {
          logger.debug("creating random guid took: {} ms", System.currentTimeMillis() - session("createRandomGuid").as[Long])
          session
        })
        .exec(new GetCampaign("${randomCampaignGuid}").buildHttpRequest())
    })

  setUp(campaignScenario.inject(
    rampUsersPerSec(1) to settings.simulation.injection.usersPerSec during (60 seconds),
    constantUsersPerSec(settings.simulation.injection.usersPerSec) during (settings.simulation.injection.duration seconds)))
    .protocols(HTTPConfig.mimConf)
}

