/*
 * Copyright (C) 2018 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.UUID
import java.util.concurrent.ThreadLocalRandom

import com.datastax.driver.core.{ Cluster, ConsistencyLevel }
import com.openexchange.testing.performance.common.utils.CryptoHelper
import com.openexchange.testing.performance.config.Configuration
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.scenario.Simulation
import io.github.gatling.cql.Predef._

import scala.collection.JavaConverters._
import scala.concurrent.duration._

class CassandraGetCampaignsSimulation extends Simulation with Configuration with LazyLogging {

  val clusterBuilder = Cluster.builder()
  settings.mim.cassandraEndpoints.foreach(endpoint => {
    clusterBuilder.addContactPoint(endpoint)
  })
  val cluster = clusterBuilder.build()
  val cSession = cluster.connect()
  val cqlConfig = cql.session(cSession)

  val feeder = Iterator.continually(
    Map(
      "userHash" -> CryptoHelper.sha256(s"""${settings.mim.campaignUserPattern}${ThreadLocalRandom.current().nextInt(1, settings.mim.campaignPlaceholderAbsolute)}""")))

  val scn = scenario("get active campaigns for user").repeat(1) { //Name your scenario
    feed(feeder)
      .exec(cql("find user")
        .execute("SELECT * FROM mim.user_profile WHERE id = '${userHash}'")
        .consistencyLevel(ConsistencyLevel.ONE)
        .check(rowCount is 1, columnValue("active_campaigns").find.saveAs("activeCampaigns")))
      .doIf(session => session.contains("activeCampaigns")) {
        foreach(session => session("activeCampaigns").as[java.util.LinkedHashMap[String, String]].entrySet().asScala.toList, "entry") {
          exec(session => {
            session.set("campaignId", UUID.fromString(session("entry").as[java.util.Map.Entry[String, String]].getKey))
          })
            .exec(
              cql("find campaign")
                .execute("SELECT id, categories, enddate, lastmodified, positionfixed, positions, sender, startdate, subject FROM mim.campaign WHERE id = ${campaignId}")
                .consistencyLevel(ConsistencyLevel.ONE)
                .check(rowCount is 1))

        }
      }
  }

  setUp(scn.inject(
    rampUsersPerSec(1) to settings.simulation.injection.usersPerSec during (60 seconds),
    constantUsersPerSec(settings.simulation.injection.usersPerSec) during (settings.simulation.injection.duration seconds)))
    .protocols(cqlConfig)

  after(cluster.close())

}
