#!/bin/bash

# 1. download apache-cxf tarball, extract it and "cd" into the directory, e.g.
#    tar zxvpf apache-cxf-3.0.0-milestone1.tar.gz; cd apache-cxf-3.0.0-milestone1
# 2. change variables CODEBASE, JAVA_HOME and WSDLURL
# 3. run this script "bash oxaas-wsdl2java"
# 4. fix com/openexchange/wsdl/context/soap/ListResponse.java: there is a name clash between some custom "List" soap datatype and java.util.List. We need to fully qualify the "List" in this file to be java.util.List twice.

export JAVA_HOME="/usr/lib/jvm/java-8-openjdk-amd64"
CODEBASE="."
WSDLURL="http://192.168.1.174/webservices"
CXF_HOME=/opt/apache-cxf-3.1.13
 
rm -rf $CODEBASE/com $CODEBASE/admin
frontend=jaxws21
dbinding=jaxb
 
JAXBTMP=/tmp/jaxb$$.xml
rm -f $JAXBTMP

#cat<<EOF > $JAXBTMP
#<jaxb:bindings version="2.1" xmlns:jaxb="http://java.sun.com/xml/ns/jaxb" xmlns:xjc="http://java.sun.com/xml/ns/jaxb/xjc" xmlns:xs="http://www.w3.org/2001/XMLSchema">
#   <jaxb:globalBindings generateElementProperty="false"/>
#</jaxb:bindings>
#EOF

cat<<EOF > $JAXBTMP
<jaxb:bindings version="2.1" xmlns:jaxb="http://java.sun.com/xml/ns/jaxb" xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <jaxb:bindings>
    <jaxb:globalBindings generateElementProperty="false" />
  </jaxb:bindings>
</jaxb:bindings>
EOF
 
# JAXBTMP2=/tmp/jaxb2-$$.xml


for class in Context User Group Resource; do
    lname=$(echo $class | tr '[:upper:]' '[:lower:]')
    #if [ -d $DEST/$lname ]; then
    #    rm -rf $DEST/$lname
    #fi
    pname="com.openexchange.wsdl.$lname"

    wsdl_location="${WSDLURL}/OX${class}Service?wsdl"

#    rm -f $JAXBTMP2
#    cat<<EOF > $JAXBTMP2
#<jaxws:bindings
#    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
#    xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
#    wsdlLocation="$wsdl_location"
#    xmlns="http://java.sun.com/xml/ns/jaxws"
#    xmlns:jaxws="http://java.sun.com/xml/ns/jaxws">
#    <enableWrapperStyle>false</enableWrapperStyle>
#</jaxws:bindings>
#EOF

    # $CXF_HOME/bin/wsdl2java -databinding $dbinding -frontend $frontend -client -impl -d $CODEBASE -keep -b $JAXBTMP -b $JAXBTMP2 \
    $CXF_HOME/bin/wsdl2java -databinding $dbinding -frontend $frontend -client -impl -d $CODEBASE -keep -b $JAXBTMP \
        -p "http://soap.admin.openexchange.com=${pname}.soap" \
        -p "http://dataobjects.soap.admin.openexchange.com/xsd=${pname}.soap.dataobjects" \
        -p "http://dataobjects.rmi.admin.openexchange.com/xsd=${pname}.rmi.dataobjects" \
        -p "http://exceptions.rmi.admin.openexchange.com/xsd=${pname}.rmi.exceptions" \
        -p "http://rmi.java/xsd=${pname}.java.rmi" \
        -p "http://io.java/xsd=${pname}.java.io" \
-encoding UTF-8 \
        $wsdl_location
done


 
rm -f $JAXBTMP
# rm -f $JAXBTMP2
