/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.webservices.SoapAPI
import com.openexchange.testing.performance.soap.{ SoapFeeder, WSDLServiceName }
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder

import scala.concurrent.duration.FiniteDuration

/* Parameters:
 *
 * simulation.injection.users : concurrency
 * simulation.injection.repeatCounter : iterations   -> total number of contexts provisioned: users*repeatCounter
 * simulation.soap.startId : start context id
 * simulation.soap.usersPerContext : users per context. total number of users provisioned: users*repeatCounter*usersPerContext
 */

/**
 * [[ChangeCapabilitiesSimulation]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
class ChangeCapabilitiesSimulation extends Simulation with Configuration {

  private val usersPerContext = settings.soap.usersPerContext.getOrElse(10)

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .exec(session => {
      session.setAll(
        "oxadminmaster" -> settings.soap.adminmaster,
        "masterpassword" -> settings.soap.masterpassword,
        "adminpassword" -> settings.soap.adminpassword,
        "oxadmin" -> settings.soap.admin)
    })
    .doIf(settings.soap.capabilitiesToAdd.isDefined) {
      exec(session => {
        session.set("capabilities_to_add", settings.soap.capabilitiesToAdd.get)
      })
    }
    .doIf(settings.soap.capabilitiesToRemove.isDefined) {
      exec(session => {
        session.set("capabilities_to_remove", settings.soap.capabilitiesToRemove.get)
      })
    }
    .doIf(settings.soap.capabilitiesToDrop.isDefined) {
      exec(session => {
        session.set("capabilities_to_drop", settings.soap.capabilitiesToDrop.get)
      })
    }
    .repeat(settings.simulation.injection.repeatCounter) {
      pace(new FiniteDuration(usersPerContext * 1000, TimeUnit.MILLISECONDS))
        .feed(SoapFeeder.contextFeeder)
        .repeat(usersPerContext, "userCounter") {
          pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
            // set: user_id, capabilities_to_(add|remove|drop)
            .exec(session => {
              // add 3 because oxadmin is 2 and we want to change only user users which start with id 3
              // this will break if the ids are not consecutive, like after userdelete and create and stuff
              session.set("user_id", session("userCounter").as[Int] + 3)
            })
            .exec(SoapAPI.changeCapabilities(WSDLServiceName.OX_USER_SERVICE))
        }
    }

  setUp(scn.inject(atOnceUsers(settings.simulation.injection.users))).protocols(HTTPConfig.appsuiteConf)
}
