/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.UUID
import java.util.concurrent.ConcurrentLinkedDeque

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.WSDLServiceName
import com.openexchange.testing.performance.soap.webservices.admin.context.Change
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._

import scala.collection.JavaConverters._

/**
 * [[ChangeContextSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class ChangeContextSimulation extends Simulation with Configuration with LazyLogging {
  val listOfContexts = new ConcurrentLinkedDeque[Int]()
  listOfContexts.addAll((settings.soap.startId to settings.soap.endId)
    .asJavaCollection)

  object Rename {
    val rename = {
      exec(session => {
        session.setAll(
          "oxadminmaster" -> settings.soap.adminmaster,
          "masterpassword" -> settings.soap.masterpassword,
          "adminpassword" -> settings.soap.adminpassword,
          "oxadmin" -> settings.soap.admin)
      })
        .asLongAs(session => listOfContexts.size() > 0) {
          exec(session => {
            logger.trace("List of contexts: {}", listOfContexts)
            session.setAll("ctx_id" -> listOfContexts.poll(), "ctx_name" -> UUID.randomUUID().toString)
          })
            .group(WSDLServiceName.OX_CONTEXT_SERVICE) {
              exec(new Change(WSDLServiceName.OX_CONTEXT_SERVICE))
            }
            .exec(session => for {
              cid <- session("ctx_id").validate[String]
              ctx_name <- session("ctx_name").validate[String]
            } yield {
              logger.debug("Successfully renamed context with id {}", cid)
              session.removeAll("ctx_id", "ctx_name")
            })
        }
    }
  }

  val renameScn = {
    scenario(s"Rename ${settings.simulation.injection.repeatCounter * settings.simulation.injection.users} contexts")
      .exec(Rename.rename)
  }

  setUp(renameScn.inject(atOnceUsers(settings.simulation.injection.users)))
    .protocols(HTTPConfig.appsuiteConf)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
