/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.webservices.SoapAPI
import com.openexchange.testing.performance.soap.{ SoapFeeder, WSDLServiceName }
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder

import scala.concurrent.duration._

/**
 * [[CreateContextSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
class CreateContextSimulation extends Simulation with Configuration {

  val schemaFeeder = csv(settings.soap.schemaFeeder.getOrElse("data/schemas-dummy.csv")).circular.convert {
    case ("writeDBPoolId", string) => string.toInt
    case ("filestoreId", string)   => string.toInt
  }

  val missingFeeder = csv(settings.soap.missingFeeder.getOrElse("data/missing-dummy.csv")).convert {
    case ("ctx_id", string) => string.toInt
  }

  /*
  val contextFeeder = settings.soap.contextFeeder match {
    case Some("PS") => SoapFeeder.psContextFeeder
    case _          => SoapFeeder.contextFeeder
  }

  val userFeeder = settings.soap.userFeeder match {
    case Some("PS") => SoapFeeder.psUserFeeder
    case _          => SoapFeeder.in8UserFeeder
  }
  */

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .exec(session => {
      session.setAll(
        "oxadminmaster" -> settings.soap.adminmaster,
        "masterpassword" -> settings.soap.masterpassword,
        "adminpassword" -> settings.soap.adminpassword,
        "oxadmin" -> settings.soap.admin)
    })
    .doIf(settings.soap.filestoreId.isDefined) {
      exec(session => session.set("filestoreId", settings.soap.filestoreId.get))
    }
    .doIf(settings.soap.writeDBPoolId.isDefined) {
      exec(session => session.set("writeDBPoolId", settings.soap.writeDBPoolId.get))
    }

    .repeat(settings.simulation.injection.repeatCounter) {
      pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
        .feed(SoapFeeder.contextFeeder)
        .doIf(settings.soap.schemaFeeder.isDefined) {
          feed(schemaFeeder)
        }
        .doIf(settings.soap.missingFeeder.isDefined) {
          feed(missingFeeder)
        }
        .exec(SoapAPI.createModuleAccessByName(WSDLServiceName.OX_CONTEXT_SERVICE, Some(settings.soap.accessCombinationName)))
    }

  setUp(scn.inject(atOnceUsers(settings.simulation.injection.users))).protocols(HTTPConfig.appsuiteConf)
}
