/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.webservices.admin.context.DeleteContext
import com.openexchange.testing.performance.soap.WSDLServiceName
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder

import scala.concurrent.duration._

/**
 * [[DeleteContextFromCSVSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
class DeleteContextFromCSVSimulation extends Simulation with Configuration with LazyLogging {

  val missingFeeder = csv(settings.soap.missingFeeder.getOrElse("data/missing-dummy.csv")).convert {
    case ("ctx_id", string) => string.toInt
  }

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .exec(session => {
      session.setAll(
        "oxadminmaster" -> settings.soap.adminmaster,
        "masterpassword" -> settings.soap.masterpassword,
        "adminpassword" -> settings.soap.adminpassword,
        "oxadmin" -> settings.soap.admin)
    })

    //.repeat(settings.simulation.injection.repeatCounter) {
    // run into an exception when the feeder is empty :-/
    .asLongAs(true) {
      pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
        .feed(missingFeeder)
        .group(WSDLServiceName.OX_CONTEXT_SERVICE) {
          exec(new DeleteContext(WSDLServiceName.OX_CONTEXT_SERVICE))
        }
        .exec(session => for {
          cid <- session("ctx_id").validate[String]
        } yield {
          logger.debug("Successfully deleted context with id {}", cid)
          session.remove("ctx_id")
        })
    }

  setUp(scn.inject(atOnceUsers(settings.simulation.injection.users))).protocols(HTTPConfig.appsuiteConf)
}
