/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.TimeUnit

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.{ SoapFeeder, WSDLServiceName }
import com.openexchange.testing.performance.soap.webservices.SoapAPI
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._

import scala.concurrent.duration.FiniteDuration

/* Parameters:
 *
 * simulation.injection.users : concurrency
 * simulation.injection.repeatCounter : iterations   -> total number of contexts provisioned: users*repeatCounter
 * simulation.soap.startID : start context id
 */

/**
 * [[DeleteContextSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class DeleteContextSimulation extends Simulation with Configuration with LazyLogging {

  object Delete {
    val delete = {
      exec(session => {
        session.setAll(
          "oxadminmaster" -> settings.soap.adminmaster,
          "masterpassword" -> settings.soap.masterpassword,
          "oxadmin" -> settings.soap.admin)
      })
        .repeat(settings.simulation.injection.repeatCounter) {
          pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
            .feed(SoapFeeder.contextFeeder)
            .exec(SoapAPI.deleteContext(WSDLServiceName.OX_CONTEXT_SERVICE))
        }
    }
  }

  val deleteScn = {
    scenario(s"Delete ${settings.soap.endId - settings.soap.startId + 1} contexts")
      .exec(Delete.delete)
  }

  setUp(deleteScn.inject(atOnceUsers(settings.simulation.injection.users)))
    .protocols(HTTPConfig.appsuiteConf)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
