/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.webservices.SoapAPI
import com.openexchange.testing.performance.soap.{ JaxbHelper, WSDLServiceName }
import com.openexchange.wsdl.context.soap.ListResponse
import com.typesafe.scalalogging.LazyLogging
import io.gatling.commons.validation.{ Failure, Success }
import io.gatling.core.Predef._
import io.gatling.core.structure.{ ChainBuilder, ScenarioBuilder }

import scala.collection.JavaConverters._

/**
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
class ListContextSimulation extends Simulation with Configuration with LazyLogging {

  val list: ChainBuilder = SoapAPI.listContext(WSDLServiceName.OX_CONTEXT_SERVICE, "*", 0, keepResponse = true)

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .exec(session => {
      session.setAll(
        "oxadminmaster" -> settings.soap.adminmaster,
        "masterpassword" -> settings.soap.masterpassword)
    })
    .exec(list)
    .exec(session => {

      var s = session
      for {
        soapResponse <- session("soapResponse").validate[String]
      } yield {
        JaxbHelper.unserializeCtx[ListResponse](soapResponse) match {
          case Success(lr) => {
            val listResponse = lr.getReturn.asScala
            printf("%6s %3s %-15s %s\n", "cid", "fid", "name", "lmappings")
            for (e <- listResponse) {
              printf("%6d %3d %-15s %s\n", e.getId, e.getFilestoreId, e.getName, e.getLoginMappings)
            }

            /*
              println("size of returned list: " + contextIdsList.size)
              for (contextid <- contextIdsList) {
                println("context id: " + contextid)
              }
*/
          }
          case Failure(error) => println("Failure: " + error)
        }
      }
      s.remove("soapResponse")
    })

  setUp(scn.inject(atOnceUsers(1)))
    .protocols(HTTPConfig.appsuiteConf)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
