/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import java.util.concurrent.{ ConcurrentLinkedDeque, TimeUnit }

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.WSDLServiceName
import com.openexchange.testing.performance.soap.webservices.admin.context.MoveContextFilestore
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._

import scala.collection.JavaConverters._
import scala.concurrent.duration.FiniteDuration

/**
 * [[MoveContextFilestoreSimulation]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class MoveContextFilestoreSimulation extends Simulation with Configuration with LazyLogging {
  val listOfContexts = new ConcurrentLinkedDeque[Int]()
  listOfContexts.addAll((settings.soap.startId to settings.soap.endId)
    .asJavaCollection)

  object MoveContextFilestore {
    val moveContextFilestore = {
      exec(session => {
        session.setAll(
          "oxadminmaster" -> settings.soap.adminmaster,
          "masterpassword" -> settings.soap.masterpassword,
          "oxadmin" -> settings.soap.admin,
          "filestore_id" -> 77)
      })
        .asLongAs(session => listOfContexts.size() > 0) {
          pace(new FiniteDuration(1000, TimeUnit.MILLISECONDS))
            .exec(session => {
              logger.trace("List of contexts: {}", listOfContexts)
              session.set("ctx_id", listOfContexts.poll())
            })
            .group(WSDLServiceName.OX_CONTEXT_SERVICE) {
              exec(new MoveContextFilestore(WSDLServiceName.OX_CONTEXT_SERVICE))
            }
            .exec(session => for {
              cid <- session("ctx_id").validate[String]
              filestore_id <- session("filestore_id").validate[String]
            } yield {
              logger.debug("Successfully moved context with id {} to filestore {}", cid, filestore_id)
              session.remove("ctx_id")
            })
        }
    }
  }

  val moveContextFilestoreScn = {
    scenario(s"Move ${settings.soap.endId - settings.soap.startId + 1} contexts")
      .exec(MoveContextFilestore.moveContextFilestore)
  }

  setUp(moveContextFilestoreScn.inject(atOnceUsers(settings.simulation.injection.users)))
    .protocols(HTTPConfig.appsuiteConf)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
