/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.simulations

import com.openexchange.testing.performance.config.{ Configuration, HTTPConfig }
import com.openexchange.testing.performance.soap.webservices.SoapAPI
import com.openexchange.testing.performance.soap.{ SoapFeeder, WSDLServiceName }
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.core.structure.{ ChainBuilder, ScenarioBuilder }

/**
 * The [[QAProvisioningSimulation]] simulates OX6
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
class QAProvisioningSimulation extends Simulation with Configuration with LazyLogging {

  object Context {
    val create: ChainBuilder = SoapAPI.createModuleAccessByName(WSDLServiceName.OX_CONTEXT_SERVICE, Some("all"))

    val delete: ChainBuilder = SoapAPI.deleteAllContexts(WSDLServiceName.OX_CONTEXT_SERVICE)

    val list: ChainBuilder = SoapAPI.listContext(WSDLServiceName.OX_CONTEXT_SERVICE, "*",
      settings.soap.startId)
  }

  val scn: ScenarioBuilder = scenario(getClass.getSimpleName)
    .exec(session => {
      session.setAll(
        "oxadminmaster" -> settings.soap.adminmaster,
        "masterpassword" -> settings.soap.masterpassword,
        "oxadmin" -> settings.soap.admin,
        "adminpassword" -> settings.soap.adminpassword,
        "schema_name" -> "",
        "filestoreId" -> settings.soap.filestoreId.getOrElse("""<xsd:filestoreId xsi:nil="true" />"""),
        "writeDBPoolId" -> settings.soap.writeDBPoolId.getOrElse("""<xsd:id xsi:nil="true" />"""))
    })
    .repeat(settings.simulation.injection.repeatCounter) {
      // the contextFeeder should feed all relevant data also for the context admin user
      feed(SoapFeeder.contextFeeder)
        .exec(Context.create)
    }

    // delete all created contexts
    .exec(Context.list)
    .exec(Context.delete)

  setUp(scn.inject(atOnceUsers(settings.simulation.injection.users)))
    .protocols(HTTPConfig.appsuiteConf)
    .assertions(global.successfulRequests.percent.gte(settings.simulation.assertions.successfulRequests))
}
