/*
 * Copyright (C) 2016 OX Software GmbH
 *
 * This file is part of OX Gatling Performance Tests.
 *
 * OX Gatling Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX Gatling Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX Gatling Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.soap

import com.openexchange.testing.performance.soap.SoapFeeder.settings
import com.openexchange.wsdl.context.rmi.dataobjects.Credentials
import com.openexchange.wsdl.context.soap.MoveContextFilestore
import com.openexchange.wsdl.context.soap.dataobjects._
import io.gatling.core.Predef.Session

/**
 * [[ContextDataobjectsFactory]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
object ContextDataobjectsFactory {

  def buildCreateModuleAccessByName(session: Session): com.openexchange.wsdl.context.soap.CreateModuleAccessByName = {

    val createModuleAccessByName = new com.openexchange.wsdl.context.soap.CreateModuleAccessByName

    createModuleAccessByName.setAdminUser(buildAdminUser(session))
    createModuleAccessByName.setAuth(buildCredentials(session))
    createModuleAccessByName.setCtx(buildContext(session))
    createModuleAccessByName.setAccessCombinationName(session("access_combination_name").as[String])
    createModuleAccessByName.setSchemaSelectStrategy(buildSchemaSelectStrategy(session))

    createModuleAccessByName

  }

  def buildCreate(session: Session): com.openexchange.wsdl.context.soap.Create = {

    val create = new com.openexchange.wsdl.context.soap.Create

    create.setAdminUser(buildAdminUser(session))
    create.setAuth(buildCredentials(session))
    create.setCtx(buildContext(session))
    create.setSchemaSelectStrategy(buildSchemaSelectStrategy(session))

    create

  }

  def buildChange(session: Session): com.openexchange.wsdl.context.soap.Change = {

    val change = new com.openexchange.wsdl.context.soap.Change

    change.setAuth(buildCredentials(session))

    val ctx = new Context
    ctx.setId(session("ctx_id").as[Int])
    ctx.setName(session("ctx_name").as[String])
    change.setCtx(ctx)

    change

  }

  def buildDelete(session: Session): com.openexchange.wsdl.context.soap.Delete = {

    val delete = new com.openexchange.wsdl.context.soap.Delete

    delete.setAuth(buildCredentials(session))

    val ctx = new Context
    ctx.setId(session("ctx_id").as[Int])
    delete.setCtx(ctx)

    delete

  }

  def buildMoveContextFilestore(session: Session): com.openexchange.wsdl.context.soap.MoveContextFilestore = {

    val moveContextFilestore = new MoveContextFilestore

    moveContextFilestore.setAuth(buildCredentials(session))

    val ctx = new Context
    ctx.setId(session("ctx_id").as[Int])
    moveContextFilestore.setCtx(ctx)

    val filestore = new Filestore
    filestore.setId(session("filestore_id").as[Int])
    moveContextFilestore.setDstFilestoreId(filestore)

    moveContextFilestore

  }

  def buildListAll(session: Session): com.openexchange.wsdl.context.soap.ListAll = {

    val listAll = new com.openexchange.wsdl.context.soap.ListAll

    listAll.setAuth(buildCredentials(session))

    listAll

  }

  def buildList(session: Session): com.openexchange.wsdl.context.soap.List = {

    val list = new com.openexchange.wsdl.context.soap.List

    list.setSearchPattern(session("search_pattern").as[String])
    list.setAuth(buildCredentials(session))

    list

  }

  def buildContext(session: Session): Context = {

    val ctx = new Context

    if (!settings.soap.reseller) {
      ctx.setName(session("ctx_name").as[String])
      ctx.setId(session("ctx_id").as[Int])
    }
    ctx.setMaxQuota(session("ctx_maxQuota").as[Int])
    for { filestoreId <- session("filestoreId").validate[Int] } yield {
      //println("Setting filestoreId to " + filestoreId)
      ctx.setFilestoreId(filestoreId)
    }
    for { writeDBPoolId <- session("writeDBPoolId").validate[Int] } yield {
      //println("Setting writeDBPoolId to " + writeDBPoolId)
      val db: Database = new Database
      db.setId(writeDBPoolId)
      ctx.setWriteDatabase(db)
    }

    // ctx.getLoginMappings().add(session("addmapping").as[String])

    /* how to set taxonomy/types=super_webhosting
    var taxonomy: SOAPMapEntry = new SOAPMapEntry
    taxonomy.setKey("taxonomy")
    var taxtypeval: SOAPStringMap = new SOAPStringMap
    var taxtypeent: Entry = new Entry
    taxtypeent.setKey("types")
    taxtypeent.setValue("super_webhosting")
    taxtypeval.getEntries().add(taxtypeent)
    taxonomy.setValue(taxtypeval)

    var userattrs: SOAPStringMapMap = new SOAPStringMapMap
    userattrs.getEntries().add(taxonomy)
    ctx.setUserAttributes(userattrs)
    */

    ctx

  }

  def buildAdminUser(session: Session): User = {

    val oxadmin: User = new User
    oxadmin.setName(session("oxadmin").as[String])
    oxadmin.setPassword(session("adminpassword").as[String])
    oxadmin.setDisplayName(session("display_name").as[String])
    oxadmin.setSurName(session("sur_name").as[String])
    oxadmin.setGivenName(session("given_name").as[String])
    oxadmin.setPrimaryEmail(session("primaryEmail").as[String])
    oxadmin.setEmail1(session("email1").as[String])
    oxadmin.setDefaultSenderAddress(session("primaryEmail").as[String])
    for { v <- session("language").validate[String] } yield oxadmin.setLanguage(v)
    for { v <- session("timezone").validate[String] } yield oxadmin.setTimezone(v)
    for { v <- session("imapServer").validate[String] } yield oxadmin.setImapServer(v)
    for { v <- session("smtpServer").validate[String] } yield oxadmin.setSmtpServer(v)

    oxadmin

  }

  def buildCredentials(session: Session): Credentials = {

    val creds: Credentials = new Credentials
    creds.setLogin(session("oxadminmaster").as[String])
    creds.setPassword(session("masterpassword").as[String])

    creds

  }

  def buildSchemaSelectStrategy(session: Session): SchemaSelectStrategy = {

    val strat: SchemaSelectStrategy = new SchemaSelectStrategy

    // pick one of the following 3 (selecting something else will yield an error)
    if (session.contains("schema_name")) {
      strat.setSchemaName(session("schema_name").as[String])
      // println("setting schema_name=" + strat.getSchemaName)
    } else if (session.contains("schema_select_strategy")) {
      strat.setStrategy(session("schema_select_strategy").as[String])
      // println("setting schema_select_strategy to " + strat.getStrategy)
    } else {
      strat.setStrategy("automatic")
      // println("setting schema_select_strategy to automatic")
    }

    strat

  }

}
