/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.soap

import java.util.Locale
import java.util.concurrent.atomic.AtomicInteger

import com.openexchange.testing.performance.common.utils.FakerHelper.faker
import com.openexchange.testing.performance.config.Configuration
import io.gatling.core.Predef._

/**
 * [[SoapFeeder]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
object SoapFeeder extends Configuration {

  val contextFeeder = new Feeder[Any] {
    override def hasNext = true

    val ctxIdCounter = new AtomicInteger(settings.soap.startId)
    val ctxNameCounter = new AtomicInteger(settings.soap.startName.getOrElse(1))

    override def next(): Map[String, Any] = {
      if (settings.soap.reseller) {
        Map("ctx_maxQuota" -> settings.soap.maxQuota)
      } else {
        val ctx_id = ctxIdCounter.getAndIncrement()
        val ctx_name = "context" + ctxNameCounter.getAndIncrement.toString
        val user_name = "user0"
        val email1 = user_name + "." + ctx_name + "@" + settings.simulation.data.user.domain
        val given_name = faker.name().firstName()
        val sur_name = faker.name().lastName()
        val display_name = s"$given_name $sur_name"
        Map(
          "ctx_id" -> ctx_id,
          "ctx_name" -> ctx_name,
          "oxadmin" -> user_name,
          "display_name" -> display_name,
          "sur_name" -> sur_name,
          "given_name" -> given_name,
          "email1" -> email1,
          "primaryEmail" -> email1)
      }

    }
  }

  // user "feeder" make no sense as they should not feed "globally", but rather per gatling-user
  // need some other construct
  // use this with flattenMapIntoAttributes
  // see https://gatling.io/docs/2.3/session/feeder/#non-shared-data
  def userMap(ctx_name: String, userCounter: Int): Map[String, String] = {
    val user_name = "user" + userCounter.toString
    val email1 = user_name + "." + ctx_name + "@" + settings.simulation.data.user.domain
    val given_name = faker.name().firstName()
    val sur_name = faker.name().lastName()
    val display_name = s"$given_name $sur_name"
    Map(
      "name" -> user_name,
      "display_name" -> display_name,
      "sur_name" -> sur_name,
      "given_name" -> given_name,
      "email1" -> email1,
      "primaryEmail" -> email1)
  }
}
