/*
 * Copyright (C) 2016 OX Software GmbH
 *
 * This file is part of OX Gatling Performance Tests.
 *
 * OX Gatling Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX Gatling Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX Gatling Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.soap

import com.openexchange.wsdl.user.soap.dataobjects.User
import com.openexchange.wsdl.user.soap.dataobjects.Context
import com.openexchange.wsdl.user.rmi.dataobjects.Credentials
import com.openexchange.wsdl.user.soap.ChangeCapabilities
import io.gatling.core.Predef.Session

/**
 * [[UserDataobjectsFactory]]
 *
 * @author <a href="mailto:dominik.epple@open-xchange.com">Dominik Epple</a>
 */
object UserDataobjectsFactory {

  def buildCreate(session: Session): com.openexchange.wsdl.user.soap.Create = {

    val create = new com.openexchange.wsdl.user.soap.Create

    create.setUsrdata(buildUsrdata(session))
    create.setAuth(buildCredentials(session))
    create.setCtx(buildContext(session))

    create

  }

  def buildChange(session: Session): com.openexchange.wsdl.user.soap.Change = {

    val change = new com.openexchange.wsdl.user.soap.Change

    change.setAuth(buildCredentials(session))
    change.setCtx(buildContext(session))
    change.setUsrdata(buildUsrdata(session))

    change

  }

  def buildContext(session: Session): Context = {

    val ctx = new Context
    ctx.setId(session("ctx_id").as[Int])
    if (session.contains("maxQuota")) {
      ctx.setMaxQuota(session("ctx_maxQuota").as[Long])
    }

    ctx

  }

  def buildUsrdata(session: Session) = {

    // println("username: " + session("name").as[String])

    val user: User = new User
    user.setDisplayName(session("display_name").as[String])
    user.setEmail1(session("email1").as[String])
    user.setGivenName(session("given_name").as[String])
    for { imapLogin <- session("imapLogin").validate[String] } yield user.setImapLogin(imapLogin)
    for { imapServer <- session("imapServer").validate[String] } yield user.setImapServer(imapServer)
    user.setName(session("name").as[String])
    user.setPassword(session("userpassword").as[String])
    user.setPrimaryEmail(session("primaryEmail").as[String])
    for { smtpServer <- session("smtpServer").validate[String] } yield user.setSmtpServer(smtpServer)
    user.setSurName(session("sur_name").as[String])
    user.setDefaultSenderAddress(session("primaryEmail").as[String])
    // user.setLanguage(session("language").as[String])
    // user.setTimezone(session("timezone").as[String])

    user

  }

  def buildChangeCapabilities(session: Session) = {
    val chCapa = new ChangeCapabilities

    chCapa.setCtx(buildContext(session))
    chCapa.setUser(buildUsrdataForChange(session))
    for { capabilities_to_add <- session("capabilities_to_add").validate[String] } yield chCapa.setCapsToAdd(capabilities_to_add)
    for { capabilities_to_remove <- session("capabilities_to_remove").validate[String] } yield chCapa.setCapsToRemove(capabilities_to_remove)
    for { capabilities_to_drop <- session("capabilities_to_drop").validate[String] } yield chCapa.setCapsToDrop(capabilities_to_drop)
    chCapa.setAuth(buildCredentials(session))

    chCapa

  }

  def buildUsrdataForChange(session: Session) = {
    val user: User = new User
    for { userId <- session("user_id").validate[Int] } yield user.setId(userId)

    user
  }

  def buildCredentials(session: Session): Credentials = {

    val creds: Credentials = new Credentials
    creds.setLogin(session("oxadmin").as[String])
    creds.setPassword(session("adminpassword").as[String])

    creds

  }

}