/*
 * Copyright (C) 2017 OX Software GmbH
 *
 * This file is part of OX API-Performance Tests.
 *
 * OX API-Performance Tests is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OX API-Performance Tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OX API-Performance Tests.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.testing.performance.soap.webservices

import com.openexchange.testing.performance.common.request.RequestMethod
import com.openexchange.testing.performance.config.Configuration
import com.typesafe.scalalogging.LazyLogging
import io.gatling.core.Predef._
import io.gatling.http.Predef._
import io.gatling.http.action.sync.HttpRequestActionBuilder

import scala.language.implicitConversions

/**
 * [[AbstractSoapRequest]]
 *
 * @author <a href="mailto:nikolaos.tsapanidis@open-xchange.com">Nikolaos Tsapanidis</a>
 */
abstract class AbstractSoapRequest extends LazyLogging with Configuration {

  val wsdlEndpoint: String

  val soapAction: String

  val soapBody: io.gatling.core.session.Expression[scala.Predef.String]

  val requestMethod: String = RequestMethod.POST

  lazy val relativeURL: String = s"/webservices/$wsdlEndpoint"

  def build() = {
    http(session => getClass.getSimpleName)
      .httpRequest(requestMethod.toString, relativeURL)
      .headers(Map("SOAPAction" -> s"urn:$soapAction"))
      .body(StringBody(soapBody)).asXML
      .check(status.is(200))
  }
}

object AbstractSoapRequest {
  implicit def toActionBuilder(request: AbstractSoapRequest): HttpRequestActionBuilder = new HttpRequestActionBuilder(request.build())
}
