# OX API-Performance Tests Installation Guide

[TOC]

## Software Requirements
* Java SE 8
* Git
* SBT >= 0.13.0

## Getting Performance Repository
* Cloning Repository

```bash
$ git clone https://git.open-xchange.com/git/wd/testing/performance
```

* Checkout a specific branch, e.g. `develop`

```bash
$ git checkout develop
```

## Install SBT
Since we use the Gatling-SBT plugin, which allows us to use Gatling as a testing framework, we'll need to install SBT first.

### Installing sbt on Mac
* [Macports](http://macports.org)

```bash
$ port install sbt
```

* [Homebrew](http://mxcl.github.com/homebrew)

```bash
$ brew install sbt
```

### Installing sbt on Windows
* Download the [msi installer](https://dl.bintray.com/sbt/native-packages/sbt/0.13.11/sbt-0.13.11.msi) and install it.

### Installing sbt on Linux
* [RPM](https://dl.bintray.com/sbt/rpm/sbt-0.13.11.rpm) package
* [DEB](https://dl.bintray.com/sbt/debian/sbt-0.13.11.deb) package

### Installing sbt manually
* [Manual](http://www.scala-sbt.org/0.13/tutorial/Manual-Installation.html) instructions

## Build and execute the tests with the SBT-Plugin
After SBT has been successfully installed we can switch to the project's directory:

```bash
$ cd ~/performance/com.openexchange.test.performance.gatling
```

SBT can be used both as a command line script and as a build console. Most of the commands can be run standalone by passing the command as an argument to SBT, e.g.

```bash
$ sbt update
```

Display all available projects:

```bash
$ sbt projects
[info] Loading project definition from ~/performance/com.openexchange.test.performance.gatling/project
[info] Set current project to root (in build file:~/performance/com.openexchange.test.performance.gatling/)
[info] In file:~/performance/com.openexchange.test.performance.gatling/
[info] 	   project-ajax
[info] 	   project-common
[info] 	   project-drive
[info] 	   project-eas
[info] 	   project-guard
[info] 	   project-in8
[info] 	   project-jolokia
[info] 	   project-mailapp
[info] 	   project-rest
[info] 	   project-soap
[info] 	   project-usm
[info] 	 * root

```

Switch to a specific project:

```bash
root> project project-ajax
[info] Set current project to project-ajax (in build file:~/performance/com.openexchange.test.performance.gatling/)
project-ajax>
```

To execute all tests in the project just enter:

```bash
project-ajax> gatling:test
[info] Formatting 21 Scala sources {file:~/performance/com.openexchange.test.performance.gatling/}project-common(test) ...
[info] Formatting 66 Scala sources {file:~/performance/com.openexchange.test.performance.gatling/}project-ajax(test) ...
[info] Compiling 21 Scala sources to ~/performance/com.openexchange.test.performance.gatling/project-common/target/scala-2.10/test-classes...
[info] Compiling 66 Scala sources to ~/performance/com.openexchange.test.performance.gatling/project-ajax/target/scala-2.10/test-classes...
[info] Compiling 66 Scala sources to ~/performance/com.openexchange.test.performance.gatling/project-ajax/target/scala-2.10/gatling-classes...
Simulation com.openexchange.test.performance.gatling.simulations.QAOXAppSuitePerfSimulation started...
```

Note that if a command takes arguments, you need to quote the entire argument path, e.g.

```bash
$ sbt 'project-ajax/gatling:testOnly com.openexchange.test.performance.gatling.simulations.QAOXAppSuitePerfSimulation'
```


The following command line script will execute the Gatling AppSuite Performance test:

```bash
$ sbt "project project-ajax" 'set javaOptions ++=Seq("-Dox.backend.host=10.0.0.1", "-Dox.backend.protocol=http", "-Dox.backend.port=80", "-Dox.injection.users=100", "-Dox.injection.durationTime=120m")' clean "gatling:testOnly com.openexchange.test.performance.gatling.simulations.QAOXAppSuitePerfSimulation"
```

## Development Environment
If you want to write and/or execute tests via an IDE like Eclipse or IntelliJ IDEA then you will also need to install the following software:

### IntelliJ IDEA
Download and install the [IntelliJ IDEA](http://www.jetbrains.com/idea/download/).

### Eclipse
* [Eclipse 4.4](https://www.eclipse.org) or later
* [Scala IDE plugin](http://scala-ide.org)

Since there are a few problems with the Scala IDE for eclipse we recommend to use IntelliJ which works out of the box.

### Importing Project into IntelliJ

#### Create project files
To create the Idea project files please use the sbt task `gen-idea`.

```bash
$ cd ~/performance/com.openexchange.test.performance.gatling

$ sbt gen-idea
[info] Loading project definition from ~/performance/com.openexchange.test.performance.gatling/project
[info] Set current project to root (in build file:~/performance/com.openexchange.test.performance.gatling/)
```

This task will resolve all project dependencies and create the IDEA modules accordingly.

#### Import project

1. Start IntelliJ
2. Click on Import Project...
3. Select the gatling folder within the performance repository
4. Import project from external model → SBT Project → Finish

## Configurable Parameters

We are using [Typesafe's Config](https://github.com/typesafehub/config) library for configuration settings. Here's an example of our default conf file `ox.conf`:

```json
####################
# OX Configuration #
####################

ox {
  backend {
    protocol = "http"
    host = "localhost"
    port = 80
    url = ${ox.backend.protocol}"://"${ox.backend.host}":"${ox.backend.port}
    dispatcherPrefix = "/ajax/"             # The servlet path. "/ajax/", "/appsuite/api/", ...
  }

  injection {
    durationTime = 10s
    rampup = 10
    users = 1
    usersPerSec = 1
    loopTime = 5m
    repeatCounter = 1
  }

  data {
    userFeeder = "data/jenkins.csv"         # The filename which contains the login data
    basicAuthUser = "open-xchange"          # The user name for the Basic authentication mechanism
    basicAuthPass = "secret"                # The password for the Basic authentication mechanism
    keepData = "no"                         # When set to false or no, all created data will be deleted after the testrun
    soap {
      startId = 1                           # Counter variable
      maxQuota = -1                         # The filestore quota in MB
      adminmaster = "oxadminmaster"         # Master admin user name
      admin = "oxadmin"                     # Admin user name
      password = "secret"                   # The password which is used for the oxadmin and oxadminmaster
      schemaSelectStrategy = "automatic"    # The schema select strategy. "automatic" or "in-memory"
      imapServer = "localhost"              # FQDN or IP of the ImapServer
      smtpServer = "localhost"              # FQDN or IP of the SmtpServer
      # filestoreId = 2                     # The filestore identifier
      # writeDBPoolId = 3                   # The write DBPool identifier
    }
  }

  modules = [
    "webmail"
    "calendar"
    "contacts"
    "infostore"
    "tasks"
  ]

  assertions {
    successfulRequests = 0
  }

  custom {
    mailfolders = "yes"
  }
}
```

You can *override* configuration settings in each projects resource directory usually ``` project-${PROJECT_NAME}/src/test/resources ``` or use *java system properties* (by using the command line flag `-D`).

## Test Execution

### Pre-Requisites for the Open-Xchange Groupware Server

#### Disabling rate limitation
* Modify the parameter maxRateLenientClients

```bash
$ vim /opt/etc/open-xchange/etc/server.properties
```

* Specify a new User-Agent which is excluded from rate limit checks "GATLING*"

#### Resources/Data
In the data folder you can find an sample csv file which is used to perform a login. If you want to use your own CSV-File place it into the `data` directory.

* Structure of the csv file:

```csv
  username,password
  user1,secret
  user2,secret
  user3,secret
  ...
  user998,secret
  user999,secret
  user1000,secret
```

## Practical hints and tips
* If you want to run tests for more than 24h, uncomment the "timeOut" → "simulation" property at `gatling.conf` and define a value of more than 24h (in seconds).
* If you experience OutOfMemory issues when generating charts, adjust "charting" → "accuracy" to a higher value. This will lower the accuracy of the generated graphs. Especially for long-running tests, this could be set to 100ms.
* For debugging purposes it's sometimes useful to log all `HTTP` requests and responses or log to a specific file instead of the console output and so on and so forth. See comments in the `logback.xml` in the resources directory ``` project-${PROJECT_NAME}/src/test/resources ``. Here's the default `logback.xml`:

```xml
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <appender name="CONSOLE" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <pattern>%d{HH:mm:ss.SSS} [%-5level] %logger{15} - %msg%n%rEx</pattern>
            <immediateFlush>false</immediateFlush>
        </encoder>
    </appender>
    <appender class="ch.qos.logback.core.rolling.RollingFileAppender" name="FILE">
        <file>/var/log/gatling/gatling.log</file>
        <rollingPolicy class="ch.qos.logback.core.rolling.FixedWindowRollingPolicy">
            <fileNamePattern>/var/log/gatling/gatling.log.%i</fileNamePattern>
            <minIndex>1</minIndex>
            <maxIndex>15</maxIndex>
        </rollingPolicy>

        <triggeringPolicy class="ch.qos.logback.core.rolling.SizeBasedTriggeringPolicy">
            <maxFileSize>2MB</maxFileSize>
        </triggeringPolicy>

        <encoder>
            <pattern>%d{HH:mm:ss.SSS} [%-5level] %logger{15} - %msg%n%rEx</pattern>
        </encoder>
    </appender>

    <!-- Uncomment for logging ALL HTTP request and responses -->
    <!-- <logger name="io.gatling.http" level="TRACE"/> -->
    <!-- Uncomment for logging ONLY FAILED HTTP request and responses -->
    <!-- <logger name="io.gatling.http" level="DEBUG" /> -->

    <!-- OX-Logger -->
    <logger name="com.openexchange.test.performance.gatling" level="ERROR"/>

    <root level="WARN">
        <appender-ref ref="CONSOLE"/>
        <!--<appender-ref ref="FILE"/>-->
    </root>
</configuration>
```
