# Monitoring

[TOC]

## InfluxDB
[InfluxDB](https://influxdata.com/time-series-platform/influxdb/) is an open-source time series database written in `Go` which focuses on storing data for a series and holding a given value for a point in time.

### Getting started with InfluxDB

#### Installation and configuration on Linux
This downloads and installs InfluxDB into `/etc/influxdb`:

```bash
wget https://s3.amazonaws.com/influxdb/influxdb_0.11.0-1_amd64.deb
dpkg -i influxdb_0.11.0-1_amd64.deb
```

Edit the configuration file `/etc/influxdb/influxdb.conf` and enable the graphite listener:

```bash
[[graphite]]
  enabled = true
  database = "metrics"
  bind-address = ":2003"
  protocol = "tcp"
  consistency-level = "one"
  name-separator = "_"

  ## parsing the metric name from graphite protocol;
  templates = [
     "gatling.* .simulation.request.module.measurement*"
 ]
```

Start InfluxDB:

```bash
$ service influxdb start
```
The admin web interface is available at `http://${INFLUXDB_HOSTNAME}:8083` and the `HTTP-API` on port `8086`. 

## Grafana
[Grafana](http://grafana.org) is a Graphite and InfluxDB Dashboard and Graph Editor.

### Getting started with Grafana

#### Installation and configuration on Linux
Download and install the latest stable release:

```bash
$ wget https://grafanarel.s3.amazonaws.com/builds/grafana_${VERSION}_amd64.deb

$ dpkg -i grafana_${VERSION}_amd64.deb
```

Edit the configuration file at `/etc/grafana/grafana.ini`. This file contains global settings for the Grafana installation.

Start Grafana-Server:

```bash
$ service grafana-server start
```

Now point your browser to `http://${GRAFANA_HOSTNAME}:3000` we should see the default Grafana login page. Finally follow the steps to add an [influxdb data source.](http://docs.grafana.org/datasources/influxdb/).

## Telegraf
[Telegraf](https://influxdata.com/time-series-platform/telegraf/) is an open source agent written in `Go` for collecting metrics and data on the system it's running on or from other services.

### Getting started with Telegraf

#### Installation and configuration on Linux
This downloads and installs Telegraf into `/etc/telegraf`:

```bash
wget http://get.influxdb.org/telegraf/telegraf_0.11.1-1_amd64.deb
dpkg -i telegraf_0.11.1-1_amd64.deb
```

Edit the configuration file `/etc/telegraf/telegraf.conf` and specify your inputs (where the metrics come from) and outputs (where the metrics go).

Define the InfluxDB HTTP-Endpoint and the database name at the `OUTPUTS` section:

```bash
###############################################################################
#                                  OUTPUTS                                    #
###############################################################################

# Configuration for influxdb server to send metrics to
[[outputs.influxdb]]
  urls = ["http://${INFLUXDB_HOSTNAME}:8086"] # required
  ## The target database for metrics (telegraf will create it if not exists).
  database = "telegraf" # required
```

Enable all your desired inputs at the `INPUTS` section:

```bash
###############################################################################
#                                  INPUTS                                     #
###############################################################################

# Get kernel statistics from /proc/stat
[[inputs.kernel]]

# Read metrics about memory usage
[[inputs.mem]]

# Get the number of processes and group them by status
[[inputs.processes]]

# Read metrics about swap memory usage
[[inputs.swap]]

# Read metrics about system load & uptime
[[inputs.system]]
```

Now you can start the telegraf service:

```bash
$ service telegraf start
```

## Jmxtrans

### Getting started with jmxtrans

#### Installation and configuration on Linux
Download the latest stable version of [jmxtrans](http://central.maven.org/maven2/org/jmxtrans/jmxtrans/).

```bash
wget http://central.maven.org/maven2/org/jmxtrans/jmxtrans/${VERSION}/jmxtrans-${VERSION}.deb
dpkg -i jmxtrans-${VERSION}.deb
```

== Notes ==
* The application is installed in: `/usr/share/jmxtrans`
* All configuration settings: `/etc/default/jmxtrans`
* The init script: `/etc/init.d/jmxtrans`
* All .json files: `/var/lib/jmxtrans`

Once you have jmxtrans installed, you also need to make sure to enable remote access to [JMX](https://en.wikipedia.org/wiki/Java_Management_Extensions) on the [JVM](https://en.wikipedia.org/wiki/Java_virtual_machine)s that you want to monitor. You can do this e.g. on each OX-Node by adding the following parameters at the end of the `JAVA_XTRAOPTS` in the `/opt/open-xchange/etc/ox-scriptconf.sh` file:

```bash
-Dcom.sun.management.jmxremote.port=9010 -Dcom.sun.management.jmxremote.authenticate=false -Dcom.sun.management.jmxremote.ssl=false -Djava.rmi.server.hostname=${IP_ADDRESS}
```

Restart the Open-Xchange service on all OX-Nodes:

``` bash
$ service open-xchange restart
```


Here is an example for a `JSON` configuration file which queries two OX-Nodes about their `Heap` and `NonHeap` attributes and sends them by using the `InfluxDbWriterFactory` output writer directly (in the correct format with tags, fields, ...) to InfluxDB:

```json
{
   "servers":[
      {
         "port":"9010",
         "host":"${HOSTNAME/IP_OX_NODE_1}",
         "alias":"OX1",
         "queries":[
            {
               "outputWriters":[
                  {
                     "@class":"com.googlecode.jmxtrans.model.output.InfluxDbWriterFactory",
                     "username":"foo",
                     "password":"bar",
                     "database":"metrics",
                     "url":"http://${INFLUXDB_HOSTNAME}:8086",
                     "resultTags":[
                        "attributeName",
                        "typeName"
                     ],
                     "retentionPolicy":"default",
                     "writeConsistency":"ALL"
                  }
               ],
               "obj":"java.lang:type=Memory",
               "resultAlias":"jvm_heap",
               "attr":[
                  "HeapMemoryUsage",
                  "NonHeapMemoryUsage"
               ]
            }
         ],
         "numQueryThreads":2
      },
      {
         "port":"9010",
         "host":"${HOSTNAME/IP_OX_NODE_2}",
         "alias":"OX2",
         "queries":[
            {
               "outputWriters":[
                  {
                     "@class":"com.googlecode.jmxtrans.model.output.InfluxDbWriterFactory",
                     "username":"foo",
                     "password":"bar",
                     "database":"metrics",
                     "url":"http://${INFLUXDB_HOSTNAME}:8086",
                     "resultTags":[
                        "attributeName",
                        "typeName"
                     ],
                     "retentionPolicy":"default",
                     "writeConsistency":"ALL"
                  }
               ],
               "obj":"java.lang:type=Memory",
               "resultAlias":"jvm_heap",
               "attr":[
                  "HeapMemoryUsage",
                  "NonHeapMemoryUsage"
               ]
            }
         ],
         "numQueryThreads":2
      }
   ]
}
```

After placing the .json configuration file in the `/var/lib/jmxtrans` directory and enabling JMX on the OX-Nodes you can start the jmxtrans service to gather all the JVM informations you want.

``` bash
$ service jmxtrans start
```

## Metrics Data Flow Diagram

The diagram describes the data flow where the metrics ==*travel*== from different applications, get stored into InfluxDB and finally visualized in Grafana.

```mermaid
graph TB;
	subgraph Host A
      a1(Telegraf)
      a2(OX)
    end

    subgraph Host B
      b1(Telegraf)
      b2(OX)
    end

    subgraph Host C
      c1(Gatling)
    end

    subgraph Monitoring Server
      d1(Grafana)
      e1(InfluxDB)
      f1(Jmxtrans)
    end

    a1-- :8086 -->e1
    a2-- :9010 -->f1
    b1-- :8086 -->e1
    b2-. :9010 .->f1
    c1-- :2003 -->e1
    f1-->e1
    d1-- :8086 -->e1
```