# UI Metrics stopwatch tests
This tool lets you run the ui metrics tests which are implemented in the App Suite UI

## Requirements
To use this test automation, a couple of prerequisites are needed:


* A computer running Linux, macOS or Windows (with installed Firefox, Chrome, IE or Safari browsers)
* The backend needs to be configured (see Configuration)
* Several files and folders needs to be created before executing the tests
* Java 8 JDK or later needs to be installed (https://java.sun.com/)
* Maven needs to be installed (https://maven.apache.org/)
* Git needs to be installed (https://git-scm.com/)


## Getting started
At first make sure that you have the latest copy of the framework (`git pull`) and that the selected branch (`git branch`, `git checkout <branch>`) .

In order to execute these tests you have to build them first. Compiling and building packages is done using Maven. The easiest way to do that is to just invoke the following command within the repository folder on the command-line:

    $ mvn -B -U clean package

This command will clean up everything inside the `target` folder, resolve dependencies and create all needed packages. After successful execution you'll find 2 `.jar` files within the `target` folder.
Anyway we will just need the following one:

* `uimetrics-1.0-SNAPSHOT-jar-with-dependencies.jar` which is the main executable and contains all necessarry dependencies and logic in order to make tests work.



Note that the `target` folder is managed locally, whenever running Maven commands it might get cleared or updated. This also means that no changes are applied to the Git repository when compiling or just executing tests.

## Configuration
While executing this tests will use some files and mails that need to be created prior to the first execution. Open a browser window and sign in to App Suite with the user that should be used for testing.
Following is a list of files and folders that are required:
* Mail:  `inbox/test` - a subfolder in the users mail inbox which contains at least one mail with image attachments
* Mail:  `inbox/test/MessageWithThumnails.eml` - Mail located in the test folder with at least one image as attachment
* Drive: `folder/image.jpg` - An image (approx. 2MB) located in drive used for sending a new mail
* Drive: `results` - a drive folder in which the test report will be written

After creating those requirements you have to obtain the folder ids and file ids. Use the UI to get them.

Now the UI itself needs to know this ids. You can either provide them by executing a few lines of js-code or by providing a properties file on the backend.

The following settings are available for the ui metrics tests:
* `inbox` Your inbox folder. Default: 'default0/INBOX'
* `test-folder` Another mail test folder (not Inbox) that is not empty. Default: 'default0/INBOX/Test'
* `subject` A test subject. Default: 'Automatic performance test'
* `first-letters` The first letters of the recipient used by auto-complete. Default 'bigge'. Should match the test recipient, of course. First search result is used.
* `recipient` The recipient who receives the test mails. Default: 'matthias.biggeleben@open-xchange.com'
* `keyword` The keyword to search for in INBOX. At least one message subject must contain that keyword.
* `cloud-attachment` An image in OX Drive (size approx. 2MB). folder_id and id. See below.
* `message-with-thumbnails` A mail with thumbnails (also approx. 2MB). folder_id and id. See below.
* `store-folder` The Drive folder to store the results. folder_id.

Just replace the ids in the following examples with your ids

### Configuring by backend properties file
Connect to your Open-Xchange App Suite backend and place a file with the following content in your `/opt/open-xchange/etc/settings/` directory.
In our example: `/opt/open-xchange/etc/settings/ui-metrics.properties`

```
io.ox/core/metrics//demo/inbox=default0/INBOX
io.ox/core/metrics//demo/test-folder=default0/INBOX/Test
io.ox/core/metrics//demo/subject="Automatic performance test"
io.ox/core/metrics//demo/first-letters=userna
io.ox/core/metrics//demo/recipient="username@mail-domain.invalid"
io.ox/core/metrics//demo/keyword=performance
io.ox/core/metrics//demo/cloud-attachment/folder_id=58
io.ox/core/metrics//demo/cloud-attachment/id=1
io.ox/core/metrics//demo/message-with-thumbnails/folder_id=default0/INBOX/Test
io.ox/core/metrics//demo/message-with-thumbnails/id=1
io.ox/core/metrics//demo/store-folder=142  
```
After creating this file restart the backend with `service open-xchange restart`
 
 
### Configuring by JS code
Open the browsers developer tools console, paste the following snippet and hit return. Make sure to set your proper configuration / ids in this script. 

```javascript
require(['settings!io.ox/core/metrics'], function (settings) {
    settings.set('demo', {
        'inbox': 'default0/INBOX',
        'test-folder': 'default0/INBOX/Test',
        'subject': 'Automatic performance test',
        'first-letters': 'userna',
        'recipient': 'username@mail-domain.invalid',
        'keyword': 'performance',
        'cloud-attachment': { folder_id: '58', id: '1' },
        'message-with-thumbnails': { folder_id: 'default0/INBOX/Test', id: 1 },
        'store-folder': 142
    })
    .save();
});
```


## Executing tests
Switch to the `target` folder and execute:

    $ java -jar uimetrics-1.0-SNAPSHOT-jar-with-dependencies.jar

The mandatory options for test execution are:
usage: java -jar uimetrics-1.0-SNAPSHOT-jar-with-dependencies.jar [-B <arg>] -C | -F | -I | -S [-c <arg>]   -p <arg>  -U <arg> -u <arg>
- `-B`,`--binary <arg>`     path to the browser binary (e.g. "C:\Programme\Mozilla Firefox\firefox.exe")
- `-C`,`--chrome`           Use Chrome
- `-c`,`--context <arg>`    OX App Suite user context
- `-F`,`--firefox`          Use Firefox
- `-I`,`--iexplorer`        Use the Internet Explorer (only on Windows)
- `-p`,`--password <arg>`   OX App Suite user password
- `-S`,`--safari`           Use Safari (only on Mac OSX)
- `-U`,`--url <arg>`        URL to the OX-Server, e.g. https;//example.org
- `-u`,`--user <arg>`       OX App Suite username

By specifying those command-line options the following command will for example execute the tests by using the installed chrome browser:

    $ java -jar uimetrics-1.0-SNAPSHOT-jar-with-dependencies.jar -C -c 10 -u username -p secret -U http://ox-host -B "C:\Program Files (x86)\Google\Chrome\Application\chrome.exe"

After a test run has completed, results are provided as a report file located in your specified store folder
