/*
 * Copyright (C) 2016 OX Software GmbH
 *
 * This file is part of uimetrics.
 *
 * uimetrics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * uimetrics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with uimetrics.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.openexchange.test.performance.uimetrics.launcher;

import java.io.IOException;
import java.util.UUID;

import com.openexchange.test.performance.uimetrics.config.Configuration;
import com.openexchange.testing.httpclient.invoker.ApiClient;
import com.openexchange.testing.httpclient.invoker.ApiException;
import com.openexchange.testing.httpclient.models.TokenLoginResponse;
import com.openexchange.testing.httpclient.modules.LoginApi;
import org.apache.commons.lang3.SystemUtils;

/**
 * All stuff around the browser
 *
 * @author <a href="mailto:markus.wagner@open-xchange.com">Markus Wagner</a>
 */
public class BrowserLauncher {

    private static Configuration CONFIGURATION;

    public BrowserLauncher() {
        super();
        CONFIGURATION = Configuration.getInstance();
    }

    public void startBrowser() throws IOException, ApiException {
        final StringBuilder cmd = new StringBuilder();



        if (CONFIGURATION.getAsBoolean(Configuration.Property.USE_IE)) {
            if (SystemUtils.IS_OS_WINDOWS) {
                cmd.append(CONFIGURATION.get(Configuration.Property.IE_BINARY))
                    .append(" -private")
                    .append(" ")
                    .append(buildURL());
                System.out.println(cmd.toString());
                Runtime.getRuntime().exec(cmd.toString());
            } else {
                throw new UnsupportedOperationException("IE is only supported on Windows!");
            }
        } else if (CONFIGURATION.getAsBoolean(Configuration.Property.USE_FIREFOX)) {
            cmd.append(CONFIGURATION.get(Configuration.Property.FIREFOX_BINARY))
                .append(" -private-window")
                .append(" ")
                .append(buildURL());
            System.out.println(cmd.toString());
            Runtime.getRuntime().exec(cmd.toString());
        } else if (CONFIGURATION.getAsBoolean(Configuration.Property.USE_CHROME)) {
            cmd.append(CONFIGURATION.get(Configuration.Property.CHROME_BINARY))
                .append(" --incognito")
                .append(" ")
                .append(buildURL());
            System.out.println(cmd.toString());
            Runtime.getRuntime().exec(cmd.toString());
        } else if (CONFIGURATION.getAsBoolean(Configuration.Property.USE_SAFARAI)) {
            if (SystemUtils.IS_OS_MAC_OSX) {
                Runtime.getRuntime().exec(new String[] { "osascript", "-e", startSafariCmd(buildURL()) });
            } else {
                throw new UnsupportedOperationException("Safari is only supported on Mac OSX!");
            }
        }
    }

    /**
     * constructs the URL with the values of the {@link Configuration}
     *
     * @return the complete URL to the OX formlogin as {@link String}
     */
    public String buildURL() throws ApiException {
        final StringBuilder url = new StringBuilder();

        url.append(getTokenLoginUrl());

//        url
//            .append(CONFIGURATION.getOXHost())
//            .append("ajax/login?action=formlogin")
//            .append("&login=").append(CONFIGURATION.get(Configuration.Property.USER));
//        if (null != CONFIGURATION.get(Configuration.Property.CONTEXT) && 0 != CONFIGURATION.get(Configuration.Property.CONTEXT).length()) {
//            url
//                .append("@")
//                .append(CONFIGURATION.get(Configuration.Property.CONTEXT));
//        }
//        url
//            .append("&password=").append(CONFIGURATION.get(Configuration.Property.PASSWORD))
//            .append("&authId=metrics-").append(UUID.randomUUID())
//            .append("&client=open-xchange-appsuite-metrics")
//            .append("&version=1.0.0")
//            .append("&autologin=true")
//            .append("&uiWebPath=/appsuite/%23cap=metrics-demo");
        return url.toString();
    }

    public String getTokenLoginUrl() throws ApiException {
        String authId = UUID.randomUUID().toString();
        String clientToken = UUID.randomUUID().toString();
        String client = "open-xchange-appsuite-metrics";
        String version = "1";
        String clientIp = null;

        final String user = CONFIGURATION.get(Configuration.Property.USER);
        final String context = CONFIGURATION.get(Configuration.Property.CONTEXT);
        final String password = CONFIGURATION.get(Configuration.Property.PASSWORD);
        final String sAppsuiteUrl =  CONFIGURATION.getOXHost();

        final ApiClient apiClient = new ApiClient();
        apiClient.setBasePath(sAppsuiteUrl + "/ajax");

        LoginApi loginApi = new LoginApi(apiClient);
        TokenLoginResponse response = loginApi.doTokenLogin(user, password, clientToken, authId, client, version, true, null, clientIp, null, true);

        String tokenUrlPart = response.getUrl() + "&clientToken=" + clientToken + "&cap=metrics-demo";



        return sAppsuiteUrl + tokenUrlPart;
    }

    /**
     * As safari has no command line switch for private browsing, we have to use applescript
     *
     * @param url The URL which Safari should open
     *
     * @return the applescript as String
     */
    public String startSafariCmd(final String url) {
        final StringBuilder cmd = new StringBuilder();
        cmd.append("tell application \"Safari\" to activate\n")
            .append("tell application \"Safari\"\n")
            .append("  close window 1\n")
            .append("end tell\n")
            .append("")
            .append("tell application \"System Events\"\n")
            .append("  tell process \"Safari\"\n")
            .append("    tell menu bar 1\n")
            .append("      tell menu bar item \"File\"\n")
            .append("        tell menu \"File\"\n")
            .append("          click menu item \"New Private Window\"\n")
            .append("        end tell\n")
            .append("      end tell\n")
            .append("    end tell\n")
            .append("  end tell\n")
            .append("end tell\n")
            .append("")
            .append("tell application \"Safari\" to set the URL of the front document to \"")
            .append(url)
            .append("\"\n");
        return cmd.toString();
    }
}
