package com.openexchange.build.packaging

import java.io.File
import java.util.List
import java.util.Map

/**
 * Parser/Writer for debian control files according to
 * https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-debiansourcecontrolfiles.
 *
 * Validity of field names etc. is not checked.
 *
 * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
 */
class ControlFile {

    List<Paragraph> paragraphs = []
    Paragraph paragraph
    Field field

    ControlFile() {}

    static ControlFile parse(File file) {
        def controlFile = new ControlFile()
        controlFile.parse0(file)
        return controlFile
    }

    private void parse0(File file) {
        def newParagraph = true
        for (line in file.readLines()) {
            if (line.isAllWhitespace()) {
                newParagraph = true
            } else if (line.startsWith("#")) {
                // ignore comment
            } else if (line.startsWith(" ") || line.startsWith("\t")) {
                // continuation
                assert field != null
                field.value.append(line.drop(1))
            } else {
                if (newParagraph) {
                    paragraph = new Paragraph()
                    paragraphs.add(paragraph)
                    newParagraph = false
                }

                def pair = line.split(":", 2)
                assert pair.length == 2
                def name = pair[0]
                def value = pair[1].trim()
                field = new Field(name, value)
                paragraph.set(field)
            }
        }

    }

    void add(Paragraph p) {
        paragraphs.add(p)
    }

    Paragraph getFirstForField(String field) {
        for (paragraph in paragraphs) {
            if (paragraph.get(field) != null) {
                return paragraph
            }
        }
        return null
    }

    List<Paragraph> getForField(String field) {
        List<Paragraph> result = []
        for (paragraph in paragraphs) {
            if (paragraph.get(field) != null) {
                result.add(paragraph)
            }
        }
        return result
    }

    @Override
    String toString() {
        def str = new StringBuilder()
        def first = true
        for (paragraph in paragraphs) {
            if (first) {
                first = false
            } else {
                str.append("\n")
            }

            str.append(paragraph.toString())
        }
        return str.toString()
    }
}
