package com.openexchange.build.packaging

import java.security.MessageDigest
import org.apache.commons.codec.binary.Hex
import org.gradle.api.DefaultTask
import org.gradle.api.tasks.InputFiles
import org.gradle.api.tasks.OutputFile
import org.gradle.api.tasks.TaskAction

class DebianDSC extends DefaultTask {

    @InputFiles
    File[] archives = new File[0];
    // TODO can we define this distDir or at least its default somewhere globally in the packaging plugin?
    File distDir = new File(project.buildDir, 'dist')
    // TODO global versioning
    @OutputFile
    File outputFile = new File(distDir, project.name + '_' + '7.10.0-0' + '.dsc')

    @TaskAction
    def createDSC() {
        def format = project.file('debian/source/format').getText('UTF-8').trim()
        def controlFile = ControlFile.parse(project.file('debian/control'))
        def sourcePackage = controlFile.getFirstForField('Source')
        def binaryPackages = controlFile.getForField('Package')

        // Build DSC file
        def dscFile = new ControlFile()
        def dscParagraph = new Paragraph()
        dscParagraph.set(new Field('Format', format))
        dscParagraph.set(sourcePackage.get('Source'))
        def binary = new StringBuilder()
        def pkgList = new Field('Package-List')
        for (p in binaryPackages) {
            def pkgName = p.get('Package').getSimpleValue()
            binary.append(pkgName).append(", ")
            pkgList.value.append(pkgName + ' deb ' + sourcePackage.get('Section').getSimpleValue() + ' ' +
                    sourcePackage.get('Priority').getSimpleValue() + ' arch=' + p.get('Architecture').getSimpleValue())
        }
        dscParagraph.set(new Field('Binary', binary.take(binary.length() - 2)))
        dscParagraph.set(new Field('Architecture', 'all'))
        // TODO global versioning
        dscParagraph.set(new Field('Version', '7.10.0-0'))
        dscParagraph.set(sourcePackage.get('Maintainer'))
        dscParagraph.set(sourcePackage.get('Homepage'))
        dscParagraph.set(sourcePackage.get('Standards-Version'))
        dscParagraph.set(sourcePackage.get('Build-Depends'))
        dscParagraph.set(pkgList)

        // Calculate checksums and sizes
        def md5 = MessageDigest.getInstance("MD5")
        def sha1 = MessageDigest.getInstance("SHA1")
        def sha256 = MessageDigest.getInstance("SHA-256")

        def checksumsSha1 = new Field('Checksums-Sha1')
        def checksumsSha256 = new Field('Checksums-Sha256')
        def files = new Field('Files')
        archives.each { file ->
            file.withInputStream { stream ->
                def len = 0
                def totalLen = 0
                def buf = new byte[1024]
                while (len >= 0) {
                    len = stream.read(buf, 0, buf.length)
                    if (len > 0) {
                        md5.update(buf, 0, len)
                        sha1.update(buf, 0, len)
                        sha256.update(buf, 0, len)
                        totalLen += len
                    }
                }

                def tail = ' ' + Long.toString(totalLen) + ' ' + file.getName()
                files.getValue().append(Hex.encodeHexString(md5.digest()) + tail)
                checksumsSha1.getValue().append(Hex.encodeHexString(sha1.digest()) + tail)
                checksumsSha256.getValue().append(Hex.encodeHexString(sha256.digest()) + tail)

                md5.reset()
                sha1.reset()
                sha256.reset()
            }
        }

        dscParagraph.set(checksumsSha1)
        dscParagraph.set(checksumsSha256)
        dscParagraph.set(files)

        dscFile.add(dscParagraph)
        outputFile.write(dscFile.toString())
    }
}
