package com.openexchange.build.packaging

import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.Task
import org.gradle.api.tasks.Copy
import org.gradle.api.tasks.bundling.Compression
import org.gradle.api.tasks.bundling.Tar
import com.openexchange.obs.api.BuildServiceClient
import com.openexchange.projectset.data.ProjectSetFileReader

// build/distributions
// tasks: upload, wait, fetch
class PackagingPlugin implements Plugin<Project> {

    void apply(Project project) {
        ProjectSetFileReader psf = new ProjectSetFileReader(new File(project.projectDir, project.name + '.psf'));
        configureTasks(project, psf)
    }

    def configureTasks(Project project, ProjectSetFileReader psf) {
        System.out.println('project: ' + project)
        File distDir = new File(project.buildDir, 'dist')
        Tar origTar = project.tasks.create('dist', Tar)
        psf.getProjectList().each { psfProject ->
            // TODO somehow we should create this dependency smarter
            origTar.dependsOn(':' + psfProject + ':install')
        }
        // TODO define compression dynamically to the best, that is allowed by the oldest Debian distribution defined on the root project
        origTar.compression = Compression.BZIP2
        origTar.baseName = project.name
        // TODO we need a global versioning scheme that is used here
        origTar.version = '7.10.0'
        origTar.classifier = 'orig'
        origTar.extension = 'tar.bz2'
        // TODO overwrite org.gradle.api.tasks.bundling.AbstractArchiveTask.getArchiveName()
        origTar.archiveName = origTar.baseName + '_' + origTar.version + '.' + origTar.classifier + '.' + origTar.extension
        origTar.destinationDir = distDir
        // TODO can we define some Gradle interface plugin that defines above install target and this install path but not their implementation?
        System.out.println('childProjects: ' + project.rootProject.childProjects)
        System.out.println('projectList: ' + psf.projectList)
        origTar.from(psf.projectList.collect { new File(project.rootProject.childProjects.get(it).buildDir, 'install') })
//        origTar.eachFile { details ->
//            def targetPath = project.name + '-7.10.0/' +  details.path
//            details.path = targetPath
//        }
//        origTar.includeEmptyDirs = false
        origTar.into(project.name + '-7.10.0')

        Tar debianTar = project.tasks.create('debianTar', Tar)
        // TODO define compression dynamically to the best, that is allowed by the oldest Debian distribution defined on the root project
        debianTar.compression = Compression.BZIP2
        debianTar.baseName = project.name
        // TODO global versioning scheme
        debianTar.version = '7.10.0-0'
        debianTar.classifier = 'debian'
        debianTar.extension = 'tar.bz2'
        // TODO overwrite org.gradle.api.tasks.bundling.AbstractArchiveTask.getArchiveName()
        debianTar.archiveName = debianTar.baseName + '_' + debianTar.version + '.' + debianTar.classifier + '.' + debianTar.extension
        debianTar.destinationDir = distDir
        debianTar.from(project.projectDir)
        debianTar.include('debian/**')
        // TODO common rules file just for copying

        DebianDSC debianDSC = project.tasks.create('debianDSC', DebianDSC) {
            archives = [origTar.archivePath, debianTar.archivePath]
            dependsOn(origTar, debianTar)
        }

        Task debianSource = project.task('debianSource')
        debianSource.dependsOn(debianDSC)

        Copy copySpec = project.tasks.create('copySpec', Copy) {
            from project.projectDir
            include project.name + '.spec'
            into distDir
        }

        Task upload = project.tasks.create('upload') {
            doLast {
                // TODO define URL, user and password at some proper place
                def obs = new BuildServiceClient('https://buildapi.open-xchange.com', 'oxbuilduser', 'openxchange')
                // TODO allow only the 3 valid combinations (debian and/or rpm)
                File[] files = [ origTar.archivePath, debianTar.archivePath, debianDSC.outputFile, new File(distDir, project.name + '.spec') ].findAll { it.exists() }
                // TODO determine the project name dynamically
                obs.uploadSourcePackage('performance-develop', project.name, files, new String[0])
            }
            dependsOn(debianSource, copySpec, ':createProject')
        }
    }
}
